@@ -1,8 +1,8 @@
 /*
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2009 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2009 Nokia Corporation
+ * Copyright (C) 2009-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * Copyright (C) 2009-2010 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,8 +21,11 @@
 
 #include <TelepathyQt4/AbstractClient>
 
+#include <QSharedData>
 #include <QString>
 
+#include <TelepathyQt4/ChannelClassSpecList>
+
 namespace Tp
 {
 
@@ -147,7 +150,7 @@ struct TELEPATHY_QT4_NO_EXPORT AbstractClientObserver::Private
  * class MyObserver : public AbstractClientObserver
  * {
  * public:
- *     MyObserver(const ChannelClassList &channelFilter);
+ *     MyObserver(const ChannelClassSpecList &channelFilter);
  *     ~MyObserver() { }
  *
  *     void observeChannels(const MethodInvocationContextPtr<> &context,
@@ -156,10 +159,10 @@ struct TELEPATHY_QT4_NO_EXPORT AbstractClientObserver::Private
  *             const QList<ChannelPtr> &channels,
  *             const ChannelDispatchOperationPtr &dispatchOperation,
  *             const QList<ChannelRequestPtr> &requestsSatisfied,
- *             const QVariantMap &observerInfo);
+ *             const AbstractClientObserver::ObserverInfo &observerInfo);
  * };
  *
- * MyObserver::MyObserver(const ChannelClassList &channelFilter)
+ * MyObserver::MyObserver(const ChannelClassSpecList &channelFilter)
  *     : AbstractClientObserver(channelFilter)
  * {
  * }
@@ -170,7 +173,7 @@ struct TELEPATHY_QT4_NO_EXPORT AbstractClientObserver::Private
  *         const QList<ChannelPtr> &channels,
  *         const ChannelDispatchOperationPtr &dispatchOperation,
  *         const QList<ChannelRequestPtr> &requestsSatisfied,
- *         const QVariantMap &observerInfo)
+ *         const AbstractClientObserver::ObserverInfo &observerInfo)
  * {
  *     // do something, log messages, ...
  *
@@ -183,16 +186,10 @@ struct TELEPATHY_QT4_NO_EXPORT AbstractClientObserver::Private
  *
  * \code
  *
- * ChannelClassList filters;
- * QMap<QString, QDBusVariant> filter;
- * filter.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".ChannelType"),
- *               QDBusVariant(QLatin1String(TELEPATHY_INTERFACE_CHANNEL_TYPE_TEXT)));
- * filter.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandleType"),
- *               QDBusVariant((uint) Tp::HandleTypeContact));
- * filters.append(filter);
  * ClientRegistrar registrar = ClientRegistrar::create();
  * AbstractClientPtr observer = AbstractClientPtr::dynamicCast(
- *         SharedPtr<MyObserver>(new MyObserver(filter)));
+ *         SharedPtr<MyObserver>(new MyObserver(
+ *             ChannelClassSpecList() << ChannelClassSpec::textChat())));
  * registrar->registerClient(observer, "myobserver");
  *
  * \endcode
@@ -200,20 +197,42 @@ struct TELEPATHY_QT4_NO_EXPORT AbstractClientObserver::Private
  * \sa AbstractClient
  */
 
-/**
- * Construct a new AbstractClientObserver object.
- *
- * Note that using this constructor the shouldRecover() method will return
- * \c false, meaning that on crash the observer won't be able to "catch up"
- * on channels that match its observerChannelFilter() automatically.
- *
- * \param channelFilter A specification of the channels in which this observer
- *                      is interested.
- */
-AbstractClientObserver::AbstractClientObserver(
-        const ChannelClassList &channelFilter)
-    : mPriv(new Private(channelFilter, false))
+struct AbstractClientObserver::ObserverInfo::Private : public QSharedData
 {
+    Private(const QVariantMap &info)
+        : info(info) {}
+
+    QVariantMap info;
+};
+
+AbstractClientObserver::ObserverInfo::ObserverInfo(const QVariantMap &info)
+    : mPriv(new Private(info))
+{
+}
+
+AbstractClientObserver::ObserverInfo::ObserverInfo(const ObserverInfo &other)
+    : mPriv(other.mPriv)
+{
+}
+
+AbstractClientObserver::ObserverInfo::~ObserverInfo()
+{
+}
+
+AbstractClientObserver::ObserverInfo &AbstractClientObserver::ObserverInfo::operator=(
+        const ObserverInfo &other)
+{
+    if (this == &other) {
+        return *this;
+    }
+
+    mPriv = other.mPriv;
+    return *this;
+}
+
+QVariantMap AbstractClientObserver::ObserverInfo::allInfo() const
+{
+    return mPriv->info;
 }
 
 /**
@@ -226,11 +245,14 @@ AbstractClientObserver::AbstractClientObserver(
  *                      existing channel matching its observerChannelFilter().
  */
 AbstractClientObserver::AbstractClientObserver(
-        const ChannelClassList &channelFilter,
+        const ChannelClassSpecList &channelFilter,
         bool shouldRecover)
-    : mPriv(new Private(channelFilter, shouldRecover))
+    : mPriv(new Private(channelFilter.bareClasses(), shouldRecover))
+      // The channel filter is converted here to the low-level class so that any warnings are
+      // emitted immediately rather than only when the CD introspects this Client
 {
 }
+
 /**
  * Class destructor.
  */
@@ -245,9 +267,6 @@ AbstractClientObserver::~AbstractClientObserver()
  * by the channel dispatcher whenever any of the newly created channels match
  * this description.
  *
- * Only certain types have useful semantics for matching like this,
- * so only certain types are allowed:
- *
  * See <a
  * href="http://telepathy.freedesktop.org/spec/org.freedesktop.Telepathy.Client.Observer.html#org.freedesktop.Telepathy.Client.Observer.ObserverChannelFilter">
  * the Telepathy D-Bus API Specification</a> for documentation about the allowed
@@ -266,9 +285,9 @@ AbstractClientObserver::~AbstractClientObserver()
  *         interested.
  * \sa observeChannels()
  */
-ChannelClassList AbstractClientObserver::observerChannelFilter() const
+ChannelClassSpecList AbstractClientObserver::observerFilter() const
 {
-    return mPriv->channelFilter;
+    return ChannelClassSpecList(mPriv->channelFilter);
 }
 
 /**
@@ -293,7 +312,7 @@ bool AbstractClientObserver::shouldRecover() const
  *                  const QList<ChannelPtr> &channels,
  *                  const ChannelDispatchOperationPtr &dispatchOperation,
  *                  const QList<ChannelRequestPtr> &requestsSatisfied,
- *                  const QVariantMap &observerInfo);
+ *                  const ObserverInfo &observerInfo);
  *
  * Called by the channel dispatcher when channels in which the observer has
  * registered an interest are announced.
@@ -334,11 +353,7 @@ bool AbstractClientObserver::shouldRecover() const
  *                          these methods cannot return until the observer has
  *                          returned from observeChannels().
  * \param requestsSatisfied The requests satisfied by these channels.
- * \param observerInfo Additional information about these channels. No keys are
- *                     currently defined.
- *                     If keys are defined for this dictionary, all will be
- *                     optional; observers may safely ignore any entry in this
- *                     dictionary.
+ * \param observerInfo Additional information about these channels.
  */
 
 struct TELEPATHY_QT4_NO_EXPORT AbstractClientApprover::Private
@@ -410,22 +425,20 @@ struct TELEPATHY_QT4_NO_EXPORT AbstractClientApprover::Private
  * class MyApprover : public AbstractClientApprover
  * {
  * public:
- *     MyApprover(const ChannelClassList &channelFilter);
+ *     MyApprover(const ChannelClassSpecSpecList &channelFilter);
  *     ~MyApprover() { }
  *
  *     void addDispatchOperation(const MethodInvocationContextPtr<> &context,
- *             const QList<ChannelPtr> &channels,
  *             const ChannelDispatchOperationPtr &dispatchOperation);
  * };
  *
- * MyApprover::MyApprover(const ChannelClassList &channelFilter)
+ * MyApprover::MyApprover(const ChannelClassSpecList &channelFilter)
  *     : AbstractClientApprover(channelFilter)
  * {
  * }
  *
  * void MyApprover::addDispatchOperation(
  *         const MethodInvocationContextPtr<> &context,
- *         const QList<ChannelPtr> &channels,
  *         const ChannelDispatchOperationPtr &dispatchOperation)
  * {
  *     // do something with dispatchOperation
@@ -439,16 +452,10 @@ struct TELEPATHY_QT4_NO_EXPORT AbstractClientApprover::Private
  *
  * \code
  *
- * ChannelClassList filters;
- * QMap<QString, QDBusVariant> filter;
- * filter.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".ChannelType"),
- *               QDBusVariant(TELEPATHY_INTERFACE_CHANNEL_TYPE_TEXT));
- * filter.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandleType"),
- *               QDBusVariant((uint) Tp::HandleTypeContact));
- * filters.append(filter);
  * ClientRegistrar registrar = ClientRegistrar::create();
  * AbstractClientPtr approver = AbstractClientPtr::dynamicCast(
- *         SharedPtr<MyApprover>(new MyApprover(filter)));
+ *         SharedPtr<MyApprover>(new MyApprover(
+ *             ChannelClassSpecList() << ChannelClassSpec::textChat())));
  * registrar->registerClient(approver, "myapprover");
  *
  * \endcode
@@ -463,10 +470,10 @@ struct TELEPATHY_QT4_NO_EXPORT AbstractClientApprover::Private
  *                      is interested.
  */
 AbstractClientApprover::AbstractClientApprover(
-        const ChannelClassList &channelFilter)
+        const ChannelClassSpecList &channelFilter)
     : mPriv(new Private)
 {
-    mPriv->channelFilter = channelFilter;
+    mPriv->channelFilter = channelFilter.bareClasses();
 }
 
 /**
@@ -496,15 +503,14 @@ AbstractClientApprover::~AbstractClientApprover()
  *         interested.
  * \sa addDispatchOperation()
  */
-ChannelClassList AbstractClientApprover::approverChannelFilter() const
+ChannelClassSpecList AbstractClientApprover::approverFilter() const
 {
-    return mPriv->channelFilter;
+    return ChannelClassSpecList(mPriv->channelFilter);
 }
 
 /**
  * \fn void AbstractClientApprover::addDispatchOperation(
  *                  const MethodInvocationContextPtr<> &context,
- *                  const QList<ChannelPtr> &channels,
  *                  const ChannelDispatchOperationPtr &dispatchOperation);
  *
  * Called by the channel dispatcher when a dispatch operation in which the
@@ -520,28 +526,13 @@ ChannelClassList AbstractClientApprover::approverChannelFilter() const
  *
  * \param context A MethodInvocationContextPtr object that must be used to
  *                indicate whether this method finished processing.
- * \param channels The channels to be dispatched.
- *                 This argument always contains all of the channels in the
- *                 channel dispatch operation, even if not all of them actually
- *                 match the approver filter.
- *                 The actual channels to be dispatched may reduce as channels
- *                 are closed: this is signalled by
- *                 ChannelDispatchOperation::channelLost signal on the received
- *                 \a dispatchOperation object.
- *                 Approvers should connect to
- *                 ChannelDispatchOperation::channelLost() and
- *                 ChannelDispatchOperation::invalidated() signals (if desired)
- *                 before returning from addDispatchOperation, since those
- *                 signals are guaranteed not to be emitted
- *                 until after all addDispatchOperation() calls have returned
- *                 (with success or failure) or timed out.
  * \param dispatchOperation The dispatch operation to be processed.
  */
 
 struct TELEPATHY_QT4_NO_EXPORT AbstractClientHandler::Private
 {
     ChannelClassList channelFilter;
-    QStringList capabilities;
+    Capabilities capabilities;
     bool wantsRequestNotification;
 };
 
@@ -591,7 +582,7 @@ struct TELEPATHY_QT4_NO_EXPORT AbstractClientHandler::Private
  * class MyHandler : public AbstractClientHandler
  * {
  * public:
- *     MyHandler(const ChannelClassList &channelFilter);
+ *     MyHandler(const ChannelClassSpecList &channelFilter);
  *     ~MyHandler() { }
  *
  *     void bypassApproval() const;
@@ -602,10 +593,10 @@ struct TELEPATHY_QT4_NO_EXPORT AbstractClientHandler::Private
  *             const QList<ChannelPtr> &channels,
  *             const QList<ChannelRequestPtr> &requestsSatisfied,
  *             const QDateTime &userActionTime,
- *             const QVariantMap &handlerInfo);
+ *             const AbstractClientHandler::HandlerInfo &handlerInfo);
  * };
  *
- * MyHandler::MyHandler(const ChannelClassList &channelFilter)
+ * MyHandler::MyHandler(const ChannelClassSpecList &channelFilter)
  *     : AbstractClientHandler(channelFilter)
  * {
  * }
@@ -621,7 +612,7 @@ struct TELEPATHY_QT4_NO_EXPORT AbstractClientHandler::Private
  *         const QList<ChannelPtr> &channels,
  *         const QList<ChannelRequestPtr> &requestsSatisfied,
  *         const QDateTime &userActionTime,
- *         const QVariantMap &handlerInfo)
+ *         const AbstractClientHandler::HandlerInfo &handlerInfo)
  * {
  *     // do something
  *
@@ -634,16 +625,10 @@ struct TELEPATHY_QT4_NO_EXPORT AbstractClientHandler::Private
  *
  * \code
  *
- * ChannelClassList filters;
- * QMap<QString, QDBusVariant> filter;
- * filter.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".ChannelType"),
- *               QDBusVariant(TELEPATHY_INTERFACE_CHANNEL_TYPE_TEXT));
- * filter.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandleType"),
- *               QDBusVariant((uint) Tp::HandleTypeContact));
- * filters.append(filter);
  * ClientRegistrar registrar = ClientRegistrar::create();
  * AbstractClientPtr handler = AbstractClientPtr::dynamicCast(
- *         SharedPtr<MyHandler>(new MyHandler(filter)));
+ *         SharedPtr<MyHandler>(new MyHandler(
+ *             ChannelClassSpecList() << ChannelClassSpec::textChat())));
  * registrar->registerClient(handler, "myhandler");
  *
  * \endcode
@@ -651,21 +636,95 @@ struct TELEPATHY_QT4_NO_EXPORT AbstractClientHandler::Private
  * \sa AbstractClient
  */
 
-/**
- * Construct a new AbstractClientHandler object.
- *
- * \param channelFilter A specification of the channels in which this observer
- *                      is interested.
- * \param wantsRequestNotification Whether this handler wants to receive channel
- *                                 requests notification via addRequest() and
- *                                 removeRequest().
- */
-AbstractClientHandler::AbstractClientHandler(const ChannelClassList &channelFilter,
-        bool wantsRequestNotification)
-    : mPriv(new Private)
+struct AbstractClientHandler::Capabilities::Private : public QSharedData
 {
-    mPriv->channelFilter = channelFilter;
-    mPriv->wantsRequestNotification = wantsRequestNotification;
+    Private(const QStringList &tokens)
+        : tokens(QSet<QString>::fromList(tokens)) {}
+
+    QSet<QString> tokens;
+};
+
+AbstractClientHandler::Capabilities::Capabilities(const QStringList &tokens)
+    : mPriv(new Private(tokens))
+{
+}
+
+AbstractClientHandler::Capabilities::Capabilities(const Capabilities &other)
+    : mPriv(other.mPriv)
+{
+}
+
+AbstractClientHandler::Capabilities::~Capabilities()
+{
+}
+
+AbstractClientHandler::Capabilities &AbstractClientHandler::Capabilities::operator=(
+        const Capabilities &other)
+{
+    if (this == &other) {
+        return *this;
+    }
+
+    mPriv = other.mPriv;
+    return *this;
+}
+
+bool AbstractClientHandler::Capabilities::hasToken(const QString &token) const
+{
+    return mPriv->tokens.contains(token);
+}
+
+void AbstractClientHandler::Capabilities::setToken(const QString &token)
+{
+    mPriv->tokens.insert(token);
+}
+
+void AbstractClientHandler::Capabilities::unsetToken(const QString &token)
+{
+    mPriv->tokens.remove(token);
+}
+
+QStringList AbstractClientHandler::Capabilities::allTokens() const
+{
+    return mPriv->tokens.toList();
+}
+
+struct AbstractClientHandler::HandlerInfo::Private : public QSharedData
+{
+    Private(const QVariantMap &info)
+        : info(info) {}
+
+    QVariantMap info;
+};
+
+AbstractClientHandler::HandlerInfo::HandlerInfo(const QVariantMap &info)
+    : mPriv(new Private(info))
+{
+}
+
+AbstractClientHandler::HandlerInfo::HandlerInfo(const HandlerInfo &other)
+    : mPriv(other.mPriv)
+{
+}
+
+AbstractClientHandler::HandlerInfo::~HandlerInfo()
+{
+}
+
+AbstractClientHandler::HandlerInfo &AbstractClientHandler::HandlerInfo::operator=(
+        const HandlerInfo &other)
+{
+    if (this == &other) {
+        return *this;
+    }
+
+    mPriv = other.mPriv;
+    return *this;
+}
+
+QVariantMap AbstractClientHandler::HandlerInfo::allInfo() const
+{
+    return mPriv->info;
 }
 
 /**
@@ -677,15 +736,14 @@ AbstractClientHandler::AbstractClientHandler(const ChannelClassList &channelFilt
  *                                 requests notification via addRequest() and
  *                                 removeRequest().
  * \param capabilities The set of additional capabilities supported by this
- *                     handler. See the capabilities() method documetation for
- *                     more details.
+ *                     handler.
  */
-AbstractClientHandler::AbstractClientHandler(const ChannelClassList &channelFilter,
-        const QStringList &capabilities,
+AbstractClientHandler::AbstractClientHandler(const ChannelClassSpecList &channelFilter,
+        const Capabilities &capabilities,
         bool wantsRequestNotification)
     : mPriv(new Private)
 {
-    mPriv->channelFilter = channelFilter;
+    mPriv->channelFilter = channelFilter.bareClasses();
     mPriv->capabilities = capabilities;
     mPriv->wantsRequestNotification = wantsRequestNotification;
 }
@@ -716,22 +774,17 @@ AbstractClientHandler::~AbstractClientHandler()
  * \return A specification of the channels that this channel handler can deal
  *         with.
  */
-ChannelClassList AbstractClientHandler::handlerChannelFilter() const
+ChannelClassSpecList AbstractClientHandler::handlerFilter() const
 {
-    return mPriv->channelFilter;
+    return ChannelClassSpecList(mPriv->channelFilter);
 }
 
 /**
- * Return the set of additional capabilities supported by this handler. This
- * describes things like support for streamed media codecs and NAT traversal
- * mechanisms.
- *
- * See <a href="http://telepathy.freedesktop.org/spec/org.freedesktop.Telepathy.Client.Handler.html#org.freedesktop.Telepathy.Client.Handler.Capabilities">
- * the Telepathy D-Bus API Specification</a> for more details.
+ * Return the set of additional capabilities supported by this handler.
  *
  * \return The set of additional capabilities supported by this handler.
  */
-QStringList AbstractClientHandler::capabilities() const
+AbstractClientHandler::Capabilities AbstractClientHandler::handlerCapabilities() const
 {
     return mPriv->capabilities;
 }
@@ -753,7 +806,7 @@ QStringList AbstractClientHandler::capabilities() const
  *                  const QList<ChannelPtr> &channels,
  *                  const QList<ChannelRequestPtr> &requestsSatisfied,
  *                  const QDateTime &userActionTime,
- *                  const QVariantMap &handlerInfo);
+ *                  const HandlerInfo &handlerInfo);
  *
  * Called by the channel dispatcher when this handler should handle these
  * channels, or when this handler should present channels that it is already
@@ -794,11 +847,7 @@ QStringList AbstractClientHandler::capabilities() const
  *                       channel is to be handled for some reason not involving
  *                       user action. Handlers should use this for
  *                       focus-stealing prevention, if applicable.
- * \param handlerInfo Additional information about these channels. No keys are
- *                    currently defined.
- *                    If keys are defined for this dictionary, all will be
- *                    optional; handlers may safely ignore any entry in this
- *                    dictionary.
+ * \param handlerInfo Additional information about these channels.
  */
 
 /**
