@@ -29,6 +29,7 @@
 #include <TelepathyQt4/_gen/cli-account.h>
 
 #include <TelepathyQt4/Connection>
+#include <TelepathyQt4/ConnectionCapabilities>
 #include <TelepathyQt4/ConnectionFactory>
 #include <TelepathyQt4/ContactFactory>
 #include <TelepathyQt4/ChannelFactory>
@@ -36,8 +37,9 @@
 #include <TelepathyQt4/DBusProxy>
 #include <TelepathyQt4/FileTransferChannelCreationProperties>
 #include <TelepathyQt4/OptionalInterfaceFactory>
+#include <TelepathyQt4/Presence>
+#include <TelepathyQt4/ProtocolInfo>
 #include <TelepathyQt4/ReadinessHelper>
-#include <TelepathyQt4/ReadyObject>
 #include <TelepathyQt4/Types>
 #include <TelepathyQt4/Constants>
 #include <TelepathyQt4/SharedPtr>
@@ -57,13 +59,9 @@ class PendingConnection;
 class PendingOperation;
 class PendingReady;
 class PendingStringList;
-class ProtocolInfo;
 
 class TELEPATHY_QT4_EXPORT Account : public StatelessDBusProxy,
-                public OptionalInterfaceFactory<Account>,
-                public ReadyObject,
-                public RefCounted
-
+                public OptionalInterfaceFactory<Account>
 {
     Q_OBJECT
     Q_DISABLE_COPY(Account)
@@ -72,29 +70,27 @@ class TELEPATHY_QT4_EXPORT Account : public StatelessDBusProxy,
     Q_PROPERTY(QString cmName READ cmName)
     Q_PROPERTY(QString protocolName READ protocolName)
     Q_PROPERTY(QString serviceName READ serviceName NOTIFY serviceNameChanged)
+    Q_PROPERTY(ProfilePtr profile READ profile NOTIFY profileChanged)
     Q_PROPERTY(QString displayName READ displayName NOTIFY displayNameChanged)
     Q_PROPERTY(QString iconName READ iconName NOTIFY iconNameChanged)
     Q_PROPERTY(QString nickname READ nickname NOTIFY nicknameChanged)
     Q_PROPERTY(Avatar avatar READ avatar NOTIFY avatarChanged)
     Q_PROPERTY(QVariantMap parameters READ parameters NOTIFY parametersChanged)
-    Q_PROPERTY(ProtocolInfo* protocolInfo READ protocolInfo)
-    Q_PROPERTY(ConnectionCapabilities* capabilities READ capabilities NOTIFY capabilitiesChanged)
+    Q_PROPERTY(ProtocolInfo protocolInfo READ protocolInfo)
+    Q_PROPERTY(ConnectionCapabilities capabilities READ capabilities NOTIFY capabilitiesChanged)
     Q_PROPERTY(bool hasBeenOnline READ hasBeenOnline)
     Q_PROPERTY(bool connectsAutomatically READ connectsAutomatically NOTIFY connectsAutomaticallyPropertyChanged)
-    Q_PROPERTY(ConnectionStatus connectionStatus READ connectionStatus)
+    Q_PROPERTY(ConnectionStatus connectionStatus READ connectionStatus NOTIFY connectionStatusChanged)
     Q_PROPERTY(ConnectionStatusReason connectionStatusReason READ connectionStatusReason)
     Q_PROPERTY(QString connectionError READ connectionError)
-    // TODO: in API/ABI break make this be a Connection::ErrorDetails
-    Q_PROPERTY(QVariantMap connectionErrorDetails READ connectionErrorDetails)
-    Q_PROPERTY(bool haveConnection READ haveConnection NOTIFY haveConnectionChanged)
-    Q_PROPERTY(ConnectionPtr connection READ connection)
+    Q_PROPERTY(Tp::Connection::ErrorDetails connectionErrorDetails READ connectionErrorDetails)
+    Q_PROPERTY(ConnectionPtr connection READ connection NOTIFY connectionChanged)
     Q_PROPERTY(bool changingPresence READ isChangingPresence NOTIFY changingPresence)
-    Q_PROPERTY(SimplePresence automaticPresence READ automaticPresence NOTIFY automaticPresenceChanged)
-    Q_PROPERTY(SimplePresence currentPresence READ currentPresence NOTIFY currentPresenceChanged)
-    Q_PROPERTY(SimplePresence requestedPresence READ requestedPresence NOTIFY requestedPresenceChanged)
+    Q_PROPERTY(Presence automaticPresence READ automaticPresence NOTIFY automaticPresenceChanged)
+    Q_PROPERTY(Presence currentPresence READ currentPresence NOTIFY currentPresenceChanged)
+    Q_PROPERTY(Presence requestedPresence READ requestedPresence NOTIFY requestedPresenceChanged)
     Q_PROPERTY(bool online READ isOnline NOTIFY onlinenessChanged)
     Q_PROPERTY(QString uniqueIdentifier READ uniqueIdentifier)
-    Q_PROPERTY(QString connectionObjectPath READ connectionObjectPath)
     Q_PROPERTY(QString normalizedName READ normalizedName NOTIFY normalizedNameChanged)
 
 public:
@@ -102,28 +98,21 @@ public:
     static const Feature FeatureAvatar;
     static const Feature FeatureProtocolInfo;
     static const Feature FeatureCapabilities;
+    static const Feature FeatureProfile;
 
-    static AccountPtr create(const QString &busName,
-            const QString &objectPath);
-    static AccountPtr create(const QDBusConnection &bus,
-            const QString &busName, const QString &objectPath);
-
-    // API/ABI break: collapse these with the above variants and have all factories be default
-    // params for the non-bus version
     static AccountPtr create(const QString &busName, const QString &objectPath,
-            const ConnectionFactoryConstPtr &connectionFactory,
+            const ConnectionFactoryConstPtr &connectionFactory =
+                ConnectionFactory::create(QDBusConnection::sessionBus()),
             const ChannelFactoryConstPtr &channelFactory =
                 ChannelFactory::create(QDBusConnection::sessionBus()),
             const ContactFactoryConstPtr &contactFactory =
                 ContactFactory::create());
-
     static AccountPtr create(const QDBusConnection &bus,
             const QString &busName, const QString &objectPath,
             const ConnectionFactoryConstPtr &connectionFactory,
             const ChannelFactoryConstPtr &channelFactory,
             const ContactFactoryConstPtr &contactFactory =
                 ContactFactory::create());
-
     virtual ~Account();
 
     ConnectionFactoryConstPtr connectionFactory() const;
@@ -137,23 +126,25 @@ public:
 
     QString cmName() const;
 
-    TELEPATHY_QT4_DEPRECATED QString protocol() const;
     QString protocolName() const;
 
     QString serviceName() const;
     PendingOperation *setServiceName(const QString &value);
 
+    ProfilePtr profile() const;
+
     QString displayName() const;
     PendingOperation *setDisplayName(const QString &value);
 
-    TELEPATHY_QT4_DEPRECATED QString icon() const;
     QString iconName() const;
-    TELEPATHY_QT4_DEPRECATED PendingOperation *setIcon(const QString &value);
     PendingOperation *setIconName(const QString &value);
 
     QString nickname() const;
     PendingOperation *setNickname(const QString &value);
 
+    // TODO: We probably want to expose the avatar file name once we have the avatar token and MC
+    //       starts sharing the cache used by tp-qt4 and tp-glib and use Tp::AvatarData to represent
+    //       it as used in Tp::Contact
     const Avatar &avatar() const;
     PendingOperation *setAvatar(const Avatar &avatar);
 
@@ -161,10 +152,9 @@ public:
     PendingStringList *updateParameters(const QVariantMap &set,
             const QStringList &unset);
 
-    // comes from the ConnectionManager
-    ProtocolInfo *protocolInfo() const;
+    ProtocolInfo protocolInfo() const;
 
-    ConnectionCapabilities *capabilities() const;
+    ConnectionCapabilities capabilities() const;
 
     bool connectsAutomatically() const;
     PendingOperation *setConnectsAutomatically(bool value);
@@ -174,29 +164,26 @@ public:
     ConnectionStatus connectionStatus() const;
     ConnectionStatusReason connectionStatusReason() const;
     QString connectionError() const;
-    QVariantMap connectionErrorDetails() const;
-    bool haveConnection() const;
+    Connection::ErrorDetails connectionErrorDetails() const;
     ConnectionPtr connection() const;
 
     bool isChangingPresence() const;
 
-    SimplePresence automaticPresence() const;
-    PendingOperation *setAutomaticPresence(
-            const SimplePresence &value);
+    // TODO: Add overload methods to set presence from a Profile::Presence
+    // TODO: Add usablePresences() that would return a list of presences that could be set on the
+    //       account
+    Presence automaticPresence() const;
+    PendingOperation *setAutomaticPresence(const Presence &presence);
 
-    SimplePresence currentPresence() const;
+    Presence currentPresence() const;
 
-    SimplePresence requestedPresence() const;
-    PendingOperation *setRequestedPresence(
-            const SimplePresence &value);
+    Presence requestedPresence() const;
+    PendingOperation *setRequestedPresence(const Presence &presence);
 
     bool isOnline() const;
 
     QString uniqueIdentifier() const;
 
-    // doc as for advanced users
-    QString connectionObjectPath() const;
-
     QString normalizedName() const;
 
     PendingOperation *reconnect();
@@ -217,30 +204,30 @@ public:
             const QDateTime &userActionTime = QDateTime::currentDateTime(),
             const QString &preferredHandler = QString());
 
-    PendingChannelRequest *ensureMediaCall(
+    PendingChannelRequest *ensureStreamedMediaCall(
             const QString &contactIdentifier,
             const QDateTime &userActionTime = QDateTime::currentDateTime(),
             const QString &preferredHandler = QString());
-    PendingChannelRequest *ensureMediaCall(
+    PendingChannelRequest *ensureStreamedMediaCall(
             const ContactPtr &contact,
             const QDateTime &userActionTime = QDateTime::currentDateTime(),
             const QString &preferredHandler = QString());
 
-    PendingChannelRequest *ensureAudioCall(
+    PendingChannelRequest *ensureStreamedMediaAudioCall(
             const QString &contactIdentifier,
             QDateTime userActionTime = QDateTime::currentDateTime(),
             const QString &preferredHandler = QString());
-    PendingChannelRequest *ensureAudioCall(
+    PendingChannelRequest *ensureStreamedMediaAudioCall(
             const ContactPtr &contact,
             QDateTime userActionTime = QDateTime::currentDateTime(),
             const QString &preferredHandler = QString());
 
-    PendingChannelRequest *ensureVideoCall(
+    PendingChannelRequest *ensureStreamedMediaVideoCall(
             const QString &contactIdentifier,
             bool withAudio = true,
             QDateTime userActionTime = QDateTime::currentDateTime(),
             const QString &preferredHandler = QString());
-    PendingChannelRequest *ensureVideoCall(
+    PendingChannelRequest *ensureStreamedMediaVideoCall(
             const ContactPtr &contact,
             bool withAudio = true,
             QDateTime userActionTime = QDateTime::currentDateTime(),
@@ -257,12 +244,12 @@ public:
             const QDateTime &userActionTime = QDateTime::currentDateTime(),
             const QString &preferredHandler = QString());
 
-    PendingChannelRequest *createConferenceMediaCall(
+    PendingChannelRequest *createConferenceStreamedMediaCall(
             const QList<ChannelPtr> &channels,
             const QStringList &initialInviteeContactsIdentifiers = QStringList(),
             const QDateTime &userActionTime = QDateTime::currentDateTime(),
             const QString &preferredHandler = QString());
-    PendingChannelRequest *createConferenceMediaCall(
+    PendingChannelRequest *createConferenceStreamedMediaCall(
             const QList<ChannelPtr> &channels,
             const QList<ContactPtr> &initialInviteeContacts = QList<ContactPtr>(),
             const QDateTime &userActionTime = QDateTime::currentDateTime(),
@@ -292,6 +279,12 @@ public:
             const QDateTime &userActionTime = QDateTime::currentDateTime(),
             const QString &preferredHandler = QString());
 
+    PendingChannelRequest *createContactSearch(
+            const QString &server = QString(),
+            uint limit = 0,
+            const QDateTime &userActionTime = QDateTime::currentDateTime(),
+            const QString &preferredHandler = QString());
+
     // advanced
     PendingChannelRequest *createChannel(
             const QVariantMap &requestedProperties,
@@ -302,55 +295,36 @@ public:
             const QDateTime &userActionTime = QDateTime::currentDateTime(),
             const QString &preferredHandler = QString());
 
-    inline Client::DBus::PropertiesInterface *propertiesInterface() const
-    {
-        return optionalInterface<Client::DBus::PropertiesInterface>(BypassInterfaceCheck);
-    }
-
-    inline Client::AccountInterfaceAvatarInterface *avatarInterface(
-            InterfaceSupportedChecking check = CheckInterfaceSupported) const
-    {
-        return optionalInterface<Client::AccountInterfaceAvatarInterface>(check);
-    }
-
 Q_SIGNALS:
     void removed();
     void serviceNameChanged(const QString &serviceName);
+    void profileChanged(const Tp::ProfilePtr &profile);
     void displayNameChanged(const QString &displayName);
-    TELEPATHY_QT4_DEPRECATED void iconChanged(const QString &iconName);
     void iconNameChanged(const QString &iconName);
     void nicknameChanged(const QString &nickname);
     void normalizedNameChanged(const QString &normalizedName);
     void validityChanged(bool validity);
     void stateChanged(bool state);
-    void capabilitiesChanged(Tp::ConnectionCapabilities *capabilities);
+    void capabilitiesChanged(const Tp::ConnectionCapabilities &capabilities);
     void connectsAutomaticallyPropertyChanged(bool connectsAutomatically);
     void firstOnline();
     void parametersChanged(const QVariantMap &parameters);
     void changingPresence(bool value);
-    void automaticPresenceChanged(const Tp::SimplePresence &automaticPresence) const;
-    void currentPresenceChanged(const Tp::SimplePresence &currentPresence) const;
-    void requestedPresenceChanged(const Tp::SimplePresence &requestedPresence) const;
+    void automaticPresenceChanged(const Tp::Presence &automaticPresence);
+    void currentPresenceChanged(const Tp::Presence &currentPresence);
+    void requestedPresenceChanged(const Tp::Presence &requestedPresence);
     void onlinenessChanged(bool online);
     void avatarChanged(const Tp::Avatar &avatar);
-    TELEPATHY_QT4_DEPRECATED void connectionStatusChanged(Tp::ConnectionStatus status,
-            Tp::ConnectionStatusReason statusReason);
-    void statusChanged(Tp::ConnectionStatus status,
-            Tp::ConnectionStatusReason statusReason,
-            const QString &error, const QVariantMap &errorDetails);
-    void haveConnectionChanged(bool haveConnection);
-
-    void propertyChanged(const QString &propertyName);
+    void connectionStatusChanged(Tp::ConnectionStatus status);
+    void connectionChanged(const Tp::ConnectionPtr &connection);
 
 protected:
-    Account(const QString &busName, const QString &objectPath);
-    Account(const QDBusConnection &bus,
-            const QString &busName, const QString &objectPath);
     Account(const QDBusConnection &bus,
             const QString &busName, const QString &objectPath,
             const ConnectionFactoryConstPtr &connectionFactory,
             const ChannelFactoryConstPtr &channelFactory,
-            const ContactFactoryConstPtr &contactFactory);
+            const ContactFactoryConstPtr &contactFactory,
+            const Feature &coreFeature);
 
     Client::AccountInterface *baseInterface() const;
 
@@ -368,8 +342,6 @@ private:
     struct Private;
     friend struct Private;
 
-    void notify(const char *propertyName);
-
     Private *mPriv;
 };
 
