@@ -66,7 +66,7 @@ struct TELEPATHY_QT4_NO_EXPORT ChannelDispatchOperation::Private
     // Instance of generated interface class
     Client::ChannelDispatchOperationInterface *baseInterface;
 
-    // Optional interface proxies
+    // Mandatory properties interface proxy
     Client::DBus::PropertiesInterface *properties;
 
     ReadinessHelper *readinessHelper;
@@ -82,10 +82,8 @@ struct TELEPATHY_QT4_NO_EXPORT ChannelDispatchOperation::Private
 
 ChannelDispatchOperation::Private::Private(ChannelDispatchOperation *parent)
     : parent(parent),
-      baseInterface(new Client::ChannelDispatchOperationInterface(
-                    parent->dbusConnection(), parent->busName(),
-                    parent->objectPath(), parent)),
-      properties(0),
+      baseInterface(new Client::ChannelDispatchOperationInterface(parent)),
+      properties(parent->interface<Client::DBus::PropertiesInterface>()),
       readinessHelper(parent->readinessHelper()),
       gotPossibleHandlers(false)
 {
@@ -96,8 +94,8 @@ ChannelDispatchOperation::Private::Private(ChannelDispatchOperation *parent)
             SLOT(onFinished()));
 
     parent->connect(baseInterface,
-            SIGNAL(ChannelLost(const QDBusObjectPath &, const QString &, const QString &)),
-            SLOT(onChannelLost(const QDBusObjectPath &, const QString &, const QString &)));
+            SIGNAL(ChannelLost(QDBusObjectPath,QString,QString)),
+            SLOT(onChannelLost(QDBusObjectPath,QString,QString)));
 
     ReadinessHelper::Introspectables introspectables;
 
@@ -119,11 +117,6 @@ ChannelDispatchOperation::Private::~Private()
 
 void ChannelDispatchOperation::Private::introspectMain(ChannelDispatchOperation::Private *self)
 {
-    if (!self->properties) {
-        self->properties = self->parent->propertiesInterface();
-        Q_ASSERT(self->properties != 0);
-    }
-
     QVariantMap mainProps;
     foreach (QString key, self->immutableProperties.keys()) {
         if (key.startsWith(QLatin1String(TELEPATHY_INTERFACE_CHANNEL_DISPATCH_OPERATION "."))) {
@@ -168,22 +161,22 @@ void ChannelDispatchOperation::Private::extractMainProps(const QVariantMap &prop
             connectionObjectPath.path().mid(1).replace(QLatin1String("/"),
                     QLatin1String("."));
 
-        PendingReady *ready =
+        PendingReady *readyOp =
             connFactory->proxy(connectionBusName, connectionObjectPath.path(),
                     chanFactory, contactFactory);
-        connection = ConnectionPtr::dynamicCast(ready->proxy());
-        readyOps.append(ready);
+        connection = ConnectionPtr::qObjectCast(readyOp->proxy());
+        readyOps.append(readyOp);
     }
 
     if (!account && props.contains(QLatin1String("Account"))) {
         QDBusObjectPath accountObjectPath =
             qdbus_cast<QDBusObjectPath>(props.value(QLatin1String("Account")));
 
-        PendingReady *ready =
+        PendingReady *readyOp =
             accFactory->proxy(QLatin1String(TELEPATHY_ACCOUNT_MANAGER_BUS_NAME),
                     accountObjectPath.path(), connFactory, chanFactory, contactFactory);
-        account = AccountPtr::dynamicCast(ready->proxy());
-        readyOps.append(ready);
+        account = AccountPtr::qObjectCast(readyOp->proxy());
+        readyOps.append(readyOp);
     }
 
     if (!immutableProperties) {
@@ -199,11 +192,11 @@ void ChannelDispatchOperation::Private::extractMainProps(const QVariantMap &prop
             qdbus_cast<ChannelDetailsList>(props.value(QLatin1String("Channels")));
         ChannelPtr channel;
         foreach (const ChannelDetails &channelDetails, channelDetailsList) {
-            PendingReady *ready =
+            PendingReady *readyOp =
                 chanFactory->proxy(connection,
                         channelDetails.channel.path(), channelDetails.properties);
-            channels.append(ChannelPtr::dynamicCast(ready->proxy()));
-            readyOps.append(ready);
+            channels.append(ChannelPtr::qObjectCast(readyOp->proxy()));
+            readyOps.append(readyOp);
         }
 
         // saveChannels goes out of scope now, so any initial channels which don't exist anymore are
@@ -219,7 +212,7 @@ void ChannelDispatchOperation::Private::extractMainProps(const QVariantMap &prop
         debug() << "No proxies to prepare for CDO" << parent->objectPath();
         readinessHelper->setIntrospectCompleted(FeatureCore, true);
     } else {
-        parent->connect(new PendingComposite(readyOps, parent),
+        parent->connect(new PendingComposite(readyOps, ChannelDispatchOperationPtr(parent)),
                 SIGNAL(finished(Tp::PendingOperation*)),
                 SLOT(onProxiesPrepared(Tp::PendingOperation*)));
     }
@@ -284,41 +277,6 @@ void ChannelDispatchOperation::Private::extractMainProps(const QVariantMap &prop
  */
 const Feature ChannelDispatchOperation::FeatureCore = Feature(QLatin1String(ChannelDispatchOperation::staticMetaObject.className()), 0, true);
 
-/**
- * Create a new channel dispatch operation object using
- * QDBusConnection::sessionBus().
- *
- * \param objectPath The channel dispatch operation object path.
- * \param immutableProperties The immutable properties of the channel dispatch
- *        operation.
- * \return A ChannelDispatchOperationPtr pointing to the newly created
- *         ChannelDispatchOperation.
- */
-ChannelDispatchOperationPtr ChannelDispatchOperation::create(const QString &objectPath,
-        const QVariantMap &immutableProperties)
-{
-    return ChannelDispatchOperationPtr(new ChannelDispatchOperation(
-                QDBusConnection::sessionBus(), objectPath,
-                immutableProperties));
-}
-
-/**
- * Create a new channel dispatch operation object using the given \a bus.
- *
- * \param bus QDBusConnection to use.
- * \param objectPath The channel dispatch operation object path.
- * \param immutableProperties The immutable properties of the channel dispatch
- *        operation.
- * \return A ChannelDispatchOperationPtr pointing to the newly created
- *         ChannelDispatchOperation.
- */
-ChannelDispatchOperationPtr ChannelDispatchOperation::create(const QDBusConnection &bus,
-        const QString &objectPath, const QVariantMap &immutableProperties)
-{
-    return ChannelDispatchOperationPtr(new ChannelDispatchOperation(
-                bus, objectPath, immutableProperties));
-}
-
 /**
  * Create a new channel dispatch operation object using the given \a bus, the given factories and
  * the given initial channels.
@@ -348,34 +306,6 @@ ChannelDispatchOperationPtr ChannelDispatchOperation::create(const QDBusConnecti
                 connectionFactory, channelFactory, contactFactory));
 }
 
-/**
- * Construct a new channel dispatch operation object using the given \a bus.
- *
- * \param bus QDBusConnection to use
- * \param objectPath The channel dispatch operation object path.
- * \param immutableProperties The immutable properties of the channel dispatch
- *        operation.
- */
-ChannelDispatchOperation::ChannelDispatchOperation(const QDBusConnection &bus,
-        const QString &objectPath, const QVariantMap &immutableProperties)
-    : StatefulDBusProxy(bus,
-            QLatin1String(TELEPATHY_INTERFACE_CHANNEL_DISPATCHER),
-            objectPath),
-      OptionalInterfaceFactory<ChannelDispatchOperation>(this),
-      ReadyObject(this, FeatureCore),
-      mPriv(new Private(this))
-{
-    // API/ABI break TODO: remove this constructor and any other way to get "default factories" in
-    // CDO - these sort of emulate the old pre-factory behavior of CDO, making core in Acc and Conn
-    // ready, but sadly not in Channels.
-    mPriv->accFactory = AccountFactory::create(bus, Account::FeatureCore);
-    mPriv->connFactory = ConnectionFactory::create(bus, Connection::FeatureCore);
-    mPriv->chanFactory = ChannelFactory::create(bus);
-    mPriv->contactFactory = ContactFactory::create();
-
-    mPriv->immutableProperties = immutableProperties;
-}
-
 /**
  * Construct a new channel dispatch operation object using the given \a bus, the given factories and
  * the given initial channels.
@@ -399,9 +329,8 @@ ChannelDispatchOperation::ChannelDispatchOperation(const QDBusConnection &bus,
         const ContactFactoryConstPtr &contactFactory)
     : StatefulDBusProxy(bus,
             QLatin1String(TELEPATHY_INTERFACE_CHANNEL_DISPATCHER),
-            objectPath),
+            objectPath, FeatureCore),
       OptionalInterfaceFactory<ChannelDispatchOperation>(this),
-      ReadyObject(this, FeatureCore),
       mPriv(new Private(this))
 {
     if (accountFactory->dbusConnection().name() != bus.name()) {
@@ -521,7 +450,7 @@ PendingOperation *ChannelDispatchOperation::handleWith(const QString &handler)
 {
     return new PendingVoid(
             mPriv->baseInterface->HandleWith(handler),
-            this);
+            ChannelDispatchOperationPtr(this));
 }
 
 /**
@@ -560,7 +489,8 @@ PendingOperation *ChannelDispatchOperation::handleWith(const QString &handler)
  */
 PendingOperation *ChannelDispatchOperation::claim()
 {
-    return new PendingVoid(mPriv->baseInterface->Claim(), this);
+    return new PendingVoid(mPriv->baseInterface->Claim(),
+            ChannelDispatchOperationPtr(this));
 }
 
 /**
@@ -591,17 +521,6 @@ Client::ChannelDispatchOperationInterface *ChannelDispatchOperation::baseInterfa
     return mPriv->baseInterface;
 }
 
-/**
- * \fn Client::DBus::PropertiesInterface *ChannelDispatchOperation::propertiesInterface() const
- *
- * Convenience function for getting a PropertiesInterface interface proxy object
- * for this account. The ChannelDispatchOperation interface relies on
- * properties, so this interface is always assumed to be present.
- *
- * \return A pointer to the existing Client::DBus::PropertiesInterface object
- *         for this ChannelDispatchOperation object.
- */
-
 void ChannelDispatchOperation::onFinished()
 {
     debug() << "ChannelDispatchOperation finished and was removed";
