@@ -34,7 +34,6 @@
 #include <TelepathyQt4/Feature>
 #include <TelepathyQt4/OptionalInterfaceFactory>
 #include <TelepathyQt4/ReadinessHelper>
-#include <TelepathyQt4/ReadyObject>
 #include <TelepathyQt4/Types>
 #include <TelepathyQt4/SharedPtr>
 
@@ -48,9 +47,7 @@ namespace Tp
 class PendingOperation;
 
 class TELEPATHY_QT4_EXPORT ChannelDispatchOperation : public StatefulDBusProxy,
-                                 public OptionalInterfaceFactory<ChannelDispatchOperation>,
-                                 public ReadyObject,
-                                 public RefCounted
+                public OptionalInterfaceFactory<ChannelDispatchOperation>
 {
     Q_OBJECT
     Q_DISABLE_COPY(ChannelDispatchOperation)
@@ -58,11 +55,6 @@ class TELEPATHY_QT4_EXPORT ChannelDispatchOperation : public StatefulDBusProxy,
 public:
     static const Feature FeatureCore;
 
-    TELEPATHY_QT4_DEPRECATED static ChannelDispatchOperationPtr create(const QString &objectPath,
-            const QVariantMap &immutableProperties);
-    TELEPATHY_QT4_DEPRECATED static ChannelDispatchOperationPtr create(const QDBusConnection &bus,
-            const QString &objectPath, const QVariantMap &immutableProperties);
-
     static ChannelDispatchOperationPtr create(const QDBusConnection &bus,
             const QString &objectPath, const QVariantMap &immutableProperties,
             const QList<ChannelPtr> &initialChannels,
@@ -84,19 +76,11 @@ public:
 
     PendingOperation *claim();
 
-    inline Client::DBus::PropertiesInterface *propertiesInterface() const
-    {
-        return optionalInterface<Client::DBus::PropertiesInterface>(BypassInterfaceCheck);
-    }
-
 Q_SIGNALS:
-    void channelLost(const ChannelPtr &channel, const QString &errorName,
+    void channelLost(const Tp::ChannelPtr &channel, const QString &errorName,
             const QString &errorMessage);
 
 protected:
-    TELEPATHY_QT4_DEPRECATED ChannelDispatchOperation(const QDBusConnection &bus,
-            const QString &objectPath, const QVariantMap &immutableProperties);
-
     ChannelDispatchOperation(const QDBusConnection &bus,
             const QString &objectPath, const QVariantMap &immutableProperties,
             const QList<ChannelPtr> &initialChannels,
