@@ -40,12 +40,16 @@ namespace Tp
 
 struct TELEPATHY_QT4_NO_EXPORT ChannelRequest::Private
 {
-    Private(ChannelRequest *parent, const QVariantMap &immutableProperties);
+    Private(ChannelRequest *parent, const QVariantMap &immutableProperties,
+            const AccountFactoryConstPtr &, const ConnectionFactoryConstPtr &,
+            const ChannelFactoryConstPtr &, const ContactFactoryConstPtr &);
     ~Private();
 
     static void introspectMain(Private *self);
 
-    void extractMainProps(const QVariantMap &props);
+    // \param lastCall Is this the last call to extractMainProps ie. should actions that only must
+    // be done once be done in this call
+    void extractMainProps(const QVariantMap &props, bool lastCall);
 
     // Public object
     ChannelRequest *parent;
@@ -59,11 +63,11 @@ struct TELEPATHY_QT4_NO_EXPORT ChannelRequest::Private
     // Instance of generated interface class
     Client::ChannelRequestInterface *baseInterface;
 
-    QVariantMap immutableProperties;
-
-    // Optional interface proxies
+    // Mandatory properties interface proxy
     Client::DBus::PropertiesInterface *properties;
 
+    QVariantMap immutableProperties;
+
     ReadinessHelper *readinessHelper;
 
     // Introspection
@@ -71,23 +75,31 @@ struct TELEPATHY_QT4_NO_EXPORT ChannelRequest::Private
     QDateTime userActionTime;
     QString preferredHandler;
     QualifiedPropertyValueMapList requests;
+    bool propertiesDone;
 };
 
 ChannelRequest::Private::Private(ChannelRequest *parent,
-        const QVariantMap &immutableProperties)
+        const QVariantMap &immutableProperties,
+        const AccountFactoryConstPtr &accFact,
+        const ConnectionFactoryConstPtr &connFact,
+        const ChannelFactoryConstPtr &chanFact,
+        const ContactFactoryConstPtr &contactFact)
     : parent(parent),
-      baseInterface(new Client::ChannelRequestInterface(
-                    parent->dbusConnection(), parent->busName(),
-                    parent->objectPath(), parent)),
+      accFact(accFact),
+      connFact(connFact),
+      chanFact(chanFact),
+      contactFact(contactFact),
+      baseInterface(new Client::ChannelRequestInterface(parent)),
+      properties(parent->interface<Client::DBus::PropertiesInterface>()),
       immutableProperties(immutableProperties),
-      properties(0),
-      readinessHelper(parent->readinessHelper())
+      readinessHelper(parent->readinessHelper()),
+      propertiesDone(false)
 {
     debug() << "Creating new ChannelRequest:" << parent->objectPath();
 
     parent->connect(baseInterface,
-            SIGNAL(Failed(const QString &, const QString &)),
-            SIGNAL(failed(const QString &, const QString &)));
+            SIGNAL(Failed(QString,QString)),
+            SIGNAL(failed(QString,QString)));
     parent->connect(baseInterface,
             SIGNAL(Succeeded()),
             SIGNAL(succeeded()));
@@ -104,6 +116,20 @@ ChannelRequest::Private::Private(ChannelRequest *parent,
     introspectables[FeatureCore] = introspectableCore;
 
     readinessHelper->addIntrospectables(introspectables);
+
+    // For early access to the immutable properties through the friendly getters - will be called
+    // again with lastCall = true eventually, if/when becomeReady is called, though
+    QVariantMap mainProps;
+    foreach (QString key, immutableProperties.keys()) {
+        // The key.count thing is so that we don't match "org.fdo.Tp.CR.OptionalInterface.Prop" too
+        if (key.startsWith(QLatin1String(TELEPATHY_INTERFACE_CHANNEL_REQUEST "."))
+                && key.count(QLatin1Char('.')) ==
+                    QString::fromAscii(TELEPATHY_INTERFACE_CHANNEL_REQUEST ".").count(QLatin1Char('.'))) {
+            QVariant value = immutableProperties.value(key);
+            mainProps.insert(key.remove(QLatin1String(TELEPATHY_INTERFACE_CHANNEL_REQUEST ".")), value);
+        }
+    }
+    extractMainProps(mainProps, false);
 }
 
 ChannelRequest::Private::~Private()
@@ -112,11 +138,6 @@ ChannelRequest::Private::~Private()
 
 void ChannelRequest::Private::introspectMain(ChannelRequest::Private *self)
 {
-    if (!self->properties) {
-        self->properties = self->parent->propertiesInterface();
-        Q_ASSERT(self->properties != 0);
-    }
-
     QVariantMap props;
     QString key;
     bool needIntrospectMainProps = false;
@@ -151,11 +172,11 @@ void ChannelRequest::Private::introspectMain(ChannelRequest::Private *self)
                     SLOT(gotMainProperties(QDBusPendingCallWatcher*)));
         }
     } else {
-        self->extractMainProps(props);
+        self->extractMainProps(props, true);
     }
 }
 
-void ChannelRequest::Private::extractMainProps(const QVariantMap &props)
+void ChannelRequest::Private::extractMainProps(const QVariantMap &props, bool lastCall)
 {
     PendingReady *readyOp = 0;
 
@@ -176,27 +197,16 @@ void ChannelRequest::Private::extractMainProps(const QVariantMap &props)
         }
 
         // We need to check again because we might have dropped the expected account just a sec ago
-        // Note that many of the if paths will go away in the API/ABI break - they're just for
-        // backwards compat
-        if (account.isNull()) {
+        if (account.isNull() && !accountObjectPath.path().isEmpty()) {
             if (!accFact.isNull()) {
                 readyOp = accFact->proxy(
                         QLatin1String(TELEPATHY_ACCOUNT_MANAGER_BUS_NAME), accountObjectPath.path(),
                         connFact, chanFact, contactFact);
-                account = AccountPtr::dynamicCast(readyOp->proxy());
+                account = AccountPtr::qObjectCast(readyOp->proxy());
             } else {
-                // We might have the conn fact from the expected account, let's use that for good
-                // measure even if the account didn't match
-                if (!connFact.isNull()) {
-                    account = Account::create(
-                            QLatin1String(TELEPATHY_ACCOUNT_MANAGER_BUS_NAME),
-                            accountObjectPath.path(), connFact, chanFact, contactFact);
-                } else {
-                    account = Account::create(
-                            QLatin1String(TELEPATHY_ACCOUNT_MANAGER_BUS_NAME),
-                            accountObjectPath.path());
-                }
-
+                account = Account::create(
+                        QLatin1String(TELEPATHY_ACCOUNT_MANAGER_BUS_NAME),
+                        accountObjectPath.path(), connFact, chanFact, contactFact);
                 readyOp = account->becomeReady();
             }
         }
@@ -214,12 +224,16 @@ void ChannelRequest::Private::extractMainProps(const QVariantMap &props)
     parent->setInterfaces(qdbus_cast<QStringList>(props[QLatin1String("Interfaces")]));
     readinessHelper->setInterfaces(parent->interfaces());
 
+    if (lastCall) {
+        propertiesDone = true;
+    }
+
     if (account) {
         parent->connect(readyOp,
-                SIGNAL(finished(Tp::PendingOperation *)),
-                SLOT(onAccountReady(Tp::PendingOperation *)));
-    } else {
-        warning() << "ChannelRequest.Account is missing or empty. Ignoring";
+                SIGNAL(finished(Tp::PendingOperation*)),
+                SLOT(onAccountReady(Tp::PendingOperation*)));
+    } else if (lastCall) {
+        warning() << "No account for ChannelRequest" << parent->objectPath();
         readinessHelper->setIntrospectCompleted(FeatureCore, true);
     }
 }
@@ -254,35 +268,6 @@ void ChannelRequest::Private::extractMainProps(const QVariantMap &props)
  */
 const Feature ChannelRequest::FeatureCore = Feature(QLatin1String(ChannelRequest::staticMetaObject.className()), 0, true);
 
-/**
- * Create a new channel request object using QDBusConnection::sessionBus().
- *
- * \param objectPath The channel request object path.
- * \param immutableProperties The immutable properties of the channel request.
- * \return A ChannelRequestPtr pointing to the newly created ChannelRequest.
- */
-ChannelRequestPtr ChannelRequest::create(const QString &objectPath,
-        const QVariantMap &immutableProperties)
-{
-    return ChannelRequestPtr(new ChannelRequest(QDBusConnection::sessionBus(),
-                objectPath, immutableProperties));
-}
-
-/**
- * Create a new channel request object using the given \a bus.
- *
- * \param bus QDBusConnection to use.
- * \param objectPath The channel request object path.
- * \param immutableProperties The immutable properties of the channel request.
- * \return A ChannelRequestPtr pointing to the newly created ChannelRequest.
- */
-ChannelRequestPtr ChannelRequest::create(const QDBusConnection &bus,
-        const QString &objectPath, const QVariantMap &immutableProperties)
-{
-    return ChannelRequestPtr(new ChannelRequest(bus, objectPath,
-                immutableProperties));
-}
-
 /**
  * Create a new channel request object using the given \a bus and the given factories.
  *
@@ -321,25 +306,6 @@ ChannelRequestPtr ChannelRequest::create(const AccountPtr &account, const QStrin
     return ChannelRequestPtr(new ChannelRequest(account, objectPath, immutableProperties));
 }
 
-/**
- * Construct a new channel request object using the given \a bus.
- *
- * \param bus QDBusConnection to use.
- * \param objectPath The channel request object path.
- * \param immutableProperties The immutable properties of the channel request.
- * \return A ChannelRequestPtr pointing to the newly created ChannelRequest.
- */
-ChannelRequest::ChannelRequest(const QDBusConnection &bus,
-        const QString &objectPath, const QVariantMap &immutableProperties)
-    : StatefulDBusProxy(bus,
-            QLatin1String(TELEPATHY_INTERFACE_CHANNEL_DISPATCHER),
-            objectPath),
-      OptionalInterfaceFactory<ChannelRequest>(this),
-      ReadyObject(this, FeatureCore),
-      mPriv(new Private(this, immutableProperties))
-{
-}
-
 /**
  * Construct a new channel request object using the given \a bus and the given factories.
  *
@@ -360,10 +326,10 @@ ChannelRequest::ChannelRequest(const QDBusConnection &bus,
         const ContactFactoryConstPtr &contactFactory)
     : StatefulDBusProxy(bus,
             QLatin1String(TELEPATHY_INTERFACE_CHANNEL_DISPATCHER),
-            objectPath),
+            objectPath, FeatureCore),
       OptionalInterfaceFactory<ChannelRequest>(this),
-      ReadyObject(this, FeatureCore),
-      mPriv(new Private(this, immutableProperties))
+      mPriv(new Private(this, immutableProperties, accountFactory, connectionFactory,
+                  channelFactory, contactFactory))
 {
     if (accountFactory->dbusConnection().name() != bus.name()) {
         warning() << "  The D-Bus connection in the account factory is not the proxy connection";
@@ -376,11 +342,6 @@ ChannelRequest::ChannelRequest(const QDBusConnection &bus,
     if (channelFactory->dbusConnection().name() != bus.name()) {
         warning() << "  The D-Bus connection in the channel factory is not the proxy connection";
     }
-
-    mPriv->accFact = accountFactory;
-    mPriv->connFact = connectionFactory;
-    mPriv->chanFact = channelFactory;
-    mPriv->contactFact = contactFactory;
 }
 
 /**
@@ -397,15 +358,14 @@ ChannelRequest::ChannelRequest(const AccountPtr &account,
         const QString &objectPath, const QVariantMap &immutableProperties)
     : StatefulDBusProxy(account->dbusConnection(),
             QLatin1String(TELEPATHY_INTERFACE_CHANNEL_DISPATCHER),
-            objectPath),
+            objectPath, FeatureCore),
       OptionalInterfaceFactory<ChannelRequest>(this),
-      ReadyObject(this, FeatureCore),
-      mPriv(new Private(this, immutableProperties))
+      mPriv(new Private(this, immutableProperties, AccountFactoryPtr(),
+                  account->connectionFactory(),
+                  account->channelFactory(),
+                  account->contactFactory()))
 {
     mPriv->account = account;
-    mPriv->connFact = account->connectionFactory();
-    mPriv->chanFact = account->channelFactory();
-    mPriv->contactFact = account->contactFactory();
 }
 
 /**
@@ -419,7 +379,14 @@ ChannelRequest::~ChannelRequest()
 /**
  * Return the Account on which this request was made.
  *
- * This method requires ChannelRequest::FeatureCore to be enabled.
+ * This method can be used even before the ChannelRequest is ready, in which case the account object
+ * corresponding to the immutable properties is returned. In this case, the Account object is not
+ * necessarily ready either. This is useful for eg. matching ChannelRequests from
+ * ClientHandlerInterface::addRequest() with existing accounts in the application: either by object
+ * path, or if account factories are in use, even by object identity.
+ *
+ * If the account is not provided in the immutable properties, this will only return a non-\c NULL
+ * AccountPtr once ChannelRequest::FeatureCore is ready on this object.
  *
  * \return The account on which this request was made.
  */
@@ -437,7 +404,8 @@ AccountPtr ChannelRequest::account() const
  * This property is set when the channel request is created, and can never
  * change.
  *
- * This method requires ChannelRequest::FeatureCore to be enabled.
+ * This method can be used even before the ChannelRequest is ready: in this case, the user action
+ * time from the immutable properties, if any, is returned.
  *
  * \return The time at which the user action occurred.
  */
@@ -454,7 +422,8 @@ QDateTime ChannelRequest::userActionTime() const
  * This property is set when the channel request is created, and can never
  * change.
  *
- * This method requires ChannelRequest::FeatureCore to be enabled.
+ * This method can be used even before the ChannelRequest is ready: in this case, the preferred
+ * handler from the immutable properties, if any, is returned.
  *
  * \return The preferred handler or an empty string if any handler would be
  *         acceptable.
@@ -465,21 +434,61 @@ QString ChannelRequest::preferredHandler() const
 }
 
 /**
- * Return the desirable properties for the channel or channels to be created.
+ * Return the desirable properties for the channel or channels to be created, as specified when
+ * placing the request in the first place.
  *
  * This property is set when the channel request is created, and can never
  * change.
  *
- * This method requires ChannelRequest::FeatureCore to be enabled.
+ * This method can be used even before the ChannelRequest is ready: in this case, the requested
+ * channel properties from the immutable properties, if any, are returned. This is useful for e.g.
+ * matching ChannelRequests from ClientHandlerInterface::addRequest() with existing requests in the
+ * application (by the target ID or handle, most likely).
  *
- * \return The preferred handler or an empty string if any handler would be
- *         acceptable.
+ * \return The requested desirable channel properties.
  */
 QualifiedPropertyValueMapList ChannelRequest::requests() const
 {
     return mPriv->requests;
 }
 
+/**
+ * Return all of the immutable properties passed to this object when created.
+ *
+ * This is useful for e.g. getting to domain-specific properties of channel requests.
+ *
+ * \return The immutable properties.
+ */
+QVariantMap ChannelRequest::immutableProperties() const
+{
+    QVariantMap props = mPriv->immutableProperties;
+
+    if (!account().isNull()) {
+        props.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL_REQUEST ".Account"),
+            QVariant::fromValue(QDBusObjectPath(account()->objectPath())));
+    }
+
+    if (userActionTime().isValid()) {
+        props.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL_REQUEST ".UserActionTime"),
+            QVariant::fromValue(userActionTime().toTime_t()));
+    }
+
+    if (!preferredHandler().isNull()) {
+        props.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL_REQUEST ".PreferredHandler"),
+                preferredHandler());
+    }
+
+    if (!requests().isEmpty()) {
+        props.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL_REQUEST ".Requests"),
+            QVariant::fromValue(requests()));
+    }
+
+    props.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL_REQUEST ".Interfaces"),
+            QVariant::fromValue(interfaces()));
+
+    return props;
+}
+
 /**
  * Cancel the channel request.
  *
@@ -494,7 +503,8 @@ QualifiedPropertyValueMapList ChannelRequest::requests() const
  */
 PendingOperation *ChannelRequest::cancel()
 {
-    return new PendingVoid(mPriv->baseInterface->Cancel(), this);
+    return new PendingVoid(mPriv->baseInterface->Cancel(),
+            ChannelRequestPtr(this));
 }
 
 /**
@@ -509,7 +519,8 @@ PendingOperation *ChannelRequest::cancel()
  */
 PendingOperation *ChannelRequest::proceed()
 {
-    return new PendingVoid(mPriv->baseInterface->Proceed(), this);
+    return new PendingVoid(mPriv->baseInterface->Proceed(),
+            ChannelRequestPtr(this));
 }
 
 /**
@@ -543,17 +554,6 @@ Client::ChannelRequestInterface *ChannelRequest::baseInterface() const
  * methods must not be called on it.
  */
 
-/**
- * \fn Client::DBus::PropertiesInterface *ChannelRequest::propertiesInterface() const
- *
- * Convenience function for getting a PropertiesInterface interface proxy object
- * for this account. The ChannelRequest interface relies on
- * properties, so this interface is always assumed to be present.
- *
- * \return A pointer to the existing Client::DBus::PropertiesInterface object
- *         for this ChannelRequest object.
- */
-
 void ChannelRequest::gotMainProperties(QDBusPendingCallWatcher *watcher)
 {
     QDBusPendingReply<QVariantMap> reply = *watcher;
@@ -563,7 +563,7 @@ void ChannelRequest::gotMainProperties(QDBusPendingCallWatcher *watcher)
         debug() << "Got reply to Properties::GetAll(ChannelRequest)";
         props = reply.value();
 
-        mPriv->extractMainProps(props);
+        mPriv->extractMainProps(props, true);
     } else {
         mPriv->readinessHelper->setIntrospectCompleted(FeatureCore,
                 false, reply.error());
@@ -582,7 +582,10 @@ void ChannelRequest::onAccountReady(PendingOperation *op)
                 op->errorName(), op->errorMessage());
         return;
     }
-    mPriv->readinessHelper->setIntrospectCompleted(FeatureCore, true);
+
+    if (mPriv->propertiesDone && !isReady()) {
+        mPriv->readinessHelper->setIntrospectCompleted(FeatureCore, true);
+    }
 }
 
 } // Tp
