@@ -1,8 +1,8 @@
 /*
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2009 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2009 Nokia Corporation
+ * Copyright (C) 2009-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * Copyright (C) 2009-2010 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -133,20 +133,20 @@ void ClientObserverAdaptor::ObserveChannels(const QDBusObjectPath &accountPath,
             connFactory,
             chanFactory,
             contactFactory);
-    invocation->acc = AccountPtr::dynamicCast(accReady->proxy());
+    invocation->acc = AccountPtr::qObjectCast(accReady->proxy());
     readyOps.append(accReady);
 
     QString connectionBusName = connectionPath.path().mid(1).replace(
             QLatin1String("/"), QLatin1String("."));
     PendingReady *connReady = connFactory->proxy(connectionBusName, connectionPath.path(),
             chanFactory, contactFactory);
-    invocation->conn = ConnectionPtr::dynamicCast(connReady->proxy());
+    invocation->conn = ConnectionPtr::qObjectCast(connReady->proxy());
     readyOps.append(connReady);
 
     foreach (const ChannelDetails &channelDetails, channelDetailsList) {
         PendingReady *chanReady = chanFactory->proxy(invocation->conn,
                 channelDetails.channel.path(), channelDetails.properties);
-        ChannelPtr channel = ChannelPtr::dynamicCast(chanReady->proxy());
+        ChannelPtr channel = ChannelPtr::qObjectCast(chanReady->proxy());
         invocation->chans.append(channel);
         readyOps.append(chanReady);
     }
@@ -186,25 +186,20 @@ void ClientObserverAdaptor::ObserveChannels(const QDBusObjectPath &accountPath,
         readyOps.append(invocation->dispatchOp->becomeReady());
     }
 
-    // TODO make observer info nicer to use...
-    invocation->observerInfo = observerInfo;
+    invocation->observerInfo = AbstractClientObserver::ObserverInfo(observerInfo);
 
-    /*
-     * Uncomment this and the below one when we have spec 0.19.12
-     *
-     * ObjectImmutablePropertiesMap propMap = qdbus_cast<ObjectImmutablePropertiesMap>(
-     * observerInfo.value(QLatin1String("request-properties")));
-     */
-    foreach (const QDBusObjectPath &path, requestsSatisfied) {
+    ObjectImmutablePropertiesMap reqPropsMap = qdbus_cast<ObjectImmutablePropertiesMap>(
+            observerInfo.value(QLatin1String("request-properties")));
+    foreach (const QDBusObjectPath &reqPath, requestsSatisfied) {
         ChannelRequestPtr channelRequest = ChannelRequest::create(invocation->acc,
-                path.path(), QVariantMap() /* propMap.value(path.path()) */);
+                reqPath.path(), reqPropsMap.value(reqPath));
         invocation->chanReqs.append(channelRequest);
         readyOps.append(channelRequest->becomeReady());
     }
 
     invocation->ctx = MethodInvocationContextPtr<>(new MethodInvocationContext<>(mBus, message));
 
-    invocation->readyOp = new PendingComposite(readyOps, this);
+    invocation->readyOp = new PendingComposite(readyOps, invocation->ctx);
     connect(invocation->readyOp,
             SIGNAL(finished(Tp::PendingOperation*)),
             SLOT(onReadyOpFinished(Tp::PendingOperation*)));
@@ -251,8 +246,6 @@ void ClientObserverAdaptor::onReadyOpFinished(Tp::PendingOperation *op)
         debug() << "Invoking application observeChannels with" << invocation->chans.size()
             << "channels on" << mClient;
 
-        // API/ABI break TODO: make observerInfo a friendly high-level variantmap wrapper similar to
-        // Connection::ErrorDetails
         mClient->observeChannels(invocation->ctx, invocation->acc, invocation->conn,
                 invocation->chans, invocation->dispatchOp, invocation->chanReqs,
                 invocation->observerInfo);
@@ -293,7 +286,7 @@ void ClientApproverAdaptor::AddDispatchOperation(const Tp::ChannelDetailsList &c
             QLatin1String("/"), QLatin1String("."));
     PendingReady *connReady = connFactory->proxy(connectionBusName, connectionPath.path(), chanFactory,
                 contactFactory);
-    ConnectionPtr connection = ConnectionPtr::dynamicCast(connReady->proxy());
+    ConnectionPtr connection = ConnectionPtr::qObjectCast(connReady->proxy());
     readyOps.append(connReady);
 
     SharedPtr<InvocationData> invocation(new InvocationData);
@@ -301,23 +294,18 @@ void ClientApproverAdaptor::AddDispatchOperation(const Tp::ChannelDetailsList &c
     foreach (const ChannelDetails &channelDetails, channelDetailsList) {
         PendingReady *chanReady = chanFactory->proxy(connection, channelDetails.channel.path(),
                 channelDetails.properties);
-        invocation->chans.append(ChannelPtr::dynamicCast(chanReady->proxy()));
+        invocation->chans.append(ChannelPtr::qObjectCast(chanReady->proxy()));
         readyOps.append(chanReady);
     }
 
-    // Note that creating the CDO used to not pass the bus connection at all! Which means the old
-    // code would've broken if used on a != sessionBus() bus. Therefore I think we can draw a
-    // conclusion that the classes we mostly create ourselves (CDO, CR, etc.) should NOT have
-    // default parameters for the bus and/or the factories after the API/ABI break, to catch this
-    // type of error at compile time.
     invocation->dispatchOp = ChannelDispatchOperation::create(mBus,
-            dispatchOperationPath.path(), properties, QList<ChannelPtr>(), accFactory, connFactory,
+            dispatchOperationPath.path(), properties, invocation->chans, accFactory, connFactory,
             chanFactory, contactFactory);
     readyOps.append(invocation->dispatchOp->becomeReady());
 
     invocation->ctx = MethodInvocationContextPtr<>(new MethodInvocationContext<>(mBus, message));
 
-    invocation->readyOp = new PendingComposite(readyOps, this);
+    invocation->readyOp = new PendingComposite(readyOps, invocation->ctx);
     connect(invocation->readyOp,
             SIGNAL(finished(Tp::PendingOperation*)),
             SLOT(onReadyOpFinished(Tp::PendingOperation*)));
@@ -361,9 +349,7 @@ void ClientApproverAdaptor::onReadyOpFinished(Tp::PendingOperation *op)
         debug() << "Invoking application addDispatchOperation with CDO"
             << invocation->dispatchOp->objectPath() << "on" << mClient;
 
-        // API/ABI break FIXME: Don't pass the same channels as the channels arg and (separately
-        // constructed) in dispatchOp->channels() !!!
-        mClient->addDispatchOperation(invocation->ctx, invocation->chans, invocation->dispatchOp);
+        mClient->addDispatchOperation(invocation->ctx, invocation->dispatchOp);
     }
 }
 
@@ -417,36 +403,31 @@ void ClientHandlerAdaptor::HandleChannels(const QDBusObjectPath &accountPath,
             connFactory,
             chanFactory,
             contactFactory);
-    invocation->acc = AccountPtr::dynamicCast(accReady->proxy());
+    invocation->acc = AccountPtr::qObjectCast(accReady->proxy());
     readyOps.append(accReady);
 
     QString connectionBusName = connectionPath.path().mid(1).replace(
             QLatin1String("/"), QLatin1String("."));
     PendingReady *connReady = connFactory->proxy(connectionBusName, connectionPath.path(),
             chanFactory, contactFactory);
-    invocation->conn = ConnectionPtr::dynamicCast(connReady->proxy());
+    invocation->conn = ConnectionPtr::qObjectCast(connReady->proxy());
     readyOps.append(connReady);
 
     foreach (const ChannelDetails &channelDetails, channelDetailsList) {
         PendingReady *chanReady = chanFactory->proxy(invocation->conn,
                 channelDetails.channel.path(), channelDetails.properties);
-        ChannelPtr channel = ChannelPtr::dynamicCast(chanReady->proxy());
+        ChannelPtr channel = ChannelPtr::qObjectCast(chanReady->proxy());
         invocation->chans.append(channel);
         readyOps.append(chanReady);
     }
 
-    // API/ABI break TODO: make handler info nicer to use
-    invocation->handlerInfo = handlerInfo;
+    invocation->handlerInfo = AbstractClientHandler::HandlerInfo(handlerInfo);
 
-    /*
-     * Uncomment this and the below one when we have spec 0.19.12
-     *
-     * ObjectImmutablePropertiesMap propMap = qdbus_cast<ObjectImmutablePropertiesMap>(
-     * handlerInfo.value(QLatin1String("request-properties")));
-     */
-    foreach (const QDBusObjectPath &path, requestsSatisfied) {
+    ObjectImmutablePropertiesMap reqPropsMap = qdbus_cast<ObjectImmutablePropertiesMap>(
+    handlerInfo.value(QLatin1String("request-properties")));
+    foreach (const QDBusObjectPath &reqPath, requestsSatisfied) {
         ChannelRequestPtr channelRequest = ChannelRequest::create(invocation->acc,
-                path.path(), QVariantMap() /* propMap.value(path.path()) */);
+                reqPath.path(), reqPropsMap.value(reqPath));
         invocation->chanReqs.append(channelRequest);
         readyOps.append(channelRequest->becomeReady());
     }
@@ -462,7 +443,7 @@ void ClientHandlerAdaptor::HandleChannels(const QDBusObjectPath &accountPath,
                     &ClientHandlerAdaptor::onContextFinished),
                 this);
 
-    invocation->readyOp = new PendingComposite(readyOps, this);
+    invocation->readyOp = new PendingComposite(readyOps, invocation->ctx);
     connect(invocation->readyOp,
             SIGNAL(finished(Tp::PendingOperation*)),
             SLOT(onReadyOpFinished(Tp::PendingOperation*)));
@@ -509,8 +490,6 @@ void ClientHandlerAdaptor::onReadyOpFinished(Tp::PendingOperation *op)
         debug() << "Invoking application observeChannels with" << invocation->chans.size()
             << "channels on" << mClient;
 
-        // API/ABI break TODO: make handlerInfo a friendly high-level variantmap wrapper similar to
-        // Connection::ErrorDetails
         mClient->handleChannels(invocation->ctx, invocation->acc, invocation->conn,
                 invocation->chans, invocation->chanReqs, invocation->time, invocation->handlerInfo);
     }
@@ -526,9 +505,8 @@ void ClientHandlerAdaptor::onContextFinished(
         foreach (const ChannelPtr &channel, channels) {
             self->mHandledChannels.insert(channel);
             self->connect(channel.data(),
-                    SIGNAL(invalidated(Tp::DBusProxy *,
-                                       const QString &, const QString &)),
-                    SLOT(onChannelInvalidated(Tp::DBusProxy *)));
+                    SIGNAL(invalidated(Tp::DBusProxy*,QString,QString)),
+                    SLOT(onChannelInvalidated(Tp::DBusProxy*)));
         }
     }
 }
@@ -670,47 +648,25 @@ struct TELEPATHY_QT4_NO_EXPORT ClientRegistrar::Private
  * \sa AbstractClientObserver, AbstractClientApprover, AbstractClientHandler
  */
 
-QHash<QPair<QString, QString>, ClientRegistrar*> ClientRegistrar::registrarForConnection;
-
 /**
  * Create a new client registrar object using the given \a bus.
  *
- * ClientRegistrar instances are unique per D-Bus connection. The returned
- * ClientRegistrarPtr will point to the same ClientRegistrar instance on
- * successive calls with the same \a bus, unless the instance
- * had already been destroyed, in which case a new instance will be returned.
- *
- * The resulting instance will use factories from a previous ClientRegistrar with the same \a bus,
- * if any, otherwise factories returning stock TpQt4 subclasses as approriate, with no features
- * prepared. This gives fully backwards compatible behavior for this function if the factory
- * variants are never used.
- *
- * \todo API/ABI break: Drop the bus-wide singleton guarantee, it's awkward and the associated name
- * registration checks have always been implemented incorrectly anyway. We need this for the account
- * friendly channel request and handle API at least.
+ * The instance will use an account factory creating Tp::Account objects with no features
+ * ready, a connection factory creating Tp::Connection objects with no features ready, and a channel
+ * factory creating stock Telepathy-Qt4 channel subclasses, as appropriate, with no features ready.
  *
  * \param bus QDBusConnection to use.
  * \return A ClientRegistrarPtr object pointing to the ClientRegistrar.
  */
 ClientRegistrarPtr ClientRegistrar::create(const QDBusConnection &bus)
 {
-    return create(bus, AccountFactory::create(bus, Features()), ConnectionFactory::create(bus),
+    return create(bus, AccountFactory::create(bus), ConnectionFactory::create(bus),
             ChannelFactory::create(bus), ContactFactory::create());
 }
 
 /**
  * Create a new client registrar object using QDBusConnection::sessionBus() and the given factories.
  *
- * ClientRegistrar instances are unique per D-Bus connection. The returned ClientRegistrarPtr will
- * point to the same ClientRegistrar instance on successive calls, unless the instance for
- * QDBusConnection::sessionBus() had already been destroyed, in which case a new instance will be
- * returned. Therefore, the factory settings have no effect if there already is a ClientRegistrar
- * for QDBusConnection::sessionBus().
- *
- * \todo API/ABI break: Drop the bus-wide singleton guarantee, it's awkward and the associated name
- * registration checks have always been implemented incorrectly anyway. We need this for the account
- * friendly channel request and handle API at least.
- *
  * \param accountFactory The account factory to use.
  * \param connectionFactory The connection factory to use.
  * \param channelFactory The channel factory to use.
@@ -730,16 +686,6 @@ ClientRegistrarPtr ClientRegistrar::create(
 /**
  * Create a new client registrar object using the given \a bus and the given factories.
  *
- * ClientRegistrar instances are unique per D-Bus connection. The returned
- * ClientRegistrarPtr will point to the same ClientRegistrar instance on
- * successive calls with the same \a bus, unless the instance
- * had already been destroyed, in which case a new instance will be returned. Therefore, the factory
- * settings have no effect if there already is a ClientRegistrar for the given \a bus.
- *
- * \todo API/ABI break: Drop the bus-wide singleton guarantee, it's awkward and the associated name
- * registration checks have always been implemented incorrectly anyway. We need this for the account
- * friendly channel request and handle API at least.
- *
  * \param bus QDBusConnection to use.
  * \param accountFactory The account factory to use.
  * \param connectionFactory The connection factory to use.
@@ -753,12 +699,6 @@ ClientRegistrarPtr ClientRegistrar::create(const QDBusConnection &bus,
             const ChannelFactoryConstPtr &channelFactory,
             const ContactFactoryConstPtr &contactFactory)
 {
-    QPair<QString, QString> busId =
-        qMakePair(bus.name(), bus.baseService());
-    if (registrarForConnection.contains(busId)) {
-        return ClientRegistrarPtr(
-                registrarForConnection.value(busId));
-    }
     return ClientRegistrarPtr(new ClientRegistrar(bus, accountFactory, connectionFactory,
                 channelFactory, contactFactory));
 }
@@ -766,20 +706,10 @@ ClientRegistrarPtr ClientRegistrar::create(const QDBusConnection &bus,
 /**
  * Create a new client registrar object using the bus and factories of the given Account \a manager.
  *
- * ClientRegistrar instances are unique per D-Bus connection. The returned ClientRegistrarPtr will
- * point to the same ClientRegistrar instance on successive calls with the same bus, unless the
- * instance had already been destroyed, in which case a new instance will be returned. Therefore,
- * the factory settings have no effect if there already is a ClientRegistrar for the bus of the
- * given \a manager.
- *
- * Using this create() method will enable (like any other way of passing the same factories to an AM
+ * Using this create method will enable (like any other way of passing the same factories to an AM
  * and a registrar) getting the same Account/Connection etc. proxy instances from both
  * AccountManager and AbstractClient implementations.
  *
- * \todo API/ABI break: Drop the bus-wide singleton guarantee, it's awkward and the associated name
- * registration checks have always been implemented incorrectly anyway. We need this for the account
- * friendly channel request and handle API at least.
- *
  * \param manager The AccountManager the bus and factories of which should be used.
  * \return A ClientRegistrarPtr object pointing to the ClientRegistrar.
  */
@@ -793,23 +723,6 @@ ClientRegistrarPtr ClientRegistrar::create(const AccountManagerPtr &manager)
             manager->connectionFactory(), manager->channelFactory(), manager->contactFactory());
 }
 
-/**
- * Construct a new client registrar object using the given \a bus.
- *
- * The resulting registrar will use factories constructing stock TpQt4 classes with no features
- * prepared. This is equivalent to the old pre-factory behavior of ClientRegistrar.
- *
- * \param bus QDBusConnection to use.
- */
-ClientRegistrar::ClientRegistrar(const QDBusConnection &bus)
-    : mPriv(new Private(bus, AccountFactory::create(bus, Features()),
-                ConnectionFactory::create(bus),   ChannelFactory::create(bus),
-                ContactFactory::create()))
-{
-    registrarForConnection.insert(qMakePair(bus.name(),
-                bus.baseService()), this);
-}
-
 /**
  * Construct a new client registrar object using the given \a bus and the given factories.
  *
@@ -824,10 +737,9 @@ ClientRegistrar::ClientRegistrar(const QDBusConnection &bus,
         const ConnectionFactoryConstPtr &connectionFactory,
         const ChannelFactoryConstPtr &channelFactory,
         const ContactFactoryConstPtr &contactFactory)
-    : mPriv(new Private(bus, accountFactory, connectionFactory, channelFactory, contactFactory))
+    : Object(),
+      mPriv(new Private(bus, accountFactory, connectionFactory, channelFactory, contactFactory))
 {
-    registrarForConnection.insert(qMakePair(bus.name(),
-                bus.baseService()), this);
 }
 
 /**
@@ -835,8 +747,6 @@ ClientRegistrar::ClientRegistrar(const QDBusConnection &bus,
  */
 ClientRegistrar::~ClientRegistrar()
 {
-    registrarForConnection.remove(qMakePair(mPriv->bus.name(),
-                mPriv->bus.baseService()));
     unregisterClients();
     delete mPriv;
 }
