@@ -1,8 +1,8 @@
 /*
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2009 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2009 Nokia Corporation
+ * Copyright (C) 2009-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * Copyright (C) 2009-2010 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -30,27 +30,25 @@
 #include <TelepathyQt4/ChannelFactory>
 #include <TelepathyQt4/ConnectionFactory>
 #include <TelepathyQt4/ContactFactory>
-#include <TelepathyQt4/RefCounted>
+#include <TelepathyQt4/Object>
 #include <TelepathyQt4/Types>
 
 #include <QDBusConnection>
-#include <QObject>
 #include <QString>
 
 namespace Tp
 {
 
-class TELEPATHY_QT4_EXPORT ClientRegistrar : public QObject, public RefCounted
+class TELEPATHY_QT4_EXPORT ClientRegistrar : public Object
 {
     Q_OBJECT
     Q_DISABLE_COPY(ClientRegistrar)
 
 public:
+    static ClientRegistrarPtr create(const QDBusConnection &bus);
     static ClientRegistrarPtr create(
-            const QDBusConnection &bus = QDBusConnection::sessionBus());
-
-    static ClientRegistrarPtr create(
-            const AccountFactoryConstPtr &accountFactory,
+            const AccountFactoryConstPtr &accountFactory =
+                AccountFactory::create(QDBusConnection::sessionBus()),
             const ConnectionFactoryConstPtr &connectionFactory =
                 ConnectionFactory::create(QDBusConnection::sessionBus()),
             const ChannelFactoryConstPtr &channelFactory =
@@ -62,7 +60,6 @@ public:
             const ConnectionFactoryConstPtr &connectionFactory,
             const ChannelFactoryConstPtr &channelFactory,
             const ContactFactoryConstPtr &contactFactory);
-
     static ClientRegistrarPtr create(const AccountManagerPtr &accountManager);
 
     ~ClientRegistrar();
@@ -81,7 +78,6 @@ public:
     void unregisterClients();
 
 private:
-    TELEPATHY_QT4_DEPRECATED ClientRegistrar(const QDBusConnection &bus);
     ClientRegistrar(const QDBusConnection &bus,
             const AccountFactoryConstPtr &accountFactory,
             const ConnectionFactoryConstPtr &connectionFactory,
