@@ -1,8 +1,8 @@
 /*
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2009 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2009 Nokia Corporation
+ * Copyright (C) 2009-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * Copyright (C) 2009-2010 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -35,22 +35,33 @@ namespace Tp
 class TELEPATHY_QT4_EXPORT ConnectionCapabilities : public CapabilitiesBase
 {
 public:
+    ConnectionCapabilities();
     virtual ~ConnectionCapabilities();
 
-    bool supportsTextChatrooms() const;
+    bool textChatrooms() const;
+
+    bool conferenceStreamedMediaCalls() const;
+    bool conferenceStreamedMediaCallsWithInvitees() const;
+    bool conferenceTextChats() const;
+    bool conferenceTextChatsWithInvitees() const;
+    bool conferenceTextChatrooms() const;
+    bool conferenceTextChatroomsWithInvitees() const;
+
+    bool contactSearch();
+    bool contactSearchWithSpecificServer() const;
+    bool contactSearchWithLimit() const;
 
 private:
+    friend class Account;
     friend class Connection;
     friend class ProtocolInfo;
 
-    ConnectionCapabilities();
-    ConnectionCapabilities(const RequestableChannelClassList &classes);
-
-    struct Private;
-    friend struct Private;
-    Private *mPriv;
+    ConnectionCapabilities(const RequestableChannelClassList &rccs);
+    ConnectionCapabilities(const RequestableChannelClassSpecList &rccSpecs);
 };
 
 } // Tp
 
+Q_DECLARE_METATYPE(Tp::ConnectionCapabilities);
+
 #endif
