@@ -36,13 +36,16 @@ class TELEPATHY_QT4_NO_EXPORT Connection::PendingConnect : public PendingReady
     Q_OBJECT
 
 public:
-    PendingConnect(Connection *parent, const Features &requestedFeatures);
+    PendingConnect(const ConnectionPtr &connection, const Features &requestedFeatures);
 
 private Q_SLOTS:
     void onConnectReply(QDBusPendingCallWatcher *);
-    void onStatusChanged(Tp::Connection::Status newStatus);
+    void onStatusChanged(Tp::ConnectionStatus newStatus);
     void onBecomeReadyReply(Tp::PendingOperation *);
     void onConnInvalidated(Tp::DBusProxy *proxy, const QString &error, const QString &message);
+
+private:
+    friend class ConnectionLowlevel;
 };
 
 class ConnectionHelper
