@@ -20,12 +20,14 @@
  */
 
 #include <TelepathyQt4/ConnectionManager>
+#include <TelepathyQt4/ConnectionManagerLowlevel>
 #include "TelepathyQt4/connection-manager-internal.h"
 
 #include "TelepathyQt4/_gen/cli-connection-manager-body.hpp"
 #include "TelepathyQt4/_gen/cli-connection-manager.moc.hpp"
 #include "TelepathyQt4/_gen/connection-manager.moc.hpp"
 #include "TelepathyQt4/_gen/connection-manager-internal.moc.hpp"
+#include "TelepathyQt4/_gen/connection-manager-lowlevel.moc.hpp"
 
 #include "TelepathyQt4/debug-internal.h"
 
@@ -42,315 +44,38 @@
 #include <QStringList>
 #include <QTimer>
 
+namespace
+{
+
+bool checkValidProtocolName(const QString &protocolName)
+{
+    if (!protocolName[0].isLetter()) {
+        return false;
+    }
+
+    int length = protocolName.length();
+    if (length <= 1) {
+        return true;
+    }
+
+    QChar ch;
+    for (int i = 1; i < length; ++i) {
+        ch = protocolName[i];
+        if (!ch.isLetterOrNumber() && ch != QLatin1Char('-')) {
+            return false;
+        }
+    }
+
+    return true;
+}
+
+}
+
 namespace Tp
 {
 
-struct TELEPATHY_QT4_NO_EXPORT ProtocolParameter::Private
-{
-    Private(const QString &name, const QDBusSignature &dbusSignature, QVariant::Type type,
-            const QVariant &defaultValue, ConnMgrParamFlag flags)
-        : name(name), dbusSignature(dbusSignature), type(type), defaultValue(defaultValue),
-          flags(flags) {}
-
-    QString name;
-    QDBusSignature dbusSignature;
-    QVariant::Type type;
-    QVariant defaultValue;
-    ConnMgrParamFlag flags;
-};
-
-ProtocolParameter::ProtocolParameter(const QString &name,
-                                     const QDBusSignature &dbusSignature,
-                                     QVariant defaultValue,
-                                     ConnMgrParamFlag flags)
-    : mPriv(new Private(name, dbusSignature,
-                ManagerFile::variantTypeFromDBusSignature(dbusSignature.signature()), defaultValue,
-                flags))
-{
-}
-
-ProtocolParameter::~ProtocolParameter()
-{
-    delete mPriv;
-}
-
-QString ProtocolParameter::name() const
-{
-    return mPriv->name;
-}
-
-QDBusSignature ProtocolParameter::dbusSignature() const
-{
-    return mPriv->dbusSignature;
-}
-
-QVariant::Type ProtocolParameter::type() const
-{
-    return mPriv->type;
-}
-
-QVariant ProtocolParameter::defaultValue() const
-{
-    return mPriv->defaultValue;
-}
-
-bool ProtocolParameter::isRequired() const
-{
-    return mPriv->flags & ConnMgrParamFlagRequired;
-}
-
-bool ProtocolParameter::isSecret() const
-{
-    return mPriv->flags & ConnMgrParamFlagSecret;
-}
-
-bool ProtocolParameter::isRequiredForRegistration() const
-{
-    return mPriv->flags & ConnMgrParamFlagRegister;
-}
-
-bool ProtocolParameter::operator==(const ProtocolParameter &other) const
-{
-    return (mPriv->name == other.name());
-}
-
-bool ProtocolParameter::operator==(const QString &name) const
-{
-    return (mPriv->name == name);
-}
-
-struct TELEPATHY_QT4_NO_EXPORT ProtocolInfo::Private
-{
-    Private(const QString &cmName, const QString &name)
-        : cmName(cmName),
-          name(name),
-          caps(new ConnectionCapabilities()),
-          iconName(QString(QLatin1String("im-%1")).arg(name))
-    {
-    }
-
-    ~Private()
-    {
-        delete caps;
-    }
-
-    QString cmName;
-    QString name;
-    ProtocolParameterList params;
-    ConnectionCapabilities *caps;
-    QString vcardField;
-    QString englishName;
-    QString iconName;
-};
-
-/**
- * \class ProtocolInfo
- * \ingroup clientcm
- * \headerfile TelepathyQt4/connection-manager.h <TelepathyQt4/ConnectionManager>
- *
- * \brief Object representing a Telepathy protocol info.
- */
-
-/**
- * Construct a new ProtocolInfo object.
- *
- * \param cmName Name of the connection manager.
- * \param name Name of the protocol.
- */
-ProtocolInfo::ProtocolInfo(const QString &cmName, const QString &name)
-    : mPriv(new Private(cmName, name))
-{
-}
-
-/**
- * Class destructor.
- */
-ProtocolInfo::~ProtocolInfo()
-{
-    foreach (ProtocolParameter *param, mPriv->params) {
-        delete param;
-    }
-
-    delete mPriv;
-}
-
-/**
- * Return the short name of the connection manager (e.g. "gabble").
- *
- * \return The name of the connection manager.
- */
-QString ProtocolInfo::cmName() const
-{
-    return mPriv->cmName;
-}
-
-/**
- * Return the string identifying the protocol as described in the Telepathy
- * D-Bus API Specification (e.g. "jabber").
- *
- * This identifier is not intended to be displayed to users directly; user
- * interfaces are responsible for mapping them to localized strings.
- *
- * \return A string identifying the protocol.
- */
-QString ProtocolInfo::name() const
-{
-    return mPriv->name;
-}
-
-/**
- * Return all supported parameters. The parameters' names
- * may either be the well-known strings specified by the Telepathy D-Bus
- * API Specification (e.g. "account" and "password"), or
- * implementation-specific strings.
- *
- * \return A list of parameters.
- */
-const ProtocolParameterList &ProtocolInfo::parameters() const
-{
-    return mPriv->params;
-}
-
-/**
- * Return whether a given parameter can be passed to the connection
- * manager when creating a connection to this protocol.
- *
- * \param name The name of a parameter.
- * \return true if the given parameter exists.
- */
-bool ProtocolInfo::hasParameter(const QString &name) const
-{
-    foreach (ProtocolParameter *param, mPriv->params) {
-        if (param->name() == name) {
-            return true;
-        }
-    }
-    return false;
-}
-
-/**
- * Return whether it might be possible to register new accounts on this
- * protocol, by setting the special parameter named
- * <code>register</code> to <code>true</code>.
- *
- * \return The same thing as hasParameter("register").
- * \sa hasParameter()
- */
-bool ProtocolInfo::canRegister() const
-{
-    return hasParameter(QLatin1String("register"));
-}
-
-/**
- * Return the capabilities that are expected to be available from a connection
- * to this protocol, i.e. those for which Connection::createChannel() can
- * reasonably be expected to succeed.
- * User interfaces can use this information to show or hide UI components.
- *
- * @return An object representing the capabilities expected to be available from
- *         a connection to this protocol.
- */
-ConnectionCapabilities *ProtocolInfo::capabilities() const
-{
-    return mPriv->caps;
-}
-
-/**
- * Return the name of the most common vCard field used for this protocol's
- * contact identifiers, normalized to lower case.
- *
- * One valid use of this field is to answer the question: given a contact's
- * vCard containing an X-JABBER field, how can you communicate with the contact?
- * By iterating through protocols looking for an x-jabber VCardField, one can
- * build up a list of protocols that handle x-jabber, then offer the user a list
- * of accounts for those protocols and/or the option to create a new account for
- * one of those protocols.
- * It is not necessarily valid to interpret contacts' identifiers as values of
- * this vCard field. For instance, telepathy-sofiasip supports contacts whose
- * identifiers are of the form sip:jenny@example.com or tel:8675309, which would
- * not normally both be represented by any single vCard field.
- *
- * \return The most common vCard field used for this protocol's contact
- *         identifiers, or an empty string if there is no such field.
- */
-QString ProtocolInfo::vcardField() const
-{
-    return mPriv->vcardField;
-}
-
-/**
- * Return the English-language name of the protocol, such as "AIM" or "Yahoo!".
- *
- * The name can be used as a fallback if an application doesn't have a localized name for the
- * protocol.
- *
- * If the manager file or the CM service doesn't specify the english name, it is inferred from the
- * protocol name, such that for example "google-talk" becomes "Google Talk", but "local-xmpp"
- * becomes "Local Xmpp".
- *
- * \return An English-language name for the protocol.
- */
-QString ProtocolInfo::englishName() const
-{
-    return mPriv->englishName;
-}
-
-/**
- * Return the name of an icon for the protocol in the system's icon theme, such as "im-msn".
- *
- * If the manager file or the CM service doesn't specify the icon name, "im-<protocolname>" is
- * assumed.
- *
- * \return The likely name of an icon for the given \a protocol.
- */
-QString ProtocolInfo::iconName() const
-{
-    return mPriv->iconName;
-}
-
-void ProtocolInfo::addParameter(const ParamSpec &spec)
-{
-    QVariant defaultValue;
-    if (spec.flags & ConnMgrParamFlagHasDefault) {
-        defaultValue = spec.defaultValue.variant();
-    }
-
-    uint flags = spec.flags;
-    if (spec.name.endsWith(QLatin1String("password"))) {
-        flags |= ConnMgrParamFlagSecret;
-    }
-
-    ProtocolParameter *param = new ProtocolParameter(spec.name,
-            QDBusSignature(spec.signature),
-            defaultValue,
-            (ConnMgrParamFlag) flags);
-
-    mPriv->params.append(param);
-}
-
-void ProtocolInfo::setVCardField(const QString &vcardField)
-{
-    mPriv->vcardField = vcardField;
-}
-
-void ProtocolInfo::setEnglishName(const QString &englishName)
-{
-    mPriv->englishName = englishName;
-}
-
-void ProtocolInfo::setIconName(const QString &iconName)
-{
-    mPriv->iconName = iconName;
-}
-
-void ProtocolInfo::setRequestableChannelClasses(
-        const RequestableChannelClassList &caps)
-{
-    mPriv->caps->updateRequestableChannelClasses(caps);
-}
-
 ConnectionManager::Private::PendingNames::PendingNames(const QDBusConnection &bus)
-    : PendingStringList(),
+    : PendingStringList(SharedPtr<RefCounted>()),
       mBus(bus)
 {
     mMethodsQueue.enqueue(QLatin1String("ListNames"));
@@ -393,8 +118,8 @@ void ConnectionManager::Private::PendingNames::invokeMethod(const QLatin1String
                 method, QList<QVariant>());
     QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(call, this);
     connect(watcher,
-            SIGNAL(finished(QDBusPendingCallWatcher *)),
-            SLOT(onCallFinished(QDBusPendingCallWatcher *)));
+            SIGNAL(finished(QDBusPendingCallWatcher*)),
+            SLOT(onCallFinished(QDBusPendingCallWatcher*)));
 }
 
 void ConnectionManager::Private::PendingNames::parseResult(const QStringList &names)
@@ -414,11 +139,10 @@ ConnectionManager::Private::ProtocolWrapper::ProtocolWrapper(
         const QDBusConnection &bus,
         const QString &busName, const QString &objectPath,
         const QString &cmName, const QString &name, const QVariantMap &props)
-    : StatelessDBusProxy(bus, busName, objectPath),
+    : StatelessDBusProxy(bus, busName, objectPath, FeatureCore),
       OptionalInterfaceFactory<ProtocolWrapper>(this),
-      ReadyObject(this, FeatureCore),
       mReadinessHelper(readinessHelper()),
-      mInfo(new ProtocolInfo(cmName, name)),
+      mInfo(ProtocolInfo(cmName, name)),
       mImmutableProps(props)
 {
     fillRCCs();
@@ -435,7 +159,6 @@ ConnectionManager::Private::ProtocolWrapper::ProtocolWrapper(
     introspectables[FeatureCore] = introspectableCore;
 
     mReadinessHelper->addIntrospectables(introspectables);
-    mReadinessHelper->becomeReady(Features() << FeatureCore);
 }
 
 ConnectionManager::Private::ProtocolWrapper::~ProtocolWrapper()
@@ -450,20 +173,20 @@ void ConnectionManager::Private::ProtocolWrapper::introspectMain(
 
     if (self->receiveProperties(self->mImmutableProps)) {
         debug() << "Got everything we want from the immutable props for" <<
-            self->info()->name();
+            self->info().name();
         self->mReadinessHelper->setIntrospectCompleted(FeatureCore, true);
         return;
     }
 
     debug() << "Not enough immutable properties, calling Properties::GetAll(Protocol) for" <<
-        self->info()->name();
+        self->info().name();
     QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(
             properties->GetAll(
                 QLatin1String(TELEPATHY_INTERFACE_PROTOCOL)),
             self);
     self->connect(watcher,
-            SIGNAL(finished(QDBusPendingCallWatcher *)),
-            SLOT(gotMainProperties(QDBusPendingCallWatcher *)));
+            SIGNAL(finished(QDBusPendingCallWatcher*)),
+            SLOT(gotMainProperties(QDBusPendingCallWatcher*)));
 }
 
 void ConnectionManager::Private::ProtocolWrapper::gotMainProperties(
@@ -552,7 +275,7 @@ void ConnectionManager::Private::ProtocolWrapper::fillRCCs()
 
     // That also settles upgrading calls, because the media classes don't have ImmutableStreams
 
-    mInfo->setRequestableChannelClasses(classes);
+    mInfo.setRequestableChannelClasses(classes);
 }
 
 bool ConnectionManager::Private::ProtocolWrapper::receiveProperties(const QVariantMap &props)
@@ -572,46 +295,61 @@ bool ConnectionManager::Private::ProtocolWrapper::receiveProperties(const QVaria
     ParamSpecList parameters = qdbus_cast<ParamSpecList>(
             props[QLatin1String(TELEPATHY_INTERFACE_PROTOCOL ".Parameters")]);
     foreach (const ParamSpec &spec, parameters) {
-        mInfo->addParameter(spec);
+        mInfo.addParameter(spec);
     }
 
-    mInfo->setVCardField(qdbus_cast<QString>(
+    mInfo.setVCardField(qdbus_cast<QString>(
                 props[QLatin1String(TELEPATHY_INTERFACE_PROTOCOL ".VCardField")]));
     QString englishName = qdbus_cast<QString>(
             props[QLatin1String(TELEPATHY_INTERFACE_PROTOCOL ".EnglishName")]);
     if (englishName.isEmpty()) {
-        QStringList words = mInfo->name().split(QLatin1Char('-'));
+        QStringList words = mInfo.name().split(QLatin1Char('-'));
         for (int i = 0; i < words.size(); ++i) {
             words[i][0] = words[i].at(0).toUpper();
         }
         englishName = words.join(QLatin1String(" "));
     }
-    mInfo->setEnglishName(englishName);
+    mInfo.setEnglishName(englishName);
     QString iconName = qdbus_cast<QString>(
             props[QLatin1String(TELEPATHY_INTERFACE_PROTOCOL ".Icon")]);
     if (iconName.isEmpty()) {
-        iconName = QString(QLatin1String("im-%1")).arg(mInfo->name());
+        iconName = QString(QLatin1String("im-%1")).arg(mInfo.name());
     }
-    mInfo->setIconName(iconName);
+    mInfo.setIconName(iconName);
 
     // Don't overwrite the everything-is-possible RCCs with an empty list if there is no RCCs key
     if (props.contains(QLatin1String(TELEPATHY_INTERFACE_PROTOCOL ".RequestableChannelClasses"))) {
-        mInfo->setRequestableChannelClasses(qdbus_cast<RequestableChannelClassList>(
+        mInfo.setRequestableChannelClasses(qdbus_cast<RequestableChannelClassList>(
                 props[QLatin1String(TELEPATHY_INTERFACE_PROTOCOL ".RequestableChannelClasses")]));
     }
 
     return gotEverything;
 }
 
-ConnectionManager::Private::Private(ConnectionManager *parent, const QString &name)
+ConnectionManager::Private::Private(ConnectionManager *parent, const QString &name,
+        const ConnectionFactoryConstPtr &connFactory,
+        const ChannelFactoryConstPtr &chanFactory,
+        const ContactFactoryConstPtr &contactFactory)
     : parent(parent),
+      lowlevel(ConnectionManagerLowlevelPtr(new ConnectionManagerLowlevel(parent))),
       name(name),
-      baseInterface(new Client::ConnectionManagerInterface(parent->dbusConnection(),
-                    parent->busName(), parent->objectPath(), parent)),
-      readinessHelper(parent->readinessHelper())
+      baseInterface(new Client::ConnectionManagerInterface(parent)),
+      properties(parent->interface<Client::DBus::PropertiesInterface>()),
+      readinessHelper(parent->readinessHelper()),
+      connFactory(connFactory),
+      chanFactory(chanFactory),
+      contactFactory(contactFactory)
 {
     debug() << "Creating new ConnectionManager:" << parent->busName();
 
+    if (connFactory->dbusConnection().name() != parent->dbusConnection().name()) {
+        warning() << "  The D-Bus connection in the connection factory is not the proxy connection";
+    }
+
+    if (chanFactory->dbusConnection().name() != parent->dbusConnection().name()) {
+        warning() << "  The D-Bus connection in the channel factory is not the proxy connection";
+    }
+
     ReadinessHelper::Introspectables introspectables;
 
     // As ConnectionManager does not have predefined statuses let's simulate one (0)
@@ -624,22 +362,10 @@ ConnectionManager::Private::Private(ConnectionManager *parent, const QString &na
     introspectables[FeatureCore] = introspectableCore;
 
     readinessHelper->addIntrospectables(introspectables);
-    readinessHelper->becomeReady(Features() << FeatureCore);
 }
 
 ConnectionManager::Private::~Private()
 {
-    foreach (ProtocolWrapper *wrapper, wrappers) {
-        /* wrapper->info() is not deleted by ProtocolWrapper as we borrow it in
-         * case it gets ready */
-        delete wrapper->info();
-        delete wrapper;
-    }
-
-    foreach (ProtocolInfo *info, protocols) {
-        delete info;
-    }
-
     delete baseInterface;
 }
 
@@ -650,32 +376,20 @@ bool ConnectionManager::Private::parseConfigFile()
         return false;
     }
 
-    foreach (QString protocol, f.protocols()) {
-        ProtocolInfo *info = new ProtocolInfo(name, protocol);
-        protocols.append(info);
+    foreach (const QString &protocol, f.protocols()) {
+        ProtocolInfo info(name, protocol);
 
-        foreach (ParamSpec spec, f.parameters(protocol)) {
-            info->addParameter(spec);
+        foreach (const ParamSpec &spec, f.parameters(protocol)) {
+            info.addParameter(spec);
         }
-        info->setRequestableChannelClasses(
+        info.setRequestableChannelClasses(
                 f.requestableChannelClasses(protocol));
-        info->setVCardField(f.vcardField(protocol));
-        info->setEnglishName(f.englishName(protocol));
-        info->setIconName(f.iconName(protocol));
-    }
+        info.setVCardField(f.vcardField(protocol));
+        info.setEnglishName(f.englishName(protocol));
+        info.setIconName(f.iconName(protocol));
 
-#if 0
-    foreach (ProtocolInfo *info, protocols) {
-        qDebug() << "protocol name   :" << info->name();
-        qDebug() << "protocol cn name:" << info->cmName();
-        foreach (ProtocolParameter *param, info->parameters()) {
-            qDebug() << "\tparam name:       " << param->name();
-            qDebug() << "\tparam is required:" << param->isRequired();
-            qDebug() << "\tparam is secret:  " << param->isSecret();
-            qDebug() << "\tparam value:      " << param->defaultValue();
-        }
+        protocols.append(info);
     }
-#endif
 
     return true;
 }
@@ -690,17 +404,14 @@ void ConnectionManager::Private::introspectMain(ConnectionManager::Private *self
     warning() << "Error parsing config file for connection manager"
         << self->name << "- introspecting";
 
-    Client::DBus::PropertiesInterface *properties = self->parent->propertiesInterface();
-    Q_ASSERT(properties != 0);
-
     debug() << "Calling Properties::GetAll(ConnectionManager)";
     QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(
-            properties->GetAll(
+            self->properties->GetAll(
                 QLatin1String(TELEPATHY_INTERFACE_CONNECTION_MANAGER)),
             self->parent);
     self->parent->connect(watcher,
-            SIGNAL(finished(QDBusPendingCallWatcher *)),
-            SLOT(gotMainProperties(QDBusPendingCallWatcher *)));
+            SIGNAL(finished(QDBusPendingCallWatcher*)),
+            SLOT(gotMainProperties(QDBusPendingCallWatcher*)));
 }
 
 void ConnectionManager::Private::introspectProtocolsLegacy()
@@ -709,8 +420,8 @@ void ConnectionManager::Private::introspectProtocolsLegacy()
     QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(
             baseInterface->ListProtocols(), parent);
     parent->connect(watcher,
-            SIGNAL(finished(QDBusPendingCallWatcher *)),
-            SLOT(gotProtocolsLegacy(QDBusPendingCallWatcher *)));
+            SIGNAL(finished(QDBusPendingCallWatcher*)),
+            SLOT(gotProtocolsLegacy(QDBusPendingCallWatcher*)));
 }
 
 void ConnectionManager::Private::introspectParametersLegacy()
@@ -720,8 +431,8 @@ void ConnectionManager::Private::introspectParametersLegacy()
         QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(
                 baseInterface->GetParameters(protocolName), parent);
         parent->connect(watcher,
-                SIGNAL(finished(QDBusPendingCallWatcher *)),
-                SLOT(gotParametersLegacy(QDBusPendingCallWatcher *)));
+                SIGNAL(finished(QDBusPendingCallWatcher*)),
+                SLOT(gotParametersLegacy(QDBusPendingCallWatcher*)));
     }
 }
 
@@ -737,14 +448,24 @@ QString ConnectionManager::Private::makeObjectPath(const QString &name)
                 TELEPATHY_CONNECTION_MANAGER_OBJECT_PATH_BASE)).append(name);
 }
 
-ProtocolInfo *ConnectionManager::Private::protocol(const QString &protocolName)
+ConnectionManagerLowlevel::ConnectionManagerLowlevel(ConnectionManager *cm)
+    : mPriv(new Private(cm))
 {
-    foreach (ProtocolInfo *info, protocols) {
-        if (info->name() == protocolName) {
-            return info;
-        }
-    }
-    return NULL;
+}
+
+ConnectionManagerLowlevel::~ConnectionManagerLowlevel()
+{
+    delete mPriv;
+}
+
+bool ConnectionManagerLowlevel::isValid() const
+{
+    return !mPriv->cm.isNull();
+}
+
+ConnectionManagerPtr ConnectionManagerLowlevel::connectionManager() const
+{
+    return ConnectionManagerPtr(mPriv->cm);
 }
 
 /**
@@ -778,41 +499,70 @@ const Feature ConnectionManager::FeatureCore = Feature(QLatin1String(ConnectionM
 /**
  * Create a new ConnectionManager object.
  *
- * \param name Name of the connection manager.
- * \return A ConnectionManagerPtr object pointing to the newly created
- *         ConnectionManager object.
- */
-ConnectionManagerPtr ConnectionManager::create(const QString &name)
-{
-    return ConnectionManagerPtr(new ConnectionManager(name));
-}
-
-/**
- * Create a new ConnectionManager object using the given \a bus.
+ * The instance will use a connection factory creating Tp::Connection objects with no features
+ * ready, and a channel factory creating stock Telepathy-Qt4 channel subclasses, as appropriate,
+ * with no features ready.
  *
  * \param bus QDBusConnection to use.
  * \param name Name of the connection manager.
  * \return A ConnectionManagerPtr object pointing to the newly created
  *         ConnectionManager object.
  */
-ConnectionManagerPtr ConnectionManager::create(const QDBusConnection &bus,
-        const QString &name)
+ConnectionManagerPtr ConnectionManager::create(const QDBusConnection &bus, const QString &name)
 {
-    return ConnectionManagerPtr(new ConnectionManager(bus, name));
+    return ConnectionManagerPtr(new ConnectionManager(QDBusConnection::sessionBus(), name,
+                ConnectionFactory::create(bus), ChannelFactory::create(bus),
+                ContactFactory::create()));
 }
 
 /**
- * Construct a new ConnectionManager object.
+ * Create a new ConnectionManager using QDBusConnection::sessionBus() and the given factories.
+ *
+ * The channel factory is passed to any Connection objects created by this manager
+ * object. In fact, they're not used directly by ConnectionManager at all.
+ *
+ * A warning is printed if the factories are for a bus different from QDBusConnection::sessionBus().
  *
  * \param name Name of the connection manager.
+ * \param connectionFactory The connection factory to use.
+ * \param channelFactory The channel factory to use.
+ * \param contactFactory The contact factory to use.
+ * \return A ConnectionManagerPtr object pointing to the newly created
+ *         ConnectionManager object.
  */
-ConnectionManager::ConnectionManager(const QString &name)
-    : StatelessDBusProxy(QDBusConnection::sessionBus(),
-            Private::makeBusName(name), Private::makeObjectPath(name)),
-      OptionalInterfaceFactory<ConnectionManager>(this),
-      ReadyObject(this, FeatureCore),
-      mPriv(new Private(this, name))
+ConnectionManagerPtr ConnectionManager::create(const QString &name,
+        const ConnectionFactoryConstPtr &connectionFactory,
+        const ChannelFactoryConstPtr &channelFactory,
+        const ContactFactoryConstPtr &contactFactory)
 {
+    return ConnectionManagerPtr(new ConnectionManager(QDBusConnection::sessionBus(), name,
+                connectionFactory, channelFactory, contactFactory));
+}
+
+/**
+ * Create a new ConnectionManager using the given \a bus and the given factories.
+ *
+ * The channel factory is passed to any Connection objects created by this manager
+ * object. In fact, they're not used directly by ConnectionManager at all.
+ *
+ * A warning is printed if the factories are for a bus different from QDBusConnection::sessionBus().
+ *
+ * \param bus QDBusConnection to use.
+ * \param name Name of the connection manager.
+ * \param connectionFactory The connection factory to use.
+ * \param channelFactory The channel factory to use.
+ * \param contactFactory The contact factory to use.
+ * \return A ConnectionManagerPtr object pointing to the newly created
+ *         ConnectionManager object.
+ */
+ConnectionManagerPtr ConnectionManager::create(const QDBusConnection &bus,
+        const QString &name,
+        const ConnectionFactoryConstPtr &connectionFactory,
+        const ChannelFactoryConstPtr &channelFactory,
+        const ContactFactoryConstPtr &contactFactory)
+{
+    return ConnectionManagerPtr(new ConnectionManager(bus, name,
+                connectionFactory, channelFactory, contactFactory));
 }
 
 /**
@@ -822,12 +572,14 @@ ConnectionManager::ConnectionManager(const QString &name)
  * \param name Name of the connection manager.
  */
 ConnectionManager::ConnectionManager(const QDBusConnection &bus,
-        const QString &name)
+        const QString &name,
+        const ConnectionFactoryConstPtr &connectionFactory,
+        const ChannelFactoryConstPtr &channelFactory,
+        const ContactFactoryConstPtr &contactFactory)
     : StatelessDBusProxy(bus, Private::makeBusName(name),
-            Private::makeObjectPath(name)),
+            Private::makeObjectPath(name), FeatureCore),
       OptionalInterfaceFactory<ConnectionManager>(this),
-      ReadyObject(this, FeatureCore),
-      mPriv(new Private(this, name))
+      mPriv(new Private(this, name, connectionFactory, channelFactory, contactFactory))
 {
 }
 
@@ -849,6 +601,51 @@ QString ConnectionManager::name() const
     return mPriv->name;
 }
 
+/**
+ * Return the connection factory used by this manager.
+ *
+ * Only read access is provided. This allows constructing object instances and examining the object
+ * construction settings, but not changing settings. Allowing changes would lead to tricky
+ * situations where objects constructed at different times by the manager would have unpredictably
+ * different construction settings (eg. subclass).
+ *
+ * \return Read-only pointer to the connection factory.
+ */
+ConnectionFactoryConstPtr ConnectionManager::connectionFactory() const
+{
+    return mPriv->connFactory;
+}
+
+/**
+ * Return the channel factory used by this manager.
+ *
+ * Only read access is provided. This allows constructing object instances and examining the object
+ * construction settings, but not changing settings. Allowing changes would lead to tricky
+ * situations where objects constructed at different times by the manager would have unpredictably
+ * different construction settings (eg. subclass).
+ *
+ * \return Read-only pointer to the channel factory.
+ */
+ChannelFactoryConstPtr ConnectionManager::channelFactory() const
+{
+    return mPriv->chanFactory;
+}
+
+/**
+ * Return the contact factory used by this manager.
+ *
+ * Only read access is provided. This allows constructing object instances and examining the object
+ * construction settings, but not changing settings. Allowing changes would lead to tricky
+ * situations where objects constructed at different times by the manager would have unpredictably
+ * different construction settings (eg. subclass).
+ *
+ * \return Read-only pointer to the contact factory.
+ */
+ContactFactoryConstPtr ConnectionManager::contactFactory() const
+{
+    return mPriv->contactFactory;
+}
+
 /**
  * Return a list of strings identifying the protocols supported by this
  * connection manager, as described in the Telepathy
@@ -864,8 +661,8 @@ QString ConnectionManager::name() const
 QStringList ConnectionManager::supportedProtocols() const
 {
     QStringList protocols;
-    foreach (const ProtocolInfo *info, mPriv->protocols) {
-        protocols.append(info->name());
+    foreach (const ProtocolInfo &info, mPriv->protocols) {
+        protocols.append(info.name());
     }
     return protocols;
 }
@@ -895,8 +692,8 @@ const ProtocolInfoList &ConnectionManager::protocols() const
  */
 bool ConnectionManager::hasProtocol(const QString &protocolName) const
 {
-    foreach (const ProtocolInfo *info, mPriv->protocols) {
-        if (info->name() == protocolName) {
+    foreach (const ProtocolInfo &info, mPriv->protocols) {
+        if (info.name() == protocolName) {
             return true;
         }
     }
@@ -909,21 +706,19 @@ bool ConnectionManager::hasProtocol(const QString &protocolName) const
  *
  * This method requires ConnectionManager::FeatureCore to be enabled.
  *
- * The returned pointer points to an internal data structure, which should not be deleted by the
- * caller.
- *
- * \return A ProtocolInfo object or 0 if the protocol specified by \a
- *         protocolName is not supported.
+ * \param protocolName The name of the protocol.
+ * \return A ProtocolInfo object which will return \c for ProtocolInfo::isValid() if the protocol
+ *         specified by \a protocolName is not supported.
  * \sa hasProtocol()
  */
-ProtocolInfo *ConnectionManager::protocol(const QString &protocolName) const
+ProtocolInfo ConnectionManager::protocol(const QString &protocolName) const
 {
-    foreach (ProtocolInfo *info, mPriv->protocols) {
-        if (info->name() == protocolName) {
+    foreach (const ProtocolInfo &info, mPriv->protocols) {
+        if (info.name() == protocolName) {
             return info;
         }
     }
-    return 0;
+    return ProtocolInfo();
 }
 
 /**
@@ -938,22 +733,18 @@ ProtocolInfo *ConnectionManager::protocol(const QString &protocolName) const
  * \return A PendingOperation which will emit PendingConnection::finished when
  *         the account has been created of failed its creation process.
  */
-PendingConnection *ConnectionManager::requestConnection(const QString &protocol,
+PendingConnection *ConnectionManagerLowlevel::requestConnection(const QString &protocol,
         const QVariantMap &parameters)
 {
-    return new PendingConnection(ConnectionManagerPtr(this),
+    if (!isValid()) {
+        return new PendingConnection(TP_QT4_ERROR_NOT_AVAILABLE,
+                QLatin1String("The connection manager has been destroyed already"));
+    }
+
+    return new PendingConnection(ConnectionManagerPtr(mPriv->cm),
             protocol, parameters);
 }
 
-/**
- * \fn DBus::propertiesInterface *ConnectionManager::propertiesInterface() const
- *
- * Convenience function for getting a Properties interface proxy. The
- * Properties interface is not necessarily reported by the services, so a
- * <code>check</code> parameter is not provided, and the interface is
- * always assumed to be present.
- */
-
 /**
  * Return a pending operation from which a list of all installed connection
  * manager short names (such as "gabble" or "haze") can be retrieved if it
@@ -968,6 +759,16 @@ PendingStringList *ConnectionManager::listNames(const QDBusConnection &bus)
     return new ConnectionManager::Private::PendingNames(bus);
 }
 
+ConnectionManagerLowlevelPtr ConnectionManager::lowlevel()
+{
+    return mPriv->lowlevel;
+}
+
+ConnectionManagerLowlevelConstPtr ConnectionManager::lowlevel() const
+{
+    return mPriv->lowlevel;
+}
+
 /**
  * Return the Client::ConnectionManagerInterface for this ConnectionManager.
  * This method is protected since the convenience methods provided by this
@@ -1014,15 +815,24 @@ void ConnectionManager::gotMainProperties(QDBusPendingCallWatcher *watcher)
         ProtocolPropertiesMap::const_iterator i = protocolsMap.constBegin();
         ProtocolPropertiesMap::const_iterator end = protocolsMap.constEnd();
         while (i != end) {
+            QString protocolName = i.key();
+            if (!checkValidProtocolName(protocolName)) {
+                warning() << "Protocol has an invalid name" << protocolName << "- ignoring";
+                continue;
+            }
+
+            QString escapedProtocolName = protocolName;
+            escapedProtocolName.replace(QLatin1Char('-'), QLatin1Char('_'));
             QString protocolPath = QString(
-                    QLatin1String("%1/%2")).arg(objectPath()).arg(i.key());
-            Private::ProtocolWrapper *wrapper = new Private::ProtocolWrapper(
-                    dbusConnection(), busName(), protocolPath,
-                    mPriv->name, i.key(), i.value());
+                    QLatin1String("%1/%2")).arg(objectPath()).arg(escapedProtocolName);
+            SharedPtr<Private::ProtocolWrapper> wrapper = SharedPtr<Private::ProtocolWrapper>(
+                    new Private::ProtocolWrapper(
+                        dbusConnection(), busName(), protocolPath,
+                        mPriv->name, protocolName, i.value()));
             connect(wrapper->becomeReady(),
-                    SIGNAL(finished(Tp::PendingOperation *)),
-                    SLOT(onProtocolReady(Tp::PendingOperation *)));
-            mPriv->wrappers.insert(wrapper, wrapper);
+                    SIGNAL(finished(Tp::PendingOperation*)),
+                    SLOT(onProtocolReady(Tp::PendingOperation*)));
+            mPriv->wrappers.insert(wrapper);
             ++i;
         }
     } else {
@@ -1042,8 +852,7 @@ void ConnectionManager::gotProtocolsLegacy(QDBusPendingCallWatcher *watcher)
         protocolsNames = reply.value();
 
         foreach (const QString &protocolName, protocolsNames) {
-            mPriv->protocols.append(new ProtocolInfo(mPriv->name,
-                        protocolName));
+            mPriv->protocols.append(ProtocolInfo(mPriv->name, protocolName));
             mPriv->parametersQueue.enqueue(protocolName);
         }
 
@@ -1065,20 +874,30 @@ void ConnectionManager::gotParametersLegacy(QDBusPendingCallWatcher *watcher)
 {
     QDBusPendingReply<ParamSpecList> reply = *watcher;
     QString protocolName = mPriv->parametersQueue.dequeue();
-    ProtocolInfo *info = mPriv->protocol(protocolName);
+    bool found = false;
+    int pos = 0;
+    foreach (const ProtocolInfo &info, mPriv->protocols) {
+        if (info.name() == protocolName) {
+            found = true;
+            break;
+        }
+        ++pos;
+    }
+    Q_ASSERT(found);
 
     if (!reply.isError()) {
         debug() << QString(QLatin1String("Got reply to ConnectionManager.GetParameters(%1)")).arg(protocolName);
         ParamSpecList parameters = reply.value();
+        ProtocolInfo &info = mPriv->protocols[pos];
         foreach (const ParamSpec &spec, parameters) {
             debug() << "Parameter" << spec.name << "has flags" << spec.flags
                 << "and signature" << spec.signature;
 
-            info->addParameter(spec);
+            info.addParameter(spec);
         }
     } else {
         // let's remove this protocol as we can't get the params
-        mPriv->protocols.removeOne(info);
+        mPriv->protocols.removeAt(pos);
 
         warning().nospace() <<
             QString(QLatin1String("ConnectionManager.GetParameters(%1) failed: ")).arg(protocolName) <<
@@ -1092,19 +911,6 @@ void ConnectionManager::gotParametersLegacy(QDBusPendingCallWatcher *watcher)
             // we could not retrieve the params for any protocol, fail core.
             mPriv->readinessHelper->setIntrospectCompleted(FeatureCore, false, reply.error());
         }
-
-#if 0
-        foreach (ProtocolInfo *info, mPriv->protocols) {
-            qDebug() << "protocol name   :" << info->name();
-            qDebug() << "protocol cn name:" << info->cmName();
-            foreach (ProtocolParameter *param, info->parameters()) {
-                qDebug() << "\tparam name:       " << param->name();
-                qDebug() << "\tparam is required:" << param->isRequired();
-                qDebug() << "\tparam is secret:  " << param->isSecret();
-                qDebug() << "\tparam value:      " << param->defaultValue();
-            }
-        }
-#endif
     }
 
     watcher->deleteLater();
@@ -1113,23 +919,19 @@ void ConnectionManager::gotParametersLegacy(QDBusPendingCallWatcher *watcher)
 void ConnectionManager::onProtocolReady(Tp::PendingOperation *op)
 {
     PendingReady *pr = qobject_cast<PendingReady*>(op);
-    Private::ProtocolWrapper *wrapper =
-        qobject_cast<Private::ProtocolWrapper*>(pr->object());
-    ProtocolInfo *info = wrapper->info();
+    SharedPtr<Private::ProtocolWrapper> wrapper =
+        SharedPtr<Private::ProtocolWrapper>::qObjectCast(pr->proxy());
+    ProtocolInfo info = wrapper->info();
 
     mPriv->wrappers.remove(wrapper);
 
     if (!op->isError()) {
         mPriv->protocols.append(info);
     } else {
-        warning().nospace() << "Protocol(" << info->name() << ")::becomeReady "
+        warning().nospace() << "Protocol(" << info.name() << ")::becomeReady "
             "failed: " << op->errorName() << ": " << op->errorMessage();
-        delete info;
     }
 
-    /* use deleteLater as "op" is a child of wrapper, so we don't crash */
-    wrapper->deleteLater();
-
     if (mPriv->wrappers.isEmpty()) {
         if (!mPriv->protocols.isEmpty()) {
             mPriv->readinessHelper->setIntrospectCompleted(FeatureCore, true);
