@@ -1,8 +1,8 @@
 /*
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008, 2009 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008, 2009 Nokia Corporation
+ * Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * Copyright (C) 2008-2010 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -28,12 +28,12 @@
 
 #include <TelepathyQt4/_gen/cli-connection.h>
 
+#include <TelepathyQt4/ConnectionCapabilities>
 #include <TelepathyQt4/Contact>
 #include <TelepathyQt4/DBus>
 #include <TelepathyQt4/DBusProxy>
 #include <TelepathyQt4/OptionalInterfaceFactory>
 #include <TelepathyQt4/ReadinessHelper>
-#include <TelepathyQt4/ReadyObject>
 #include <TelepathyQt4/Types>
 #include <TelepathyQt4/SharedPtr>
 
@@ -49,6 +49,7 @@ namespace Tp
 
 class Channel;
 class ConnectionCapabilities;
+class ConnectionLowlevel;
 class Contact;
 class ContactManager;
 class PendingChannel;
@@ -58,13 +59,10 @@ class PendingOperation;
 class PendingReady;
 
 class TELEPATHY_QT4_EXPORT Connection : public StatefulDBusProxy,
-                   public OptionalInterfaceFactory<Connection>,
-                   public ReadyObject,
-                   public RefCounted
+                   public OptionalInterfaceFactory<Connection>
 {
     Q_OBJECT
     Q_DISABLE_COPY(Connection)
-    Q_ENUMS(Status)
 
 public:
     static const Feature FeatureCore;
@@ -74,24 +72,24 @@ public:
     static const Feature FeatureRosterGroups;
     static const Feature FeatureAccountBalance; // TODO unit tests for this
 
-    enum Status {
-        StatusDisconnected = ConnectionStatusDisconnected,
-        StatusConnecting = ConnectionStatusConnecting,
-        StatusConnected = ConnectionStatusConnected,
-        StatusUnknown = 0xFFFFFFFF
-    };
-
     static ConnectionPtr create(const QString &busName,
-            const QString &objectPath);
+            const QString &objectPath,
+            const ChannelFactoryConstPtr &channelFactory,
+            const ContactFactoryConstPtr &contactFactory);
     static ConnectionPtr create(const QDBusConnection &bus,
-            const QString &busName, const QString &objectPath);
+            const QString &busName, const QString &objectPath,
+            const ChannelFactoryConstPtr &channelFactory,
+            const ContactFactoryConstPtr &contactFactory);
 
     virtual ~Connection();
 
+    ChannelFactoryConstPtr channelFactory() const;
+    ContactFactoryConstPtr contactFactory() const;
+
     QString cmName() const;
     QString protocolName() const;
 
-    Status status() const;
+    ConnectionStatus status() const;
     ConnectionStatusReason statusReason() const;
 
     class ErrorDetails
@@ -157,114 +155,20 @@ public:
     uint selfHandle() const;
     ContactPtr selfContact() const;
 
-    SimpleStatusSpecMap allowedPresenceStatuses() const;
-    PendingOperation *setSelfPresence(const QString &status, const QString &statusMessage);
-
     CurrencyAmount accountBalance() const;
 
-    ConnectionCapabilities *capabilities() const;
+    ConnectionCapabilities capabilities() const;
 
-    PendingChannel *createChannel(const QVariantMap &request);
-    PendingChannel *ensureChannel(const QVariantMap &request);
+    ContactManagerPtr contactManager() const;
 
-    PendingReady *requestConnect(const Features &requestedFeatures = Features());
-    PendingOperation *requestDisconnect();
-
-    PendingHandles *requestHandles(uint handleType, const QStringList &names);
-    PendingHandles *referenceHandles(uint handleType, const UIntList &handles);
-
-    PendingContactAttributes *contactAttributes(const UIntList &handles,
-            const QStringList &interfaces, bool reference = true);
-    QStringList contactAttributeInterfaces() const;
-    ContactManager *contactManager() const;
-
-    inline Client::DBus::PropertiesInterface *propertiesInterface() const
-    {
-        return optionalInterface<Client::DBus::PropertiesInterface>(BypassInterfaceCheck);
-    }
-
-    inline Client::ConnectionInterfaceAliasingInterface *aliasingInterface(
-            InterfaceSupportedChecking check = CheckInterfaceSupported) const
-    {
-        return optionalInterface<Client::ConnectionInterfaceAliasingInterface>(check);
-    }
-
-    inline Client::ConnectionInterfaceAnonymityInterface *anonymityInterface(
-            InterfaceSupportedChecking check = CheckInterfaceSupported) const
-    {
-        return optionalInterface<Client::ConnectionInterfaceAnonymityInterface>(check);
-    }
-
-    inline Client::ConnectionInterfaceAvatarsInterface *avatarsInterface(
-            InterfaceSupportedChecking check = CheckInterfaceSupported) const
-    {
-        return optionalInterface<Client::ConnectionInterfaceAvatarsInterface>(check);
-    }
-
-    inline Client::ConnectionInterfaceCapabilitiesInterface *capabilitiesInterface(
-            InterfaceSupportedChecking check = CheckInterfaceSupported) const
-    {
-        return optionalInterface<Client::ConnectionInterfaceCapabilitiesInterface>(check);
-    }
-
-    inline Client::ConnectionInterfaceContactCapabilitiesInterface *contactCapabilitiesInterface(
-            InterfaceSupportedChecking check = CheckInterfaceSupported) const
-    {
-        return optionalInterface<Client::ConnectionInterfaceContactCapabilitiesInterface>(check);
-    }
-
-    inline Client::ConnectionInterfaceLocationInterface *locationInterface(
-            InterfaceSupportedChecking check = CheckInterfaceSupported) const
-    {
-        return optionalInterface<Client::ConnectionInterfaceLocationInterface>(check);
-    }
-
-    inline Client::ConnectionInterfaceContactInfoInterface *contactInfoInterface(
-            InterfaceSupportedChecking check = CheckInterfaceSupported) const
-    {
-        return optionalInterface<Client::ConnectionInterfaceContactInfoInterface>(check);
-    }
-
-    inline Client::ConnectionInterfacePresenceInterface *presenceInterface(
-            InterfaceSupportedChecking check = CheckInterfaceSupported) const
-    {
-        return optionalInterface<Client::ConnectionInterfacePresenceInterface>(check);
-    }
-
-    inline Client::ConnectionInterfaceServicePointInterface *servicePointInterface(
-            InterfaceSupportedChecking check = CheckInterfaceSupported) const
-    {
-        return optionalInterface<Client::ConnectionInterfaceServicePointInterface>(check);
-    }
-
-    inline Client::ConnectionInterfaceSimplePresenceInterface *simplePresenceInterface(
-            InterfaceSupportedChecking check = CheckInterfaceSupported) const
-    {
-        return optionalInterface<Client::ConnectionInterfaceSimplePresenceInterface>(check);
-    }
-
-    inline Client::ConnectionInterfaceRequestsInterface *requestsInterface(
-            InterfaceSupportedChecking check = CheckInterfaceSupported) const
-    {
-        return optionalInterface<Client::ConnectionInterfaceRequestsInterface>(check);
-    }
-
-    inline Client::ConnectionInterfaceBalanceInterface *balanceInterface(
-            InterfaceSupportedChecking check = CheckInterfaceSupported) const
-    {
-        return optionalInterface<Client::ConnectionInterfaceBalanceInterface>(check);
-    }
-
-    inline Client::ConnectionInterfaceCellularInterface *cellularInterface(
-            InterfaceSupportedChecking check = CheckInterfaceSupported) const
-    {
-        return optionalInterface<Client::ConnectionInterfaceCellularInterface>(check);
-    }
+#if defined(BUILDING_TELEPATHY_QT4) || defined(TP_QT4_ENABLE_LOWLEVEL_API)
+    ConnectionLowlevelPtr lowlevel();
+    ConnectionLowlevelConstPtr lowlevel() const;
+#endif
 
 Q_SIGNALS:
-    void statusChanged(Tp::Connection::Status newStatus);
-    TELEPATHY_QT4_DEPRECATED void statusChanged(Tp::Connection::Status newStatus,
-            Tp::ConnectionStatusReason newStatusReason);
+    void statusChanged(Tp::ConnectionStatus newStatus);
+
     void selfHandleChanged(uint newHandle);
     // FIXME: might not need this when Renaming is fixed and mapped to Contacts
     void selfContactChanged();
@@ -272,9 +176,11 @@ Q_SIGNALS:
     void accountBalanceChanged(const Tp::CurrencyAmount &accountBalance);
 
 protected:
-    Connection(const QString &busName, const QString &objectPath);
     Connection(const QDBusConnection &bus, const QString &busName,
-            const QString &objectPath);
+            const QString &objectPath,
+            const ChannelFactoryConstPtr &channelFactory,
+            const ContactFactoryConstPtr &contactFactory,
+            const Feature &coreFeature);
 
     Client::ConnectionInterface *baseInterface() const;
 
@@ -290,14 +196,11 @@ private Q_SLOTS:
     void gotContactAttributeInterfaces(QDBusPendingCallWatcher *watcher);
     void gotSimpleStatuses(QDBusPendingCallWatcher *watcher);
     void gotSelfContact(Tp::PendingOperation *op);
-    void gotContactListsHandles(Tp::PendingOperation *op);
-    void gotContactListChannel(Tp::PendingOperation *op);
-    void contactListChannelReady();
-    void onNewChannels(const Tp::ChannelDetailsList &channelDetailsList);
-    void onContactListGroupChannelReady(Tp::PendingOperation *op);
-    void gotChannels(QDBusPendingCallWatcher *watcher);
 
-    void doReleaseSweep(uint type);
+    void onIntrospectRosterFinished(Tp::PendingOperation *op);
+    void onIntrospectRosterGroupsFinished(Tp::PendingOperation *op);
+
+    void doReleaseSweep(uint handleType);
 
     void onSelfHandleChanged(uint);
 
@@ -306,6 +209,7 @@ private Q_SLOTS:
 
 private:
     class PendingConnect;
+    friend class ConnectionLowlevel;
     friend class PendingChannel;
     friend class PendingConnect;
     friend class PendingContactAttributes;
@@ -313,9 +217,9 @@ private:
     friend class PendingHandles;
     friend class ReferencedHandles;
 
-    void refHandle(uint type, uint handle);
-    void unrefHandle(uint type, uint handle);
-    void handleRequestLanded(uint type);
+    void refHandle(HandleType handleType, uint handle);
+    void unrefHandle(HandleType handleType, uint handle);
+    void handleRequestLanded(HandleType handleType);
 
     struct Private;
     friend struct Private;
@@ -324,4 +228,6 @@ private:
 
 } // Tp
 
+Q_DECLARE_METATYPE(Tp::Connection::ErrorDetails);
+
 #endif
