@@ -70,8 +70,8 @@ struct TELEPATHY_QT4_NO_EXPORT FileTransferChannel::Private
 
 FileTransferChannel::Private::Private(FileTransferChannel *parent)
     : parent(parent),
-      fileTransferInterface(parent->fileTransferInterface(BypassInterfaceCheck)),
-      properties(0),
+      fileTransferInterface(parent->interface<Client::ChannelTypeFileTransferInterface>()),
+      properties(parent->interface<Client::DBus::PropertiesInterface>()),
       readinessHelper(parent->readinessHelper()),
       pendingState(FileTransferStateNone),
       pendingStateReason(FileTransferStateChangeReasonNone),
@@ -88,8 +88,8 @@ FileTransferChannel::Private::Private(FileTransferChannel *parent)
             SIGNAL(InitialOffsetDefined(qulonglong)),
             SLOT(onInitialOffsetDefined(qulonglong)));
     parent->connect(fileTransferInterface,
-            SIGNAL(FileTransferStateChanged(uint, uint)),
-            SLOT(onStateChanged(uint, uint)));
+            SIGNAL(FileTransferStateChanged(uint,uint)),
+            SLOT(onStateChanged(uint,uint)));
     parent->connect(fileTransferInterface,
             SIGNAL(TransferredBytesChanged(qulonglong)),
             SLOT(onTransferredBytesChanged(qulonglong)));
@@ -114,19 +114,14 @@ FileTransferChannel::Private::~Private()
 void FileTransferChannel::Private::introspectProperties(
         FileTransferChannel::Private *self)
 {
-    if (!self->properties) {
-        self->properties = self->parent->propertiesInterface();
-        Q_ASSERT(self->properties != 0);
-    }
-
     QDBusPendingCallWatcher *watcher =
         new QDBusPendingCallWatcher(
                 self->properties->GetAll(
                     QLatin1String(TELEPATHY_INTERFACE_CHANNEL_TYPE_FILE_TRANSFER)),
                 self->parent);
     self->parent->connect(watcher,
-            SIGNAL(finished(QDBusPendingCallWatcher *)),
-            SLOT(gotProperties(QDBusPendingCallWatcher *)));
+            SIGNAL(finished(QDBusPendingCallWatcher*)),
+            SLOT(gotProperties(QDBusPendingCallWatcher*)));
 }
 
 void FileTransferChannel::Private::extractProperties(const QVariantMap &props)
@@ -183,7 +178,7 @@ FileTransferChannelPtr FileTransferChannel::create(const ConnectionPtr &connecti
         const QString &objectPath, const QVariantMap &immutableProperties)
 {
     return FileTransferChannelPtr(new FileTransferChannel(connection, objectPath,
-                immutableProperties));
+                immutableProperties, FileTransferChannel::FeatureCore));
 }
 
 /**
@@ -193,11 +188,14 @@ FileTransferChannelPtr FileTransferChannel::create(const ConnectionPtr &connecti
  * \param connection Connection owning this channel, and specifying the service.
  * \param objectPath Path to the object on the service.
  * \param immutableProperties The immutable properties of the channel.
+ * \param coreFeature The core feature of the channel type. The corresponding introspectable should
+ * depend on FileTransferChannel::FeatureCore.
  */
 FileTransferChannel::FileTransferChannel(const ConnectionPtr &connection,
         const QString &objectPath,
-        const QVariantMap &immutableProperties)
-    : Channel(connection, objectPath, immutableProperties),
+        const QVariantMap &immutableProperties,
+        const Feature &coreFeature)
+    : Channel(connection, objectPath, immutableProperties, coreFeature),
       mPriv(new Private(this))
 {
 }
