@@ -45,6 +45,8 @@ struct TELEPATHY_QT4_NO_EXPORT IncomingFileTransferChannel::Private
     // Public object
     IncomingFileTransferChannel *parent;
 
+    Client::ChannelTypeFileTransferInterface *fileTransferInterface;
+
     QIODevice *output;
     QTcpSocket *socket;
     SocketAddressIPv4 addr;
@@ -55,6 +57,7 @@ struct TELEPATHY_QT4_NO_EXPORT IncomingFileTransferChannel::Private
 
 IncomingFileTransferChannel::Private::Private(IncomingFileTransferChannel *parent)
     : parent(parent),
+      fileTransferInterface(parent->interface<Client::ChannelTypeFileTransferInterface>()),
       output(0),
       socket(0),
       requestedOffset(0),
@@ -79,14 +82,12 @@ IncomingFileTransferChannel::Private::~Private()
  * Feature representing the core that needs to become ready to make the
  * IncomingFileTransferChannel object usable.
  *
- * Note that this feature must be enabled in order to use most
- * IncomingFileTransferChannel methods.
- * See specific methods documentation for more details.
+ * This is currently the same as FileTransferChannel::FeatureCore, but may change to include more.
  *
  * When calling isReady(), becomeReady(), this feature is implicitly added
  * to the requested features.
  */
-const Feature IncomingFileTransferChannel::FeatureCore = Feature(QLatin1String(IncomingFileTransferChannel::staticMetaObject.className()), 0);
+const Feature IncomingFileTransferChannel::FeatureCore = Feature(QLatin1String(FileTransferChannel::staticMetaObject.className()), 0);
 
 /**
  * Create a new InconmingFileTransferChannel object.
@@ -113,11 +114,14 @@ IncomingFileTransferChannelPtr IncomingFileTransferChannel::create(
  *                   service.
  * \param objectPath The object path of this channel.
  * \param immutableProperties The immutable properties of this channel.
+ * \param coreFeature The core feature of the channel type, if any. The corresponding introspectable should
+ * depend on IncomingFileTransferChannel::FeatureCore.
  */
 IncomingFileTransferChannel::IncomingFileTransferChannel(
         const ConnectionPtr &connection, const QString &objectPath,
-        const QVariantMap &immutableProperties)
-    : FileTransferChannel(connection, objectPath, immutableProperties),
+        const QVariantMap &immutableProperties,
+        const Feature &coreFeature)
+    : FileTransferChannel(connection, objectPath, immutableProperties, coreFeature),
       mPriv(new Private(this))
 {
 }
@@ -166,7 +170,8 @@ PendingOperation *IncomingFileTransferChannel::acceptFile(qulonglong offset,
         warning() << "FileTransferChannel::FeatureCore must be ready before "
             "calling acceptFile";
         return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
-                QLatin1String("Channel not ready"), this);
+                QLatin1String("Channel not ready"),
+                IncomingFileTransferChannelPtr(this));
     }
 
     // let's fail here direclty as we may only have one device to handle
@@ -175,14 +180,15 @@ PendingOperation *IncomingFileTransferChannel::acceptFile(qulonglong offset,
             "channel";
         return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
                 QLatin1String("File transfer can only be started once in the same channel"),
-                this);
+                IncomingFileTransferChannelPtr(this));
     }
 
     if ((!output->isOpen() && !output->open(QIODevice::WriteOnly)) &&
         (!output->isWritable())) {
         warning() << "Unable to open IO device for writing";
         return new PendingFailure(QLatin1String(TELEPATHY_ERROR_PERMISSION_DENIED),
-                QLatin1String("Unable to open IO device for writing"), this);
+                QLatin1String("Unable to open IO device for writing"),
+                IncomingFileTransferChannelPtr(this));
     }
 
     mPriv->output = output;
@@ -190,10 +196,10 @@ PendingOperation *IncomingFileTransferChannel::acceptFile(qulonglong offset,
     mPriv->requestedOffset = offset;
 
     PendingVariant *pv = new PendingVariant(
-            fileTransferInterface(BypassInterfaceCheck)->AcceptFile(SocketAddressTypeIPv4,
+            mPriv->fileTransferInterface->AcceptFile(SocketAddressTypeIPv4,
                 SocketAccessControlLocalhost, QDBusVariant(QVariant(QString())),
                 offset),
-            this);
+            IncomingFileTransferChannelPtr(this));
     connect(pv,
             SIGNAL(finished(Tp::PendingOperation*)),
             SLOT(onAcceptFileFinished(Tp::PendingOperation*)));
