@@ -47,6 +47,8 @@ struct TELEPATHY_QT4_NO_EXPORT OutgoingFileTransferChannel::Private
     // Public object
     OutgoingFileTransferChannel *parent;
 
+    Client::ChannelTypeFileTransferInterface *fileTransferInterface;
+
     // Introspection
     QIODevice *input;
     QTcpSocket *socket;
@@ -57,6 +59,7 @@ struct TELEPATHY_QT4_NO_EXPORT OutgoingFileTransferChannel::Private
 
 OutgoingFileTransferChannel::Private::Private(OutgoingFileTransferChannel *parent)
     : parent(parent),
+      fileTransferInterface(parent->interface<Client::ChannelTypeFileTransferInterface>()),
       input(0),
       socket(0),
       pos(0)
@@ -80,14 +83,12 @@ OutgoingFileTransferChannel::Private::~Private()
  * Feature representing the core that needs to become ready to make the
  * OutgoingFileTransferChannel object usable.
  *
- * Note that this feature must be enabled in order to use most
- * OutgoingFileTransferChannel methods.
- * See specific methods documentation for more details.
+ * This is currently the same as FileTransferChannel::FeatureCore, but may change to include more.
  *
  * When calling isReady(), becomeReady(), this feature is implicitly added
  * to the requested features.
  */
-const Feature OutgoingFileTransferChannel::FeatureCore = Feature(QLatin1String(OutgoingFileTransferChannel::staticMetaObject.className()), 0);
+const Feature OutgoingFileTransferChannel::FeatureCore = Feature(QLatin1String(FileTransferChannel::staticMetaObject.className()), 0);
 
 /**
  * Create a new OutgoingFileTransferChannel object.
@@ -96,8 +97,8 @@ const Feature OutgoingFileTransferChannel::FeatureCore = Feature(QLatin1String(O
  *                   service.
  * \param objectPath The object path of this channel.
  * \param immutableProperties The immutable properties of this channel.
- * \return A StreamedMediaChannelPtr object pointing to the newly created
- *         StreamedMediaChannel object.
+ * \return An OutgoingFileTransferChannelPtr object pointing to the newly created
+ *         OutgoingFileTransferChannel object.
  */
 OutgoingFileTransferChannelPtr OutgoingFileTransferChannel::create(const ConnectionPtr &connection,
         const QString &objectPath, const QVariantMap &immutableProperties)
@@ -113,12 +114,15 @@ OutgoingFileTransferChannelPtr OutgoingFileTransferChannel::create(const Connect
  * \param connection Connection owning this channel, and specifying the service.
  * \param objectPath Path to the object on the service.
  * \param immutableProperties The immutable properties of the channel.
+ * \param coreFeature The core feature of the channel type, if any. The corresponding introspectable should
+ * depend on OutgoingFileTransferChannel::FeatureCore.
  */
 OutgoingFileTransferChannel::OutgoingFileTransferChannel(
         const ConnectionPtr &connection,
         const QString &objectPath,
-        const QVariantMap &immutableProperties)
-    : FileTransferChannel(connection, objectPath, immutableProperties),
+        const QVariantMap &immutableProperties,
+        const Feature &coreFeature)
+    : FileTransferChannel(connection, objectPath, immutableProperties, coreFeature),
       mPriv(new Private(this))
 {
 }
@@ -155,7 +159,8 @@ PendingOperation *OutgoingFileTransferChannel::provideFile(QIODevice *input)
         warning() << "FileTransferChannel::FeatureCore must be ready before "
             "calling provideFile";
         return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
-                QLatin1String("Channel not ready"), this);
+                QLatin1String("Channel not ready"),
+                OutgoingFileTransferChannelPtr(this));
     }
 
     // let's fail here direclty as we may only have one device to handle
@@ -164,14 +169,15 @@ PendingOperation *OutgoingFileTransferChannel::provideFile(QIODevice *input)
             "channel";
         return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
                 QLatin1String("File transfer can only be started once in the same channel"),
-                this);
+                OutgoingFileTransferChannelPtr(this));
     }
 
     if ((!input->isOpen() && !input->open(QIODevice::ReadOnly)) &&
         !input->isReadable()) {
         warning() << "Unable to open IO device for reading";
         return new PendingFailure(QLatin1String(TELEPATHY_ERROR_PERMISSION_DENIED),
-                QLatin1String("Unable to open IO device for reading"), this);
+                QLatin1String("Unable to open IO device for reading"),
+                OutgoingFileTransferChannelPtr(this));
     }
 
     mPriv->input = input;
@@ -180,11 +186,11 @@ PendingOperation *OutgoingFileTransferChannel::provideFile(QIODevice *input)
             SLOT(onInputAboutToClose()));
 
     PendingVariant *pv = new PendingVariant(
-            fileTransferInterface(BypassInterfaceCheck)->ProvideFile(
+            mPriv->fileTransferInterface->ProvideFile(
                 SocketAddressTypeIPv4,
                 SocketAccessControlLocalhost,
                 QDBusVariant(QVariant(QString()))),
-            this);
+            OutgoingFileTransferChannelPtr(this));
     connect(pv,
             SIGNAL(finished(Tp::PendingOperation*)),
             SLOT(onProvideFileFinished(Tp::PendingOperation*)));
