@@ -37,14 +37,7 @@ namespace Tp
 
 struct TELEPATHY_QT4_NO_EXPORT PendingAccount::Private
 {
-    Private(const AccountManagerPtr &manager) :
-        manager(manager)
-    {
-    }
-
-    AccountManagerPtr manager;
     AccountPtr account;
-    QDBusObjectPath objectPath;
 };
 
 /**
@@ -78,15 +71,15 @@ PendingAccount::PendingAccount(const AccountManagerPtr &manager,
         const QString &connectionManager, const QString &protocol,
         const QString &displayName, const QVariantMap &parameters,
         const QVariantMap &properties)
-    : PendingOperation(0),
-      mPriv(new Private(manager))
+    : PendingOperation(manager),
+      mPriv(new Private)
 {
     QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(
             manager->baseInterface()->CreateAccount(connectionManager,
                 protocol, displayName, parameters, properties), this);
     connect(watcher,
-            SIGNAL(finished(QDBusPendingCallWatcher *)),
-            SLOT(onCallFinished(QDBusPendingCallWatcher *)));
+            SIGNAL(finished(QDBusPendingCallWatcher*)),
+            SLOT(onCallFinished(QDBusPendingCallWatcher*)));
 }
 
 /**
@@ -104,7 +97,7 @@ PendingAccount::~PendingAccount()
  */
 AccountManagerPtr PendingAccount::manager() const
 {
-    return mPriv->manager;
+    return AccountManagerPtr(qobject_cast<AccountManager *>((AccountManager*) object().data()));
 }
 
 /**
@@ -127,39 +120,20 @@ AccountPtr PendingAccount::account() const
     return mPriv->account;
 }
 
-/**
- * Returns the account object path.
- *
- * This method is useful for creating custom Account objects, so instead of
- * using PendingAccount::account, one could construct a new custom account with
- * the object path.
- *
- * \return Account object path or an empty string an error occurred.
- */
-QString PendingAccount::objectPath() const
-{
-    if (!isFinished()) {
-        warning() << "PendingAccount::objectPath() called before finished";
-    } else if (!isValid()) {
-        warning() << "PendingAccount::objectPath() called when not valid";
-    }
-
-    return mPriv->objectPath.path();
-}
-
 void PendingAccount::onCallFinished(QDBusPendingCallWatcher *watcher)
 {
     QDBusPendingReply<QDBusObjectPath> reply = *watcher;
 
     if (!reply.isError()) {
-        mPriv->objectPath = reply.value();
-        debug() << "Got reply to AccountManager.CreateAccount - object path:" <<
-            mPriv->objectPath.path();
-        PendingReady *proxyOp = manager()->accountFactory()->proxy(manager()->busName(),
-                mPriv->objectPath.path(), manager()->connectionFactory(),
+        QString objectPath = reply.value().path();
+
+        debug() << "Got reply to AccountManager.CreateAccount - object path:" << objectPath;
+
+        PendingReady *readyOp = manager()->accountFactory()->proxy(manager()->busName(),
+                objectPath, manager()->connectionFactory(),
                 manager()->channelFactory(), manager()->contactFactory());
-        mPriv->account = AccountPtr::dynamicCast(proxyOp->proxy());
-        connect(proxyOp,
+        mPriv->account = AccountPtr::qObjectCast(readyOp->proxy());
+        connect(readyOp,
                 SIGNAL(finished(Tp::PendingOperation*)),
                 SLOT(onAccountBuilt(Tp::PendingOperation*)));
     } else {
@@ -183,15 +157,15 @@ void PendingAccount::onAccountBuilt(Tp::PendingOperation *op)
     } else {
         // AM is stateless, so the only way for it to become invalid is in the introspection phase,
         // and a PendingAccount should never be created if AM introspection hasn't succeeded
-        Q_ASSERT(!mPriv->manager.isNull() && mPriv->manager->isValid());
+        Q_ASSERT(!manager().isNull() && manager()->isValid());
 
-        if (mPriv->manager->allAccounts().contains(mPriv->account)) {
+        if (manager()->allAccounts().contains(mPriv->account)) {
             setFinished();
-            debug() << "New account" << objectPath() << "built";
+            debug() << "New account" << mPriv->account->objectPath() << "built";
         } else {
             // Have to wait for the AM to pick up the change and signal it so the world can be
             // assumed to be ~round when we finish
-            connect(mPriv->manager.data(),
+            connect(manager().data(),
                     SIGNAL(newAccount(Tp::AccountPtr)),
                     SLOT(onNewAccount(Tp::AccountPtr)));
         }
@@ -204,7 +178,7 @@ void PendingAccount::onNewAccount(const AccountPtr &account)
         return;
     }
 
-    debug() << "Account" << objectPath() << "added to AM, finishing PendingAccount";
+    debug() << "Account" << account->objectPath() << "added to AM, finishing PendingAccount";
     setFinished();
 }
 
