@@ -1,8 +1,8 @@
 /*
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008 Nokia Corporation
+ * Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * Copyright (C) 2008-2010 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,29 +23,24 @@
 
 #include "TelepathyQt4/_gen/pending-channel.moc.hpp"
 
-#include "TelepathyQt4/channel-factory.h"
 #include "TelepathyQt4/debug-internal.h"
 
 #include <TelepathyQt4/Channel>
+#include <TelepathyQt4/ChannelFactory>
 #include <TelepathyQt4/Connection>
+#include <TelepathyQt4/ConnectionLowlevel>
 #include <TelepathyQt4/Constants>
+#include <TelepathyQt4/PendingReady>
 
 namespace Tp
 {
 
 struct TELEPATHY_QT4_NO_EXPORT PendingChannel::Private
 {
-    Private(const ConnectionPtr &connection) :
-        connection(connection)
-    {
-    }
-
-    ConnectionPtr connection;
     bool yours;
     QString channelType;
     uint handleType;
     uint handle;
-    QDBusObjectPath objectPath;
     QVariantMap immutableProperties;
     ChannelPtr channel;
 };
@@ -69,8 +64,8 @@ struct TELEPATHY_QT4_NO_EXPORT PendingChannel::Private
  */
 PendingChannel::PendingChannel(const ConnectionPtr &connection, const QString &errorName,
         const QString &errorMessage)
-    : PendingOperation(0),
-      mPriv(new Private(connection))
+    : PendingOperation(connection),
+      mPriv(new Private)
 {
     mPriv->yours = false;
     mPriv->handleType = 0;
@@ -88,27 +83,29 @@ PendingChannel::PendingChannel(const ConnectionPtr &connection, const QString &e
  */
 PendingChannel::PendingChannel(const ConnectionPtr &connection,
         const QVariantMap &request, bool create)
-    : PendingOperation(0),
-      mPriv(new Private(connection))
+    : PendingOperation(connection),
+      mPriv(new Private)
 {
     mPriv->yours = create;
     mPriv->channelType = request.value(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".ChannelType")).toString();
     mPriv->handleType = request.value(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandleType")).toUInt();
     mPriv->handle = request.value(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandle")).toUInt();
 
+    Client::ConnectionInterfaceRequestsInterface *requestsInterface =
+        connection->interface<Client::ConnectionInterfaceRequestsInterface>();
     if (create) {
         QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(
-                connection->requestsInterface()->CreateChannel(request), this);
+                requestsInterface->CreateChannel(request), this);
         connect(watcher,
-                SIGNAL(finished(QDBusPendingCallWatcher *)),
-                SLOT(onCallCreateChannelFinished(QDBusPendingCallWatcher *)));
+                SIGNAL(finished(QDBusPendingCallWatcher*)),
+                SLOT(onCallCreateChannelFinished(QDBusPendingCallWatcher*)));
     }
     else {
         QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(
-                connection->requestsInterface()->EnsureChannel(request), this);
+                requestsInterface->EnsureChannel(request), this);
         connect(watcher,
-                SIGNAL(finished(QDBusPendingCallWatcher *)),
-                SLOT(onCallEnsureChannelFinished(QDBusPendingCallWatcher *)));
+                SIGNAL(finished(QDBusPendingCallWatcher*)),
+                SLOT(onCallEnsureChannelFinished(QDBusPendingCallWatcher*)));
     }
 }
 
@@ -127,7 +124,7 @@ PendingChannel::~PendingChannel()
  */
 ConnectionPtr PendingChannel::connection() const
 {
-    return mPriv->connection;
+    return ConnectionPtr(qobject_cast<Connection*>((Connection*) object().data()));
 }
 
 /**
@@ -230,12 +227,11 @@ QVariantMap PendingChannel::immutableProperties() const
     // be the Connection's self handle
     if (!props.contains(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".InitiatorHandle"))) {
         if (qdbus_cast<bool>(props.value(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".Requested")))) {
-            ConnectionPtr conn(mPriv->connection);
-            if (conn && conn->isReady()) {
+            if (connection() && connection()->isReady()) {
                 debug() << "CM didn't provide InitiatorHandle in channel immutable props, but we "
                     "know it's the conn's self handle (and have it)";
                 props[QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".InitiatorHandle")] =
-                    conn->selfHandle();
+                    connection()->selfHandle();
             }
         }
     }
@@ -261,52 +257,31 @@ ChannelPtr PendingChannel::channel() const
         return ChannelPtr();
     }
 
-    if (mPriv->channel) {
-        return mPriv->channel;
-    }
-
-    SharedPtr<Connection> conn(mPriv->connection);
-    mPriv->channel = ChannelFactory::create(conn,
-            mPriv->objectPath.path(), immutableProperties());
     return mPriv->channel;
 }
 
-/**
- * Returns the channel object path or an empty string on error.
- *
- * This method is useful for creating custom Channel objects, so instead of using
- * PendingChannel::channel, one could construct a new custom channel with
- * the object path.
- *
- * \return Channel object path.
- */
-QString PendingChannel::objectPath() const
-{
-    if (!isFinished()) {
-        warning() << "PendingChannel::channel called before finished";
-    } else if (!isValid()) {
-        warning() << "PendingChannel::channel called when not valid";
-    }
-
-    return mPriv->objectPath.path();
-}
-
 void PendingChannel::onCallCreateChannelFinished(QDBusPendingCallWatcher *watcher)
 {
     QDBusPendingReply<QDBusObjectPath, QVariantMap> reply = *watcher;
 
     if (!reply.isError()) {
-        mPriv->objectPath = reply.argumentAt<0>();
-        debug() << "Got reply to Connection.CreateChannel - object path:" <<
-            mPriv->objectPath.path();
-
+        QString objectPath = reply.argumentAt<0>().path();
         QVariantMap map = reply.argumentAt<1>();
+
+        debug() << "Got reply to Connection.CreateChannel - object path:" << objectPath;
+
+        PendingReady *channelReady =
+            connection()->channelFactory()->proxy(connection(), objectPath, map);
+        mPriv->channel = ChannelPtr::qObjectCast(channelReady->proxy());
+
         mPriv->immutableProperties = map;
         mPriv->channelType = map.value(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".ChannelType")).toString();
         mPriv->handleType = map.value(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandleType")).toUInt();
         mPriv->handle = map.value(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandle")).toUInt();
 
-        setFinished();
+        connect(channelReady,
+                SIGNAL(finished(Tp::PendingOperation*)),
+                SLOT(onChannelReady(Tp::PendingOperation*)));
     } else {
         debug().nospace() << "CreateChannel failed:" <<
             reply.error().name() << ": " << reply.error().message();
@@ -322,18 +297,23 @@ void PendingChannel::onCallEnsureChannelFinished(QDBusPendingCallWatcher *watche
 
     if (!reply.isError()) {
         mPriv->yours = reply.argumentAt<0>();
-
-        mPriv->objectPath = reply.argumentAt<1>();
-        debug() << "Got reply to Connection.EnsureChannel - object path:" <<
-            mPriv->objectPath.path();
-
+        QString objectPath = reply.argumentAt<1>().path();
         QVariantMap map = reply.argumentAt<2>();
+
+        debug() << "Got reply to Connection.EnsureChannel - object path:" << objectPath;
+
+        PendingReady *channelReady =
+            connection()->channelFactory()->proxy(connection(), objectPath, map);
+        mPriv->channel = ChannelPtr::qObjectCast(channelReady->proxy());
+
         mPriv->immutableProperties = map;
         mPriv->channelType = map.value(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".ChannelType")).toString();
         mPriv->handleType = map.value(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandleType")).toUInt();
         mPriv->handle = map.value(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandle")).toUInt();
 
-        setFinished();
+        connect(channelReady,
+                SIGNAL(finished(Tp::PendingOperation*)),
+                SLOT(onChannelReady(Tp::PendingOperation*)));
     } else {
         debug().nospace() << "EnsureChannel failed:" <<
             reply.error().name() << ": " << reply.error().message();
@@ -343,4 +323,15 @@ void PendingChannel::onCallEnsureChannelFinished(QDBusPendingCallWatcher *watche
     watcher->deleteLater();
 }
 
+void PendingChannel::onChannelReady(PendingOperation *op)
+{
+    if (!op->isError()) {
+        setFinished();
+    } else {
+        debug() << "Making the channel ready for" << this << "failed with" << op->errorName()
+            << ":" << op->errorMessage();
+        setFinishedWithError(op->errorName(), op->errorMessage());
+    }
+}
+
 } // Tp
