@@ -25,8 +25,11 @@
 
 #include "TelepathyQt4/debug-internal.h"
 
+#include <TelepathyQt4/ChannelFactory>
 #include <TelepathyQt4/ConnectionManager>
 #include <TelepathyQt4/Connection>
+#include <TelepathyQt4/ContactFactory>
+#include <TelepathyQt4/PendingReady>
 
 #include <QDBusObjectPath>
 #include <QDBusPendingCallWatcher>
@@ -37,15 +40,7 @@ namespace Tp
 
 struct TELEPATHY_QT4_NO_EXPORT PendingConnection::Private
 {
-    Private(const ConnectionManagerPtr &manager) :
-        manager(manager)
-    {
-    }
-
-    ConnectionManagerPtr manager;
     ConnectionPtr connection;
-    QString busName;
-    QDBusObjectPath objectPath;
 };
 
 /**
@@ -67,15 +62,28 @@ struct TELEPATHY_QT4_NO_EXPORT PendingConnection::Private
  */
 PendingConnection::PendingConnection(const ConnectionManagerPtr &manager,
         const QString &protocol, const QVariantMap &parameters)
-    : PendingOperation(0),
-      mPriv(new Private(manager))
+    : PendingOperation(manager),
+      mPriv(new Private)
 {
     QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(
             manager->baseInterface()->RequestConnection(protocol,
                 parameters), this);
     connect(watcher,
-            SIGNAL(finished(QDBusPendingCallWatcher *)),
-            SLOT(onCallFinished(QDBusPendingCallWatcher *)));
+            SIGNAL(finished(QDBusPendingCallWatcher*)),
+            SLOT(onCallFinished(QDBusPendingCallWatcher*)));
+}
+
+/**
+ * Construct a PendingConnection object which will fail immediately.
+ *
+ * \param error Name of the error to fail with.
+ * \param errorMessage Detail message for the error.
+ */
+PendingConnection::PendingConnection(const QString &error, const QString &errorMessage)
+    : PendingOperation(ConnectionManagerPtr()),
+      mPriv(new Private)
+{
+    setFinishedWithError(error, errorMessage);
 }
 
 /**
@@ -93,7 +101,7 @@ PendingConnection::~PendingConnection()
  */
 ConnectionManagerPtr PendingConnection::manager() const
 {
-    return mPriv->manager;
+    return ConnectionManagerPtr(qobject_cast<ConnectionManager*>((ConnectionManager*) object().data()));
 }
 
 /**
@@ -111,68 +119,26 @@ ConnectionPtr PendingConnection::connection() const
         return ConnectionPtr();
     }
 
-    if (!mPriv->connection) {
-        ConnectionManagerPtr manager(mPriv->manager);
-        mPriv->connection = Connection::create(manager->dbusConnection(),
-                mPriv->busName, mPriv->objectPath.path());
-    }
-
     return mPriv->connection;
 }
 
-/**
- * Returns the connection's bus name ("service name"), or an empty string on
- * error.
- *
- * This method is useful for creating custom Connection objects: instead
- * of using PendingConnection::connection, one could construct a new custom
- * connection from the bus name and object path.
- *
- * \return Connection bus name
- * \sa objectPath()
- */
-QString PendingConnection::busName() const
-{
-    if (!isFinished()) {
-        warning() << "PendingConnection::busName called before finished";
-    } else if (!isValid()) {
-        warning() << "PendingConnection::busName called when not valid";
-    }
-
-    return mPriv->busName;
-}
-
-/**
- * Returns the connection's object path or an empty string on error.
- *
- * This method is useful for creating custom Connection objects: instead
- * of using PendingConnection::connection, one could construct a new custom
- * connection with the bus name and object path.
- *
- * \return Connection object path
- * \sa busName()
- */
-QString PendingConnection::objectPath() const
-{
-    if (!isFinished()) {
-        warning() << "PendingConnection::connection called before finished";
-    } else if (!isValid()) {
-        warning() << "PendingConnection::connection called when not valid";
-    }
-
-    return mPriv->objectPath.path();
-}
-
 void PendingConnection::onCallFinished(QDBusPendingCallWatcher *watcher)
 {
     QDBusPendingReply<QString, QDBusObjectPath> reply = *watcher;
 
     if (!reply.isError()) {
-        mPriv->busName = reply.argumentAt<0>();
-        mPriv->objectPath = reply.argumentAt<1>();
+        QString busName = reply.argumentAt<0>();
+        QString objectPath = reply.argumentAt<1>().path();
+
         debug() << "Got reply to ConnectionManager.CreateConnection - bus name:" <<
-            mPriv->busName << "- object path:" << mPriv->objectPath.path();
-        setFinished();
+            busName << "- object path:" << objectPath;
+
+        PendingReady *readyOp = manager()->connectionFactory()->proxy(busName,
+                objectPath, manager()->channelFactory(), manager()->contactFactory());
+        mPriv->connection = ConnectionPtr::qObjectCast(readyOp->proxy());
+        connect(readyOp,
+                SIGNAL(finished(Tp::PendingOperation*)),
+                SLOT(onConnectionBuilt(Tp::PendingOperation*)));
     } else {
         debug().nospace() <<
             "CreateConnection failed: " <<
@@ -183,4 +149,18 @@ void PendingConnection::onCallFinished(QDBusPendingCallWatcher *watcher)
     watcher->deleteLater();
 }
 
+void PendingConnection::onConnectionBuilt(Tp::PendingOperation *op)
+{
+    Q_ASSERT(op->isFinished());
+
+    if (op->isError()) {
+        warning() << "Making connection ready using the factory failed:" <<
+            op->errorName() << op->errorMessage();
+        setFinishedWithError(op->errorName(), op->errorMessage());
+    } else {
+        setFinished();
+        debug() << "New connection" << mPriv->connection->objectPath() << "built";
+    }
+}
+
 } // Tp
