@@ -35,13 +35,7 @@ namespace Tp
 
 struct TELEPATHY_QT4_NO_EXPORT PendingContactInfo::Private
 {
-    Private(const ContactPtr &contact) :
-        contact(contact)
-    {
-    }
-
-    ContactPtr contact;
-    ContactInfoFieldList info;
+    Contact::InfoFields info;
 };
 
 /**
@@ -64,16 +58,18 @@ struct TELEPATHY_QT4_NO_EXPORT PendingContactInfo::Private
  * \param contact Contact to use.
  */
 PendingContactInfo::PendingContactInfo(const ContactPtr &contact)
-    : PendingOperation(0),
-      mPriv(new Private(contact))
+    : PendingOperation(contact),
+      mPriv(new Private)
 {
     ConnectionPtr connection = contact->manager()->connection();
+    Client::ConnectionInterfaceContactInfoInterface *contactInfoInterface =
+        connection->interface<Client::ConnectionInterfaceContactInfoInterface>();
     QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(
-            connection->contactInfoInterface()->RequestContactInfo(
+            contactInfoInterface->RequestContactInfo(
                 contact->handle()[0]), this);
     connect(watcher,
-            SIGNAL(finished(QDBusPendingCallWatcher *)),
-            SLOT(onCallFinished(QDBusPendingCallWatcher *)));
+            SIGNAL(finished(QDBusPendingCallWatcher*)),
+            SLOT(onCallFinished(QDBusPendingCallWatcher*)));
 }
 
 /**
@@ -91,7 +87,7 @@ PendingContactInfo::~PendingContactInfo()
  */
 ContactPtr PendingContactInfo::contact() const
 {
-    return mPriv->contact;
+    return ContactPtr(qobject_cast<Contact*>((Contact*) object().data()));
 }
 
 /**
@@ -99,7 +95,7 @@ ContactPtr PendingContactInfo::contact() const
  *
  * \return An object representing the contact information.
  */
-ContactInfoFieldList PendingContactInfo::info() const
+Contact::InfoFields PendingContactInfo::infoFields() const
 {
     if (!isFinished()) {
         warning() << "PendingContactInfo::info called before finished";
@@ -115,7 +111,7 @@ void PendingContactInfo::onCallFinished(QDBusPendingCallWatcher *watcher)
     QDBusPendingReply<Tp::ContactInfoFieldList> reply = *watcher;
 
     if (!reply.isError()) {
-        mPriv->info = reply.value();
+        mPriv->info = Contact::InfoFields(reply.value());
         debug() << "Got reply to ContactInfo.RequestContactInfo";
         setFinished();
     } else {
