@@ -33,8 +33,7 @@ namespace Tp
 
 struct TELEPATHY_QT4_NO_EXPORT PendingHandles::Private
 {
-    ConnectionPtr connection;
-    uint handleType;
+    HandleType handleType;
     bool isRequest;
     QStringList namesRequested;
     UIntList handlesToReference;
@@ -62,14 +61,13 @@ struct TELEPATHY_QT4_NO_EXPORT PendingHandles::Private
  * Connection::referenceHandles().
  */
 
-PendingHandles::PendingHandles(const ConnectionPtr &connection, uint handleType,
+PendingHandles::PendingHandles(const ConnectionPtr &connection, HandleType handleType,
         const QStringList &names)
-    : PendingOperation(0),
+    : PendingOperation(connection),
       mPriv(new Private)
 {
     debug() << "PendingHandles(request)";
 
-    mPriv->connection = connection;
     mPriv->handleType = handleType;
     mPriv->isRequest = true;
     mPriv->namesRequested = names;
@@ -78,26 +76,25 @@ PendingHandles::PendingHandles(const ConnectionPtr &connection, uint handleType,
     // try to request all handles at once
     QDBusPendingCallWatcher *watcher =
         new QDBusPendingCallWatcher(
-                connection->baseInterface()->RequestHandles(handleType, names),
+                connection->baseInterface()->RequestHandles(mPriv->handleType, names),
                 this);
     connect(watcher,
-            SIGNAL(finished(QDBusPendingCallWatcher *)),
-            SLOT(onRequestHandlesFinished(QDBusPendingCallWatcher *)));
+            SIGNAL(finished(QDBusPendingCallWatcher*)),
+            SLOT(onRequestHandlesFinished(QDBusPendingCallWatcher*)));
 }
 
-PendingHandles::PendingHandles(const ConnectionPtr &connection, uint handleType,
+PendingHandles::PendingHandles(const ConnectionPtr &connection, HandleType handleType,
         const UIntList &handles, const UIntList &alreadyHeld,
         const UIntList &notYetHeld)
-    : PendingOperation(0),
+    : PendingOperation(connection),
       mPriv(new Private)
 {
     debug() << "PendingHandles(reference)";
 
-    mPriv->connection = connection;
     mPriv->handleType = handleType;
     mPriv->isRequest = false;
     mPriv->handlesToReference = handles;
-    mPriv->alreadyHeld = ReferencedHandles(connection, handleType, alreadyHeld);
+    mPriv->alreadyHeld = ReferencedHandles(connection, mPriv->handleType, alreadyHeld);
     mPriv->requestsFinished = 0;
 
     if (notYetHeld.isEmpty()) {
@@ -109,14 +106,20 @@ PendingHandles::PendingHandles(const ConnectionPtr &connection, uint handleType,
 
         QDBusPendingCallWatcher *watcher =
             new QDBusPendingCallWatcher(
-                    connection->baseInterface()->HoldHandles(handleType, notYetHeld),
+                    connection->baseInterface()->HoldHandles(mPriv->handleType, notYetHeld),
                     this);
         connect(watcher,
-                SIGNAL(finished(QDBusPendingCallWatcher *)),
-                SLOT(onHoldHandlesFinished(QDBusPendingCallWatcher *)));
+                SIGNAL(finished(QDBusPendingCallWatcher*)),
+                SLOT(onHoldHandlesFinished(QDBusPendingCallWatcher*)));
     }
 }
 
+PendingHandles::PendingHandles(const QString &errorName, const QString &errorMessage)
+    : PendingOperation(ConnectionPtr()), mPriv(new Private)
+{
+    setFinishedWithError(errorName, errorMessage);
+}
+
 /**
  * Class destructor.
  */
@@ -132,7 +135,7 @@ PendingHandles::~PendingHandles()
  */
 ConnectionPtr PendingHandles::connection() const
 {
-    return mPriv->connection;
+    return ConnectionPtr(qobject_cast<Connection*>((Connection*) object().data()));
 }
 
 /**
@@ -140,7 +143,7 @@ ConnectionPtr PendingHandles::connection() const
  *
  * \return The handle type, as specified in #HandleType.
  */
-uint PendingHandles::handleType() const
+HandleType PendingHandles::handleType() const
 {
     return mPriv->handleType;
 }
@@ -268,7 +271,7 @@ void PendingHandles::onRequestHandlesFinished(QDBusPendingCallWatcher *watcher)
                             error.message()));
             }
             setFinishedWithError(error);
-            mPriv->connection->handleRequestLanded(mPriv->handleType);
+            connection()->handleRequestLanded(mPriv->handleType);
             watcher->deleteLater();
             return;
         }
@@ -282,7 +285,7 @@ void PendingHandles::onRequestHandlesFinished(QDBusPendingCallWatcher *watcher)
                     QPair<QString, QString>(error.name(),
                         error.message()));
             setFinished();
-            mPriv->connection->handleRequestLanded(mPriv->handleType);
+            connection()->handleRequestLanded(mPriv->handleType);
             watcher->deleteLater();
             return;
         }
@@ -291,22 +294,22 @@ void PendingHandles::onRequestHandlesFinished(QDBusPendingCallWatcher *watcher)
         foreach (const QString &name, mPriv->namesRequested) {
             QDBusPendingCallWatcher *watcher =
                 new QDBusPendingCallWatcher(
-                        mPriv->connection->baseInterface()->RequestHandles(
+                        connection()->baseInterface()->RequestHandles(
                             mPriv->handleType,
                             QStringList() << name),
                         this);
             connect(watcher,
-                    SIGNAL(finished(QDBusPendingCallWatcher *)),
-                    SLOT(onRequestHandlesFallbackFinished(QDBusPendingCallWatcher *)));
+                    SIGNAL(finished(QDBusPendingCallWatcher*)),
+                    SLOT(onRequestHandlesFallbackFinished(QDBusPendingCallWatcher*)));
             mPriv->idsForWatchers.insert(watcher, name);
         }
     } else {
         debug() << "Received reply to RequestHandles";
-        mPriv->handles = ReferencedHandles(mPriv->connection,
+        mPriv->handles = ReferencedHandles(connection(),
                 mPriv->handleType, reply.value());
         mPriv->validNames.append(mPriv->namesRequested);
         setFinished();
-        mPriv->connection->handleRequestLanded(mPriv->handleType);
+        connection()->handleRequestLanded(mPriv->handleType);
     }
 }
 
@@ -347,17 +350,17 @@ void PendingHandles::onHoldHandlesFinished(QDBusPendingCallWatcher *watcher)
         foreach (uint handle, mPriv->handlesToReference) {
             QDBusPendingCallWatcher *watcher =
                 new QDBusPendingCallWatcher(
-                        mPriv->connection->baseInterface()->HoldHandles(
+                        connection()->baseInterface()->HoldHandles(
                             mPriv->handleType,
                             UIntList() << handle),
                         this);
             connect(watcher,
-                    SIGNAL(finished(QDBusPendingCallWatcher *)),
-                    SLOT(onHoldHandlesFallbackFinished(QDBusPendingCallWatcher *)));
+                    SIGNAL(finished(QDBusPendingCallWatcher*)),
+                    SLOT(onHoldHandlesFallbackFinished(QDBusPendingCallWatcher*)));
             mPriv->handlesForWatchers.insert(watcher, handle);
         }
     } else {
-        mPriv->handles = ReferencedHandles(mPriv->connection,
+        mPriv->handles = ReferencedHandles(connection(),
                 mPriv->handleType, mPriv->handlesToReference);
         setFinished();
     }
@@ -389,7 +392,7 @@ void PendingHandles::onRequestHandlesFallbackFinished(QDBusPendingCallWatcher *w
                             error.message()));
             }
             setFinishedWithError(error);
-            mPriv->connection->handleRequestLanded(mPriv->handleType);
+            connection()->handleRequestLanded(mPriv->handleType);
             watcher->deleteLater();
             return;
         }
@@ -419,7 +422,7 @@ void PendingHandles::onRequestHandlesFallbackFinished(QDBusPendingCallWatcher *w
                     mPriv->validNames.append(name);
                 }
             }
-            mPriv->handles = ReferencedHandles(mPriv->connection,
+            mPriv->handles = ReferencedHandles(connection(),
                     mPriv->handleType, handles);
 
             setFinished();
@@ -429,7 +432,7 @@ void PendingHandles::onRequestHandlesFallbackFinished(QDBusPendingCallWatcher *w
         debug() << " invalidNames  :" << mPriv->invalidNames;
         debug() << " validNames    :" << mPriv->validNames;
 
-        mPriv->connection->handleRequestLanded(mPriv->handleType);
+        connection()->handleRequestLanded(mPriv->handleType);
     }
 
     watcher->deleteLater();
@@ -472,7 +475,7 @@ void PendingHandles::onHoldHandlesFallbackFinished(QDBusPendingCallWatcher *watc
         }
 
         if (handles.size() != 0) {
-            mPriv->handles = ReferencedHandles(mPriv->connection,
+            mPriv->handles = ReferencedHandles(connection(),
                     mPriv->handleType, handles);
         }
 
