@@ -25,9 +25,11 @@
 
 #include "TelepathyQt4/debug-internal.h"
 
+#include <TelepathyQt4/Constants>
 #include <TelepathyQt4/DBusProxy>
 #include <TelepathyQt4/PendingReady>
-#include <TelepathyQt4/Constants>
+#include <TelepathyQt4/RefCounted>
+#include <TelepathyQt4/SharedPtr>
 
 #include <QDBusError>
 #include <QSharedData>
@@ -91,7 +93,11 @@ ReadinessHelper::Introspectable &ReadinessHelper::Introspectable::operator=(
 struct TELEPATHY_QT4_NO_EXPORT ReadinessHelper::Private
 {
     Private(ReadinessHelper *parent,
-            QObject *object,
+            RefCounted *object,
+            uint currentStatus,
+            const Introspectables &introspectables);
+    Private(ReadinessHelper *parent,
+            DBusProxy *proxy,
             uint currentStatus,
             const Introspectables &introspectables);
     ~Private();
@@ -106,7 +112,7 @@ struct TELEPATHY_QT4_NO_EXPORT ReadinessHelper::Private
     void abortOperations(const QString &errorName, const QString &errorMessage);
 
     ReadinessHelper *parent;
-    QObject *object;
+    RefCounted *object;
     DBusProxy *proxy;
     uint currentStatus;
     QStringList interfaces;
@@ -127,12 +133,12 @@ struct TELEPATHY_QT4_NO_EXPORT ReadinessHelper::Private
 
 ReadinessHelper::Private::Private(
         ReadinessHelper *parent,
-        QObject *object,
+        RefCounted *object,
         uint currentStatus,
         const Introspectables &introspectables)
     : parent(parent),
       object(object),
-      proxy(qobject_cast<DBusProxy*>(object)),
+      proxy(0),
       currentStatus(currentStatus),
       introspectables(introspectables),
       pendingStatusChange(false),
@@ -146,20 +152,38 @@ ReadinessHelper::Private::Private(
         supportedStatuses += introspectable.mPriv->makesSenseForStatuses;
         supportedFeatures += feature;
     }
+}
 
-    if (proxy) {
-        parent->connect(proxy,
-                SIGNAL(invalidated(Tp::DBusProxy *, const QString &, const QString &)),
-                SLOT(onProxyInvalidated(Tp::DBusProxy *, const QString &, const QString &)));
+ReadinessHelper::Private::Private(
+        ReadinessHelper *parent,
+        DBusProxy *proxy,
+        uint currentStatus,
+        const Introspectables &introspectables)
+    : parent(parent),
+      object(proxy),
+      proxy(proxy),
+      currentStatus(currentStatus),
+      introspectables(introspectables),
+      pendingStatusChange(false),
+      pendingStatus(-1)
+{
+    Q_ASSERT(proxy != 0);
+
+    for (Introspectables::const_iterator i = introspectables.constBegin();
+            i != introspectables.constEnd(); ++i) {
+        Feature feature = i.key();
+        Introspectable introspectable = i.value();
+        Q_ASSERT(introspectable.mPriv->introspectFunc != 0);
+        supportedStatuses += introspectable.mPriv->makesSenseForStatuses;
+        supportedFeatures += feature;
     }
-
-    debug() << "ReadinessHelper: supportedStatuses =" << supportedStatuses;
-    debug() << "ReadinessHelper: supportedFeatures =" << supportedFeatures;
 }
 
 ReadinessHelper::Private::~Private()
 {
-    abortOperations(QLatin1String(TELEPATHY_ERROR_CANCELLED), QLatin1String("Destroyed"));
+    const static QString messageDestroyed(QLatin1String("Destroyed"));
+
+    abortOperations(TP_QT4_ERROR_CANCELLED, messageDestroyed);
 }
 
 void ReadinessHelper::Private::setCurrentStatus(uint newStatus)
@@ -176,6 +200,13 @@ void ReadinessHelper::Private::setCurrentStatus(uint newStatus)
         // retrieve all features that were requested for the new status
         pendingFeatures = requestedFeatures;
 
+        // make sure that we have inserted the dependencies for the requested features too
+        Features deps;
+        foreach (Feature feature, pendingFeatures) {
+            deps.unite(depsFor(feature));
+        }
+        pendingFeatures.unite(deps);
+
         if (supportedStatuses.contains(currentStatus)) {
             QTimer::singleShot(0, parent, SLOT(iterateIntrospection()));
         } else {
@@ -256,13 +287,15 @@ void ReadinessHelper::Private::iterateIntrospection()
         }
     }
 
+    const Features completedFeatures = satisfiedFeatures + missingFeatures;
+
     // check if any pending operations for becomeReady should finish now
     // based on their requested features having nothing more than what
     // satisfiedFeatures + missingFeatures has
     QString errorName;
     QString errorMessage;
     foreach (PendingReady *operation, pendingOperations) {
-        if ((operation->requestedFeatures() - (satisfiedFeatures + missingFeatures)).isEmpty()) {
+        if ((operation->requestedFeatures() - completedFeatures).isEmpty()) {
             if (parent->isReady(operation->requestedFeatures(), &errorName, &errorMessage)) {
                 operation->setFinished();
             } else {
@@ -271,22 +304,15 @@ void ReadinessHelper::Private::iterateIntrospection()
         }
     }
 
-    if ((requestedFeatures - (satisfiedFeatures + missingFeatures)).isEmpty()) {
+    if ((requestedFeatures - completedFeatures).isEmpty()) {
         // all requested features satisfied or missing
         emit parent->statusReady(currentStatus);
         return;
     }
 
-    // Update pendingFeatures to contain the (recursive) dependencies of what's in it currently
-    Features deps;
-    foreach (Feature feature, pendingFeatures) {
-        deps.unite(depsFor(feature));
-    }
-    pendingFeatures.unite(deps);
-
     // update pendingFeatures with the difference of requested and
     // satisfied + missing
-    pendingFeatures -= (satisfiedFeatures + missingFeatures);
+    pendingFeatures -= completedFeatures;
 
     // find out which features don't have dependencies that are still pending
     Features readyToIntrospect;
@@ -365,7 +391,7 @@ void ReadinessHelper::Private::abortOperations(const QString &errorName,
     pendingOperations.clear();
 }
 
-ReadinessHelper::ReadinessHelper(QObject *object,
+ReadinessHelper::ReadinessHelper(RefCounted *object,
         uint currentStatus,
         const Introspectables &introspectables,
         QObject *parent)
@@ -374,6 +400,15 @@ ReadinessHelper::ReadinessHelper(QObject *object,
 {
 }
 
+ReadinessHelper::ReadinessHelper(DBusProxy *proxy,
+        uint currentStatus,
+        const Introspectables &introspectables,
+        QObject *parent)
+    : QObject(parent),
+      mPriv(new Private(this, proxy, currentStatus, introspectables))
+{
+}
+
 ReadinessHelper::~ReadinessHelper()
 {
     delete mPriv;
@@ -529,12 +564,26 @@ PendingReady *ReadinessHelper::becomeReady(const Features &requestedFeatures)
 {
     Q_ASSERT(!requestedFeatures.isEmpty());
 
+    if (mPriv->proxy) {
+        connect(mPriv->proxy,
+                SIGNAL(invalidated(Tp::DBusProxy*,QString,QString)),
+                SLOT(onProxyInvalidated(Tp::DBusProxy*,QString,QString)));
+
+        if (!mPriv->proxy->isValid()) {
+            PendingReady *operation = new PendingReady(SharedPtr<RefCounted>(mPriv->object),
+                    requestedFeatures);
+            operation->setFinishedWithError(mPriv->proxy->invalidationReason(),
+                    mPriv->proxy->invalidationMessage());
+            return operation;
+        }
+    }
+
     Features supportedFeatures = mPriv->supportedFeatures;
     if (supportedFeatures.intersect(requestedFeatures) != requestedFeatures) {
         warning() << "ReadinessHelper::becomeReady called with invalid features: requestedFeatures =" <<
             requestedFeatures << "- supportedFeatures =" << mPriv->supportedFeatures;
-        PendingReady *operation =
-            new PendingReady(requestedFeatures, mPriv->object, this);
+        PendingReady *operation = new PendingReady(SharedPtr<RefCounted>(mPriv->object),
+                requestedFeatures);
         operation->setFinishedWithError(
                 QLatin1String(TELEPATHY_ERROR_INVALID_ARGUMENT),
                 QLatin1String("Requested features contains unsupported feature"));
@@ -542,8 +591,8 @@ PendingReady *ReadinessHelper::becomeReady(const Features &requestedFeatures)
     }
 
     if (mPriv->proxy && !mPriv->proxy->isValid()) {
-        PendingReady *operation =
-            new PendingReady(requestedFeatures, mPriv->object, this);
+        PendingReady *operation = new PendingReady(SharedPtr<RefCounted>(mPriv->object),
+                requestedFeatures);
         operation->setFinishedWithError(mPriv->proxy->invalidationReason(),
                 mPriv->proxy->invalidationMessage());
         return operation;
@@ -560,7 +609,14 @@ PendingReady *ReadinessHelper::becomeReady(const Features &requestedFeatures)
     // it will be updated on iterateIntrospection
     mPriv->pendingFeatures += requestedFeatures;
 
-    operation = new PendingReady(requestedFeatures, mPriv->object, this);
+    // Insert the dependencies of the requested features too
+    Features deps;
+    foreach (Feature feature, mPriv->pendingFeatures) {
+        deps.unite(mPriv->depsFor(feature));
+    }
+    mPriv->pendingFeatures.unite(deps);
+
+    operation = new PendingReady(SharedPtr<RefCounted>(mPriv->object), requestedFeatures);
     connect(operation,
             SIGNAL(finished(Tp::PendingOperation*)),
             SLOT(onOperationFinished(Tp::PendingOperation*)));
