@@ -50,8 +50,9 @@ inline PendingSendMessage::Private::Private(const Message &message)
 {
 }
 
-PendingSendMessage::PendingSendMessage(const Message &message, QObject *parent)
-    : PendingOperation(parent), mPriv(new Private(message))
+PendingSendMessage::PendingSendMessage(const TextChannelPtr &channel, const Message &message)
+    : PendingOperation(channel),
+      mPriv(new Private(message))
 {
 }
 
@@ -60,6 +61,11 @@ PendingSendMessage::~PendingSendMessage()
     delete mPriv;
 }
 
+TextChannelPtr PendingSendMessage::channel() const
+{
+    return TextChannelPtr(qobject_cast<TextChannel*>((TextChannel*) object().data()));
+}
+
 QString PendingSendMessage::sentMessageToken() const
 {
     return mPriv->token;
@@ -117,6 +123,9 @@ struct TELEPATHY_QT4_NO_EXPORT TextChannel::Private
     // Public object
     TextChannel *parent;
 
+    Client::ChannelTypeTextInterface *textInterface;
+    Client::DBus::PropertiesInterface *properties;
+
     ReadinessHelper *readinessHelper;
 
     // FeatureMessageCapabilities and FeatureMessageQueue
@@ -168,6 +177,8 @@ struct TELEPATHY_QT4_NO_EXPORT TextChannel::Private
 
 TextChannel::Private::Private(TextChannel *parent)
     : parent(parent),
+      textInterface(parent->interface<Client::ChannelTypeTextInterface>()),
+      properties(parent->interface<Client::DBus::PropertiesInterface>()),
       readinessHelper(parent->readinessHelper()),
       getAllInFlight(false),
       gotProperties(false),
@@ -229,44 +240,47 @@ void TextChannel::Private::introspectMessageQueue(
     TextChannel *parent = self->parent;
 
     if (parent->hasMessagesInterface()) {
+        Client::ChannelInterfaceMessagesInterface *messagesInterface =
+            parent->interface<Client::ChannelInterfaceMessagesInterface>();
+
         // FeatureMessageQueue needs signal connections + Get (but we
         // might as well do GetAll and reduce the number of code paths)
-        parent->connect(parent->messagesInterface(),
-                SIGNAL(MessageReceived(const Tp::MessagePartList &)),
-                SLOT(onMessageReceived(const Tp::MessagePartList &)));
-        parent->connect(parent->messagesInterface(),
-                SIGNAL(PendingMessagesRemoved(const Tp::UIntList &)),
-                SLOT(onPendingMessagesRemoved(const Tp::UIntList &)));
+        parent->connect(messagesInterface,
+                SIGNAL(MessageReceived(Tp::MessagePartList)),
+                SLOT(onMessageReceived(Tp::MessagePartList)));
+        parent->connect(messagesInterface,
+                SIGNAL(PendingMessagesRemoved(Tp::UIntList)),
+                SLOT(onPendingMessagesRemoved(Tp::UIntList)));
 
         if (!self->gotProperties && !self->getAllInFlight) {
             self->getAllInFlight = true;
             QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(
-                    parent->propertiesInterface()->GetAll(
+                    self->properties->GetAll(
                         QLatin1String(TELEPATHY_INTERFACE_CHANNEL_INTERFACE_MESSAGES)),
                         parent);
             parent->connect(watcher,
-                    SIGNAL(finished(QDBusPendingCallWatcher *)),
-                    SLOT(gotProperties(QDBusPendingCallWatcher *)));
+                    SIGNAL(finished(QDBusPendingCallWatcher*)),
+                    SLOT(gotProperties(QDBusPendingCallWatcher*)));
         } else if (self->gotProperties) {
             self->updateInitialMessages();
         }
     } else {
         // FeatureMessageQueue needs signal connections + ListPendingMessages
-        parent->connect(parent->textInterface(),
-                SIGNAL(Received(uint, uint, uint, uint, uint, const QString &)),
-                SLOT(onTextReceived(uint, uint, uint, uint, uint, const QString &)));
+        parent->connect(self->textInterface,
+                SIGNAL(Received(uint,uint,uint,uint,uint,QString)),
+                SLOT(onTextReceived(uint,uint,uint,uint,uint,const QString)));
 
         // we present SendError signals as if they were incoming
         // messages, to be consistent with Messages
-        parent->connect(parent->textInterface(),
-                SIGNAL(SendError(uint, uint, uint, const QString &)),
-                SLOT(onTextSendError(uint, uint, uint, const QString &)));
+        parent->connect(self->textInterface,
+                SIGNAL(SendError(uint,uint,uint,QString)),
+                SLOT(onTextSendError(uint,uint,uint,QString)));
 
         QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(
-                parent->textInterface()->ListPendingMessages(false), parent);
+                self->textInterface->ListPendingMessages(false), parent);
         parent->connect(watcher,
-                SIGNAL(finished(QDBusPendingCallWatcher *)),
-                SLOT(gotPendingMessages(QDBusPendingCallWatcher *)));
+                SIGNAL(finished(QDBusPendingCallWatcher*)),
+                SLOT(gotPendingMessages(QDBusPendingCallWatcher*)));
     }
 }
 
@@ -279,12 +293,12 @@ void TextChannel::Private::introspectMessageCapabilities(
         if (!self->gotProperties && !self->getAllInFlight) {
             self->getAllInFlight = true;
             QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(
-                    parent->propertiesInterface()->GetAll(
+                    self->properties->GetAll(
                         QLatin1String(TELEPATHY_INTERFACE_CHANNEL_INTERFACE_MESSAGES)),
                         parent);
             parent->connect(watcher,
-                    SIGNAL(finished(QDBusPendingCallWatcher *)),
-                    SLOT(gotProperties(QDBusPendingCallWatcher *)));
+                    SIGNAL(finished(QDBusPendingCallWatcher*)),
+                    SLOT(gotProperties(QDBusPendingCallWatcher*)));
         } else if (self->gotProperties) {
             self->updateCapabilities();
         }
@@ -302,15 +316,16 @@ void TextChannel::Private::introspectMessageSentSignal(
     TextChannel *parent = self->parent;
 
     if (parent->hasMessagesInterface()) {
-        parent->connect(parent->messagesInterface(),
-                SIGNAL(MessageSent(const Tp::MessagePartList &,
-                                   uint, const QString &)),
-                SLOT(onMessageSent(const Tp::MessagePartList &,
-                                   uint, const QString &)));
+        Client::ChannelInterfaceMessagesInterface *messagesInterface =
+            parent->interface<Client::ChannelInterfaceMessagesInterface>();
+
+        parent->connect(messagesInterface,
+                SIGNAL(MessageSent(Tp::MessagePartList,uint,QString)),
+                SLOT(onMessageSent(Tp::MessagePartList,uint,QString)));
     } else {
-        parent->connect(parent->textInterface(),
-                SIGNAL(Sent(uint, uint, const QString &)),
-                SLOT(onTextSent(uint, uint, const QString &)));
+        parent->connect(self->textInterface,
+                SIGNAL(Sent(uint,uint,QString)),
+                SLOT(onTextSent(uint,uint,QString)));
     }
 
     self->readinessHelper->setIntrospectCompleted(FeatureMessageSentSignal, true);
@@ -321,11 +336,11 @@ void TextChannel::Private::enableChatStateNotifications(
 {
     TextChannel *parent = self->parent;
     Client::ChannelInterfaceChatStateInterface *chatStateInterface =
-        parent->chatStateInterface();
+        parent->interface<Client::ChannelInterfaceChatStateInterface>();
 
     parent->connect(chatStateInterface,
-            SIGNAL(ChatStateChanged(uint, uint)),
-            SLOT(onChatStateChanged(uint, uint)));
+            SIGNAL(ChatStateChanged(uint,uint)),
+            SLOT(onChatStateChanged(uint,uint)));
 
     // FIXME fd.o#24882 - Download contacts' initial chat states
 
@@ -443,8 +458,8 @@ void TextChannel::Private::processMessageQueue()
 
     parent->connect(parent->connection()->contactManager()->contactsForHandles(
                 contactsRequired.toList()),
-            SIGNAL(finished(Tp::PendingOperation *)),
-            SLOT(onContactsFinished(Tp::PendingOperation *)));
+            SIGNAL(finished(Tp::PendingOperation*)),
+            SLOT(onContactsFinished(Tp::PendingOperation*)));
 
     awaitingContacts |= contactsRequired;
 }
@@ -487,8 +502,8 @@ void TextChannel::Private::processChatStateQueue()
 
     parent->connect(parent->connection()->contactManager()->contactsForHandles(
                 contactsRequired.toList()),
-            SIGNAL(finished(Tp::PendingOperation *)),
-            SLOT(onContactsFinished(Tp::PendingOperation *)));
+            SIGNAL(finished(Tp::PendingOperation*)),
+            SLOT(onContactsFinished(Tp::PendingOperation*)));
 
     awaitingContacts |= contactsRequired;
 }
@@ -541,6 +556,11 @@ void TextChannel::Private::contactFound(ContactPtr contact)
  * Telepathy channel of type Text.
  */
 
+/**
+ * A placeholder for "core functionality" which is currently just the same as Channel::FeatureCore.
+ */
+const Feature TextChannel::FeatureCore = Feature(QLatin1String(Channel::staticMetaObject.className()), 0, true);
+
 /**
  * Feature used in order to access the message queue info.
  *
@@ -632,7 +652,7 @@ TextChannelPtr TextChannel::create(const ConnectionPtr &connection,
         const QString &objectPath, const QVariantMap &immutableProperties)
 {
     return TextChannelPtr(new TextChannel(connection, objectPath,
-                immutableProperties));
+                immutableProperties, TextChannel::FeatureCore));
 }
 
 /**
@@ -642,11 +662,14 @@ TextChannelPtr TextChannel::create(const ConnectionPtr &connection,
  *                   service.
  * \param objectPath The object path of this channel.
  * \param immutableProperties The immutable properties of this channel.
+ * \param coreFeature The core feature of the channel type, if any. The corresponding introspectable should
+ * depend on TextChannel::FeatureCore.
  */
 TextChannel::TextChannel(const ConnectionPtr &connection,
         const QString &objectPath,
-        const QVariantMap &immutableProperties)
-    : Channel(connection, objectPath, immutableProperties),
+        const QVariantMap &immutableProperties,
+        const Feature &coreFeature)
+    : Channel(connection, objectPath, immutableProperties, coreFeature),
       mPriv(new Private(this))
 {
 }
@@ -826,7 +849,7 @@ void TextChannel::onAcknowledgePendingMessagesReply(
         debug() << "Recovering from AcknowledgePendingMessages failure for: "
             << ids;
         foreach (uint id, ids) {
-            textInterface()->AcknowledgePendingMessages(UIntList() << id);
+            mPriv->textInterface->AcknowledgePendingMessages(UIntList() << id);
         }
     }
 
@@ -872,10 +895,11 @@ void TextChannel::acknowledge(const QList<ReceivedMessage> &messages)
     forget(messages);
 
     QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(
-            textInterface()->AcknowledgePendingMessages(ids));
+            mPriv->textInterface->AcknowledgePendingMessages(ids),
+            this);
     connect(watcher,
-            SIGNAL(finished(QDBusPendingCallWatcher *)),
-            SLOT(onAcknowledgePendingMessagesReply(QDBusPendingCallWatcher *)));
+            SIGNAL(finished(QDBusPendingCallWatcher*)),
+            SLOT(onAcknowledgePendingMessagesReply(QDBusPendingCallWatcher*)));
     mPriv->acknowledgeBatches[watcher] = ids;
 }
 
@@ -907,20 +931,23 @@ PendingSendMessage *TextChannel::send(const QString &text,
         ChannelTextMessageType type, MessageSendingFlags flags)
 {
     Message m(type, text);
-    PendingSendMessage *op = new PendingSendMessage(m, this);
+    PendingSendMessage *op = new PendingSendMessage(TextChannelPtr(this), m);
 
     if (hasMessagesInterface()) {
+        Client::ChannelInterfaceMessagesInterface *messagesInterface =
+            interface<Client::ChannelInterfaceMessagesInterface>();
+
         connect(new QDBusPendingCallWatcher(
-                    messagesInterface()->SendMessage(m.parts(),
+                    messagesInterface->SendMessage(m.parts(),
                         (uint) flags)),
-                SIGNAL(finished(QDBusPendingCallWatcher *)),
+                SIGNAL(finished(QDBusPendingCallWatcher*)),
                 op,
-                SLOT(onMessageSent(QDBusPendingCallWatcher *)));
+                SLOT(onMessageSent(QDBusPendingCallWatcher*)));
     } else {
-        connect(new QDBusPendingCallWatcher(textInterface()->Send(type, text)),
-                SIGNAL(finished(QDBusPendingCallWatcher *)),
+        connect(new QDBusPendingCallWatcher(mPriv->textInterface->Send(type, text)),
+                SIGNAL(finished(QDBusPendingCallWatcher*)),
                 op,
-                SLOT(onTextSent(QDBusPendingCallWatcher *)));
+                SLOT(onTextSent(QDBusPendingCallWatcher*)));
     }
     return op;
 }
@@ -929,21 +956,24 @@ PendingSendMessage *TextChannel::send(const MessagePartList &parts,
         MessageSendingFlags flags)
 {
     Message m(parts);
-    PendingSendMessage *op = new PendingSendMessage(m, this);
+    PendingSendMessage *op = new PendingSendMessage(TextChannelPtr(this), m);
 
     if (hasMessagesInterface()) {
+        Client::ChannelInterfaceMessagesInterface *messagesInterface =
+            interface<Client::ChannelInterfaceMessagesInterface>();
+
         connect(new QDBusPendingCallWatcher(
-                    messagesInterface()->SendMessage(m.parts(),
+                    messagesInterface->SendMessage(m.parts(),
                         (uint) flags)),
-                SIGNAL(finished(QDBusPendingCallWatcher *)),
+                SIGNAL(finished(QDBusPendingCallWatcher*)),
                 op,
-                SLOT(onMessageSent(QDBusPendingCallWatcher *)));
+                SLOT(onMessageSent(QDBusPendingCallWatcher*)));
     } else {
-        connect(new QDBusPendingCallWatcher(textInterface()->Send(
+        connect(new QDBusPendingCallWatcher(mPriv->textInterface->Send(
                         m.messageType(), m.text())),
-                SIGNAL(finished(QDBusPendingCallWatcher *)),
+                SIGNAL(finished(QDBusPendingCallWatcher*)),
                 op,
-                SLOT(onTextSent(QDBusPendingCallWatcher *)));
+                SLOT(onTextSent(QDBusPendingCallWatcher*)));
     }
     return op;
 }
@@ -965,10 +995,13 @@ PendingOperation *TextChannel::requestChatState(ChannelChatState state)
             "state interface";
         return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_IMPLEMENTED),
                 QLatin1String("TextChannel does not support chat state interface"),
-                this);
+                TextChannelPtr(this));
     }
-    return new PendingVoid(chatStateInterface()->SetChatState(
-                (uint) state), this);
+
+    Client::ChannelInterfaceChatStateInterface *chatStateInterface =
+        interface<Client::ChannelInterfaceChatStateInterface>();
+    return new PendingVoid(chatStateInterface->SetChatState(
+                (uint) state), TextChannelPtr(this));
 }
 
 void TextChannel::onMessageSent(const MessagePartList &parts,
@@ -1204,10 +1237,17 @@ void TextChannel::gotPendingMessages(QDBusPendingCallWatcher *watcher)
 
     debug() << "Text::ListPendingMessages returned";
     PendingTextMessageList list = reply.value();
-    foreach (const PendingTextMessage &message, list) {
-        onTextReceived(message.identifier, message.unixTimestamp,
-                message.sender, message.messageType, message.flags,
-                message.text);
+
+    if (!list.isEmpty()) {
+        foreach (const PendingTextMessage &message, list) {
+            onTextReceived(message.identifier, message.unixTimestamp,
+                    message.sender, message.messageType, message.flags,
+                    message.text);
+        }
+        // processMessageQueue sets FeatureMessageQueue ready when the queue is empty for the first
+        // time
+    } else {
+        mPriv->readinessHelper->setIntrospectCompleted(FeatureMessageQueue, true);
     }
 
     watcher->deleteLater();
