@@ -19,14 +19,15 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef _TelepathyQt4_debug_h_HEADER_GUARD_
-#define _TelepathyQt4_debug_h_HEADER_GUARD_
+#ifndef _TelepathyQt4_utils_h_HEADER_GUARD_
+#define _TelepathyQt4_utils_h_HEADER_GUARD_
 
 #ifndef IN_TELEPATHY_QT4_HEADER
 #error IN_TELEPATHY_QT4_HEADER
 #endif
 
 #include <TelepathyQt4/Global>
+
 #include <QString>
 
 namespace Tp
