@@ -21,6 +21,8 @@
 #include "roster-item.h"
 #include "_gen/roster-item.moc.hpp"
 
+#include <TelepathyQt4/Presence>
+
 using namespace Tp;
 
 RosterItem::RosterItem(const ContactPtr &contact,
@@ -35,13 +37,13 @@ RosterItem::RosterItem(const ContactPtr &contact,
             SIGNAL(simplePresenceChanged(const QString &, uint, const QString &)),
             SLOT(onContactChanged()));
     connect(contact.data(),
-            SIGNAL(subscriptionStateChanged(Tp::Contact::PresenceState)),
+            SIGNAL(subscriptionStateChanged(Tp::Contact::PresenceState,Tp::Channel::GroupMemberChangeDetails)),
             SLOT(onContactChanged()));
     connect(contact.data(),
-            SIGNAL(publishStateChanged(Tp::Contact::PresenceState)),
+            SIGNAL(publishStateChanged(Tp::Contact::PresenceState,Tp::Channel::GroupMemberChangeDetails)),
             SLOT(onContactChanged()));
     connect(contact.data(),
-            SIGNAL(blockStatusChanged(bool)),
+            SIGNAL(blockStatusChanged(bool,Tp::Channel::GroupMemberChangeDetails)),
             SLOT(onContactChanged()));
 }
 
@@ -51,7 +53,7 @@ RosterItem::~RosterItem()
 
 void RosterItem::onContactChanged()
 {
-    QString status = mContact->presenceStatus();
+    QString status = mContact->presence().status();
     // I've asked to see the contact presence
     if (mContact->subscriptionState() == Contact::PresenceStateAsk) {
         setText(QString(QLatin1String("%1 (%2) (awaiting approval)")).arg(mContact->id()).arg(status));
