@@ -5,8 +5,13 @@
 
 #include <QtTest/QtTest>
 
+#define TP_QT4_ENABLE_LOWLEVEL_API
+
+#include <TelepathyQt4/ChannelFactory>
 #include <TelepathyQt4/Connection>
+#include <TelepathyQt4/ConnectionLowlevel>
 #include <TelepathyQt4/Contact>
+#include <TelepathyQt4/ContactFactory>
 #include <TelepathyQt4/ContactManager>
 #include <TelepathyQt4/PendingContacts>
 #include <TelepathyQt4/PendingReady>
@@ -14,7 +19,7 @@
 
 #include <telepathy-glib/debug.h>
 
-#include <tests/lib/glib/contactlist/conn.h>
+#include <tests/lib/glib/contactlist2/conn.h>
 #include <tests/lib/test.h>
 
 using namespace Tp;
@@ -29,6 +34,9 @@ public:
           mContactsAddedToGroup(0), mContactsRemovedFromGroup(0)
     { }
 
+private:
+    void causeCongestion(const ConnectionPtr &conn, const ContactPtr &contact);
+
 protected Q_SLOTS:
     void onGroupAdded(const QString &group);
     void onGroupRemoved(const QString &group);
@@ -60,29 +68,54 @@ private:
     bool mConnInvalidated;
 };
 
+void TestConnRosterGroups::causeCongestion(const ConnectionPtr &conn, const ContactPtr &contact)
+{
+    // Cause some congestion in the roster events queue so we can check that it doesn't cause
+    // inconsistent event reordering
+    for (int i = 0; i < 5; i++) {
+        QString name = QString(QLatin1String("Rush%1")).arg(i);
+        conn->contactManager()->addGroup(name);
+        conn->contactManager()->addContactsToGroup(name, QList<ContactPtr>() << contact);
+        contact->requestPresenceSubscription();
+        contact->removePresenceSubscription();
+        conn->contactManager()->removeGroup(name);
+    }
+}
+
 void TestConnRosterGroups::onGroupAdded(const QString &group)
 {
+    if (group.startsWith(QLatin1String("Rush"))) {
+        return;
+    }
+
     mGroupAdded = group;
-    mLoop->exit(0);
 }
 
 void TestConnRosterGroups::onGroupRemoved(const QString &group)
 {
-    mGroupRemoved = group;
-    mLoop->exit(0);
-}
+    if (group.startsWith(QLatin1String("Rush"))) {
+        return;
+    }
 
+    mGroupRemoved = group;
+}
 
 void TestConnRosterGroups::onContactAddedToGroup(const QString &group)
 {
+    if (group.startsWith(QLatin1String("Rush"))) {
+        return;
+    }
+
     mContactsAddedToGroup++;
-    mLoop->exit(0);
 }
 
 void TestConnRosterGroups::onContactRemovedFromGroup(const QString &group)
 {
+    if (group.startsWith(QLatin1String("Rush"))) {
+        return;
+    }
+
     mContactsRemovedFromGroup++;
-    mLoop->exit(0);
 }
 
 void TestConnRosterGroups::expectConnInvalidated()
@@ -146,23 +179,26 @@ void TestConnRosterGroups::init()
 
 void TestConnRosterGroups::testRosterGroups()
 {
-    mConn = Connection::create(mConnName, mConnPath);
+    mConn = Connection::create(mConnName, mConnPath,
+            ChannelFactory::create(QDBusConnection::sessionBus()),
+            ContactFactory::create());
 
-    QVERIFY(connect(mConn->requestConnect(),
+    QVERIFY(connect(mConn->lowlevel()->requestConnect(),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
     QCOMPARE(mConn->isReady(), true);
-    QCOMPARE(mConn->status(), Connection::StatusConnected);
+    QCOMPARE(mConn->status(), ConnectionStatusConnected);
 
-    Features features = Features() << Connection::FeatureRoster << Connection::FeatureRosterGroups;
+    Features features = Features() << Connection::FeatureRoster << Connection::FeatureRosterGroups
+        << Connection::FeatureSelfContact;
     QVERIFY(connect(mConn->becomeReady(features),
             SIGNAL(finished(Tp::PendingOperation*)),
             SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
     QCOMPARE(mConn->isReady(features), true);
 
-    ContactManager *contactManager = mConn->contactManager();
+    ContactManagerPtr contactManager = mConn->contactManager();
 
     QStringList expectedGroups;
     expectedGroups << QLatin1String("Cambridge") << QLatin1String("Francophones")
@@ -172,64 +208,23 @@ void TestConnRosterGroups::testRosterGroups()
     groups.sort();
     QCOMPARE(groups, expectedGroups);
 
-    // Cambridge
-    {
-        QStringList expectedContacts;
-        expectedContacts << QLatin1String("geraldine@example.com")
-            << QLatin1String("helen@example.com")
-            << QLatin1String("guillaume@example.com")
-            << QLatin1String("sjoerd@example.com");
-        expectedContacts.sort();
-        QStringList contacts;
-        Q_FOREACH (const ContactPtr &contact, contactManager->groupContacts(QLatin1String("Cambridge"))) {
-            contacts << contact->id();
-        }
-        contacts.sort();
-        QCOMPARE(contacts, expectedContacts);
-    }
-
-    // Francophones
-    {
-        QStringList expectedContacts;
-        expectedContacts << QLatin1String("olivier@example.com")
-            << QLatin1String("geraldine@example.com")
-            << QLatin1String("guillaume@example.com");
-        expectedContacts.sort();
-        QStringList contacts;
-        Q_FOREACH (const ContactPtr &contact, contactManager->groupContacts(QLatin1String("Francophones"))) {
-            contacts << contact->id();
-        }
-        contacts.sort();
-        QCOMPARE(contacts, expectedContacts);
-    }
-
-    // Montreal
-    {
-        QStringList expectedContacts;
-        expectedContacts << QLatin1String("olivier@example.com");
-        expectedContacts.sort();
-        QStringList contacts;
-        Q_FOREACH (const ContactPtr &contact, contactManager->groupContacts(QLatin1String("Montreal"))) {
-            contacts << contact->id();
-        }
-        contacts.sort();
-        QCOMPARE(contacts, expectedContacts);
-    }
-
     QString group(QLatin1String("foo"));
     QVERIFY(contactManager->groupContacts(group).isEmpty());
 
+    causeCongestion(mConn, mConn->selfContact());
+
     // add group foo
-    QVERIFY(connect(contactManager,
+    QVERIFY(connect(contactManager.data(),
                     SIGNAL(groupAdded(const QString&)),
                     SLOT(onGroupAdded(const QString&))));
+
+    causeCongestion(mConn, mConn->selfContact());
+
     QVERIFY(connect(contactManager->addGroup(group),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
-    while (mGroupAdded.isEmpty()) {
-        QCOMPARE(mLoop->exec(), 0);
-    }
+    QVERIFY(!mGroupAdded.isEmpty());
     QCOMPARE(mGroupAdded, group);
 
     expectedGroups << group;
@@ -238,6 +233,8 @@ void TestConnRosterGroups::testRosterGroups()
     groups.sort();
     QCOMPARE(groups, expectedGroups);
 
+    causeCongestion(mConn, mConn->selfContact());
+
     // add Montreal contacts to group foo
     Contacts contacts = contactManager->groupContacts(QLatin1String("Montreal"));
     Q_FOREACH (const ContactPtr &contact, contacts) {
@@ -245,17 +242,20 @@ void TestConnRosterGroups::testRosterGroups()
                         SIGNAL(addedToGroup(const QString&)),
                         SLOT(onContactAddedToGroup(const QString&))));
     }
+
+    causeCongestion(mConn, mConn->selfContact());
+
     QVERIFY(connect(contactManager->addContactsToGroup(group, contacts.toList()),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
-    while (mContactsAddedToGroup != contacts.size()) {
-        QCOMPARE(mLoop->exec(), 0);
-    }
+    QCOMPARE(mContactsAddedToGroup, contacts.size());
     Q_FOREACH (const ContactPtr &contact, contacts) {
         QVERIFY(contact->groups().contains(group));
     }
 
+    causeCongestion(mConn, mConn->selfContact());
+
     // remove all contacts from group foo
     contacts = contactManager->groupContacts(group);
     Q_FOREACH (const ContactPtr &contact, contacts) {
@@ -263,28 +263,32 @@ void TestConnRosterGroups::testRosterGroups()
                         SIGNAL(removedFromGroup(const QString&)),
                         SLOT(onContactRemovedFromGroup(const QString&))));
     }
+
+    causeCongestion(mConn, mConn->selfContact());
+
     QVERIFY(connect(contactManager->removeContactsFromGroup(group, contacts.toList()),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
-    while (mContactsRemovedFromGroup != contacts.size()) {
-        QCOMPARE(mLoop->exec(), 0);
-    }
+    QCOMPARE(mContactsRemovedFromGroup, contacts.size());
     Q_FOREACH (const ContactPtr &contact, contacts) {
         QVERIFY(!contact->groups().contains(group));
     }
 
-    // add group foo
-    QVERIFY(connect(contactManager,
+    causeCongestion(mConn, mConn->selfContact());
+
+    // remove group foo
+    QVERIFY(connect(contactManager.data(),
                     SIGNAL(groupRemoved(const QString&)),
                     SLOT(onGroupRemoved(const QString&))));
+
+    causeCongestion(mConn, mConn->selfContact());
+
     QVERIFY(connect(contactManager->removeGroup(group),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
-    while (mGroupRemoved.isEmpty()) {
-        QCOMPARE(mLoop->exec(), 0);
-    }
+    QVERIFY(!mGroupRemoved.isEmpty());
     QCOMPARE(mGroupRemoved, group);
 
     expectedGroups.removeOne(group);
@@ -304,26 +308,28 @@ void TestConnRosterGroups::testRosterGroups()
  */
 void TestConnRosterGroups::testNotADeathTrap()
 {
-    mConn = Connection::create(mConnName, mConnPath);
+    mConn = Connection::create(mConnName, mConnPath,
+            ChannelFactory::create(QDBusConnection::sessionBus()),
+            ContactFactory::create());
     QCOMPARE(mConn->isReady(), false);
 
     // Check that the contact manager doesn't crash, but returns an error (because the conn isn't
     // ready)
-    QVERIFY(mConn->contactManager() != 0);
+    QVERIFY(!mConn->contactManager().isNull());
     QVERIFY(connect(mConn->contactManager()->contactsForIdentifiers(QStringList()),
                 SIGNAL(finished(Tp::PendingOperation*)),
                 SLOT(expectFailure(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
 
-    QVERIFY(connect(mConn->requestConnect(),
+    QVERIFY(connect(mConn->lowlevel()->requestConnect(),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
     QCOMPARE(mConn->isReady(), true);
-    QCOMPARE(mConn->status(), Connection::StatusConnected);
+    QCOMPARE(mConn->status(), ConnectionStatusConnected);
 
     // As the conn is now ready, the contact building functions shouldn't return an error now
-    QVERIFY(mConn->contactManager() != 0);
+    QVERIFY(!mConn->contactManager().isNull());
 
     QVERIFY(connect(mConn->contactManager()->contactsForIdentifiers(QStringList()),
                 SIGNAL(finished(Tp::PendingOperation*)),
@@ -336,7 +342,7 @@ void TestConnRosterGroups::testNotADeathTrap()
     QCOMPARE(mLoop->exec(), 0);
 
     QVERIFY(connect(mConn->contactManager()->upgradeContacts(QList<ContactPtr>(),
-                    QSet<Contact::Feature>()),
+                    Features()),
                 SIGNAL(finished(Tp::PendingOperation*)),
                 SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
@@ -378,11 +384,6 @@ void TestConnRosterGroups::testNotADeathTrap()
                 SLOT(expectFailure(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
 
-    QVERIFY(connect(mConn->contactManager()->blockContacts(QList<ContactPtr>() << mContact, true),
-                SIGNAL(finished(Tp::PendingOperation*)),
-                SLOT(expectFailure(Tp::PendingOperation*))));
-    QCOMPARE(mLoop->exec(), 0);
-
     // Now, make Roster ready
     Features features = Features() << Connection::FeatureRoster;
     QVERIFY(connect(mConn->becomeReady(features),
@@ -391,6 +392,8 @@ void TestConnRosterGroups::testNotADeathTrap()
     QCOMPARE(mLoop->exec(), 0);
     QCOMPARE(mConn->isReady(features), true);
 
+    causeCongestion(mConn, mContact);
+
     // The roster functions should work now
     QVERIFY(connect(mConn->contactManager()->requestPresenceSubscription(
                     QList<ContactPtr>() << mContact,
@@ -401,6 +404,8 @@ void TestConnRosterGroups::testNotADeathTrap()
 
     QVERIFY(mContact->subscriptionState() != Contact::PresenceStateNo);
 
+    causeCongestion(mConn, mContact);
+
     QVERIFY(connect(mConn->contactManager()->removePresenceSubscription(
                     QList<ContactPtr>() << mContact,
                     QLatin1String("Please don't fail")),
@@ -410,6 +415,8 @@ void TestConnRosterGroups::testNotADeathTrap()
 
     QCOMPARE(mContact->subscriptionState(), Contact::PresenceStateNo);
 
+    causeCongestion(mConn, mContact);
+
     QVERIFY(connect(mConn->contactManager()->authorizePresencePublication(
                     QList<ContactPtr>() << mContact,
                     QLatin1String("Please don't fail")),
@@ -417,6 +424,8 @@ void TestConnRosterGroups::testNotADeathTrap()
                 SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
 
+    causeCongestion(mConn, mContact);
+
     QVERIFY(connect(mConn->contactManager()->removePresencePublication(
                     QList<ContactPtr>() << mContact,
                     QLatin1String("Please don't fail")),
@@ -424,8 +433,6 @@ void TestConnRosterGroups::testNotADeathTrap()
                 SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
 
-    // The test CM doesn't support block, so it will never be successful
-
     // ... but still not the RosterGroup ones
     QVERIFY(connect(mConn->contactManager()->addGroup(QLatin1String("Those who failed")),
                 SIGNAL(finished(Tp::PendingOperation*)),
@@ -458,7 +465,7 @@ void TestConnRosterGroups::testNotADeathTrap()
     QCOMPARE(mConn->isReady(features), true);
 
     // Now that Core, Roster and RosterGroups are all ready, everything should work
-    QVERIFY(mConn->contactManager() != 0);
+    QVERIFY(!mConn->contactManager().isNull());
 
     QVERIFY(connect(mConn->contactManager()->contactsForIdentifiers(QStringList()),
                 SIGNAL(finished(Tp::PendingOperation*)),
@@ -471,11 +478,13 @@ void TestConnRosterGroups::testNotADeathTrap()
     QCOMPARE(mLoop->exec(), 0);
 
     QVERIFY(connect(mConn->contactManager()->upgradeContacts(QList<ContactPtr>(),
-                    QSet<Contact::Feature>()),
+                    Features()),
                 SIGNAL(finished(Tp::PendingOperation*)),
                 SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
 
+    causeCongestion(mConn, mContact);
+
     QVERIFY(connect(mConn->contactManager()->requestPresenceSubscription(
                     QList<ContactPtr>() << mContact,
                     QLatin1String("Please don't fail")),
@@ -485,6 +494,8 @@ void TestConnRosterGroups::testNotADeathTrap()
 
     QVERIFY(mContact->subscriptionState() != Contact::PresenceStateNo);
 
+    causeCongestion(mConn, mContact);
+
     QVERIFY(connect(mConn->contactManager()->removePresenceSubscription(
                     QList<ContactPtr>() << mContact,
                     QLatin1String("Please don't fail")),
@@ -508,21 +519,16 @@ void TestConnRosterGroups::testNotADeathTrap()
                 SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
 
-    // The test CM doesn't support block, so it will never be successful
+    causeCongestion(mConn, mContact);
 
     QVERIFY(connect(mConn->contactManager()->addGroup(QLatin1String("My successful entourage")),
                 SIGNAL(finished(Tp::PendingOperation*)),
                 SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
 
-    qDebug() << "waiting for group to be added";
+    QVERIFY(mConn->contactManager()->allKnownGroups().contains(QLatin1String("My successful entourage")));
 
-    // FIXME: Remove this once fd.o #29728 is fixed
-    while (!mConn->contactManager()->allKnownGroups().contains(QLatin1String("My successful entourage"))) {
-        mLoop->processEvents();
-    }
-
-    qDebug() << "group has been added";
+    causeCongestion(mConn, mContact);
 
     QVERIFY(connect(mConn->contactManager()->addContactsToGroup(QLatin1String("My successful entourage"),
                     QList<ContactPtr>() << mContact),
@@ -530,32 +536,44 @@ void TestConnRosterGroups::testNotADeathTrap()
                 SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
 
+    QVERIFY(mConn->contactManager()->
+        groupContacts(QLatin1String("My successful entourage")).contains(mContact));
+
+    causeCongestion(mConn, mContact);
+
     QVERIFY(connect(mConn->contactManager()->removeContactsFromGroup(QLatin1String("My successful entourage"),
                     QList<ContactPtr>() << mContact),
                 SIGNAL(finished(Tp::PendingOperation*)),
                 SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
 
+    QVERIFY(!mConn->contactManager()->
+        groupContacts(QLatin1String("My successful entourage")).contains(mContact));
+
+    causeCongestion(mConn, mContact);
+
     QVERIFY(connect(mConn->contactManager()->removeGroup(QLatin1String("My successful entourage")),
                 SIGNAL(finished(Tp::PendingOperation*)),
                 SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
 
+    QVERIFY(!mConn->contactManager()->allKnownGroups().contains(QLatin1String("My successful entourage")));
+
     // Now, invalidate the connection by disconnecting it
     QVERIFY(connect(mConn.data(),
                 SIGNAL(invalidated(Tp::DBusProxy *,
                         const QString &, const QString &)),
                 SLOT(expectConnInvalidated())));
-    mConn->requestDisconnect();
+    mConn->lowlevel()->requestDisconnect();
 
     // Check that contactManager doesn't go NULL in the process of the connection going invalidated
     do {
-        QVERIFY(mConn->contactManager() != 0);
+        QVERIFY(!mConn->contactManager().isNull());
         mLoop->processEvents();
     } while (!mConnInvalidated);
 
     QVERIFY(!mConn->isValid());
-    QCOMPARE(mConn->status(), Connection::StatusDisconnected);
+    QCOMPARE(mConn->status(), ConnectionStatusDisconnected);
 
     // Now that the conn is invalidated NOTHING should work anymore
     QVERIFY(connect(mConn->contactManager()->contactsForIdentifiers(QStringList()),
@@ -569,7 +587,7 @@ void TestConnRosterGroups::testNotADeathTrap()
     QCOMPARE(mLoop->exec(), 0);
 
     QVERIFY(connect(mConn->contactManager()->upgradeContacts(QList<ContactPtr>(),
-                    QSet<Contact::Feature>()),
+                    Features()),
                 SIGNAL(finished(Tp::PendingOperation*)),
                 SLOT(expectFailure(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
@@ -598,11 +616,6 @@ void TestConnRosterGroups::testNotADeathTrap()
                 SLOT(expectFailure(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
 
-    QVERIFY(connect(mConn->contactManager()->blockContacts(QList<ContactPtr>() << mContact, true),
-                SIGNAL(finished(Tp::PendingOperation*)),
-                SLOT(expectFailure(Tp::PendingOperation*))));
-    QCOMPARE(mLoop->exec(), 0);
-
     QVERIFY(connect(mConn->contactManager()->addGroup(QLatin1String("Future failures")),
                 SIGNAL(finished(Tp::PendingOperation*)),
                 SLOT(expectFailure(Tp::PendingOperation*))));
@@ -628,12 +641,12 @@ void TestConnRosterGroups::testNotADeathTrap()
 
 void TestConnRosterGroups::cleanup()
 {
-    mContact.clear();
+    mContact.reset();
 
     if (mConn) {
-        if (mConn->status() != Connection::StatusDisconnected) {
+        if (mConn->status() != ConnectionStatusDisconnected) {
             // Disconnect and wait for the readiness change
-            QVERIFY(connect(mConn->requestDisconnect(),
+            QVERIFY(connect(mConn->lowlevel()->requestDisconnect(),
                         SIGNAL(finished(Tp::PendingOperation*)),
                         SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
             QCOMPARE(mLoop->exec(), 0);
