@@ -1,6 +1,11 @@
+#define TP_QT4_ENABLE_LOWLEVEL_API
+
+#include <TelepathyQt4/ChannelFactory>
 #include <TelepathyQt4/Connection>
+#include <TelepathyQt4/ConnectionLowlevel>
 #include <TelepathyQt4/Contact>
-#include <TelepathyQt4/ContactLocation>
+#include <TelepathyQt4/ContactFactory>
+#include <TelepathyQt4/LocationInfo>
 #include <TelepathyQt4/ContactManager>
 #include <TelepathyQt4/PendingContacts>
 #include <TelepathyQt4/PendingReady>
@@ -23,8 +28,8 @@ public:
 
 protected Q_SLOTS:
     void expectConnInvalidated();
-    void expectPendingContactsFinished(Tp::PendingOperation *);
-    void onContactLocationUpdated(Tp::ContactLocation *);
+    void expectPendingContactsFinished(Tp::PendingOperation *op);
+    void onLocationInfoUpdated(const Tp::LocationInfo &location);
 
 private Q_SLOTS:
     void initTestCase();
@@ -76,7 +81,7 @@ void TestContactsLocation::expectPendingContactsFinished(PendingOperation *op)
     mLoop->exit(0);
 }
 
-void TestContactsLocation::onContactLocationUpdated(Tp::ContactLocation *location)
+void TestContactsLocation::onLocationInfoUpdated(const Tp::LocationInfo &location)
 {
     Q_UNUSED(location);
     mContactsLocationUpdated++;
@@ -115,16 +120,18 @@ void TestContactsLocation::initTestCase()
     g_free(name);
     g_free(connPath);
 
-    mConn = Connection::create(mConnName, mConnPath);
+    mConn = Connection::create(mConnName, mConnPath,
+            ChannelFactory::create(QDBusConnection::sessionBus()),
+            ContactFactory::create());
     QCOMPARE(mConn->isReady(), false);
 
-    QVERIFY(connect(mConn->requestConnect(),
+    QVERIFY(connect(mConn->lowlevel()->requestConnect(),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
     QCOMPARE(mConn->isReady(), true);
 
-    QCOMPARE(mConn->status(), Connection::StatusConnected);
+    QCOMPARE(mConn->status(), ConnectionStatusConnected);
 
     QVERIFY(mConn->contactManager()->supportedFeatures().contains(Contact::FeatureLocation));
 }
@@ -141,7 +148,7 @@ void TestContactsLocation::testLocation()
         << QLatin1String("bar");
 
     PendingContacts *pending = mConn->contactManager()->contactsForIdentifiers(
-            validIDs, QSet<Contact::Feature>() << Contact::FeatureLocation);
+            validIDs, Features() << Contact::FeatureLocation);
     QVERIFY(connect(pending,
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectPendingContactsFinished(Tp::PendingOperation*))));
@@ -150,16 +157,12 @@ void TestContactsLocation::testLocation()
     for (int i = 0; i < mContacts.size(); i++) {
         ContactPtr contact = mContacts[i];
 
-        QCOMPARE(contact->requestedFeatures(),
-                 QSet<Contact::Feature>() << Contact::FeatureLocation);
-        QCOMPARE(contact->actualFeatures(),
-                 QSet<Contact::Feature>() << Contact::FeatureLocation);
+        QCOMPARE(contact->requestedFeatures().contains(Contact::FeatureLocation), true);
+        QCOMPARE(contact->actualFeatures().contains(Contact::FeatureLocation), true);
 
-        QVERIFY(contact->location() != 0);
-
-        connect(contact.data(),
-                SIGNAL(locationUpdated(Tp::ContactLocation *)),
-                SLOT(onContactLocationUpdated(Tp::ContactLocation *)));
+        QVERIFY(connect(contact.data(),
+                        SIGNAL(locationUpdated(const Tp::LocationInfo &)),
+                        SLOT(onLocationInfoUpdated(const Tp::LocationInfo &))));
     }
 
     GHashTable *location_1 = tp_asv_new(
@@ -190,9 +193,9 @@ void TestContactsLocation::testLocation()
     for (int i = 0; i < mContacts.size(); i++) {
         ContactPtr contact = mContacts[i];
 
-        QCOMPARE(contact->location()->country(),
+        QCOMPARE(contact->location().country(),
                  QLatin1String(tp_asv_get_string(locations[i], "country")));
-        QCOMPARE(contact->location()->latitude(),
+        QCOMPARE(contact->location().latitude(),
                  tp_asv_get_double(locations[i], "lat", NULL));
     }
 
@@ -209,7 +212,7 @@ void TestContactsLocation::cleanupTestCase()
 {
     if (mConn) {
         // Disconnect and wait for invalidated
-        QVERIFY(connect(mConn->requestDisconnect(),
+        QVERIFY(connect(mConn->lowlevel()->requestDisconnect(),
                         SIGNAL(finished(Tp::PendingOperation*)),
                         SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
         QCOMPARE(mLoop->exec(), 0);
